$LARGE
C
C ConMain.for - main program for CONREC	 (contours for gridded data)
C-------------------------------------------------------------------------
C
CConmain -+ GrafInit -------------- Vstats
C         + Initialize ------------ SAM
C         + Intro Screen ---------- SAM
C         + Read System Defaults -- SAM
C         + Get Screen Fields ----+ SAM
C         |                       + New Screen ------+ SAM
C         |                       + Get File Name    + Display Grid Limits - SAM  
C         |                       + Build File Name
C         |                       + Get Field Group   -- (1) 
C         |                       + Generate Metafile -- (2)
C         |                       + View --------------- (3)
C         |
C         + Write System Defaults - SAM
C         + SAM
C
C -- (1) -+ Get Field (SAM)
C         + Perform Process -+ Read Parameters ----- Read Header Data       
C                            + Save Parameters     + Read Grid 
C                            |                     + Load Parameters -- SAM
C                            |                     + SAM
C                            + Read Header Data ---- SAM
C                            + Read Grid ----------+ MPos
C                            |                     + Rank
C                            |                     + SAM
C                            + Reset Levels -------- SAM
C                            + Set Up Levels ------+ SetFmt
C                            |                     + Display Levels -- SAM
C                            |                     + SAM
C                            + AutoCalc ------------ Clgen
C                            + Build File Name     + SetFmt 
C                            + Blank File          + SAM
C                            + Display Grid Limits + SAM 
C                            + SAM
C
C -- (2) -+ SAM
C         + Get AxisType
C         + Set    (NCARSPP)
C         + LabMod (NCARSPP)
C         + Gridal (NCARSPP)
C         + Pwrit  (NCARSPP)
C         + Conrec -------+ Reset
C                         + DashD (NCARDASH)
C                         + Option (NCARSPP)
C                         + StLine ------- DrLine -+ LastD  (NCARDASH) 
C                         + WriteB (NCARSPP)       + FirstD (NCARDASH)
C                         + MinMax                 + VectD  (NCARDASH)
C
C-- (3) -+ HERSHY
C        + McInit ---+ GRAFLIB
C        + SAM       + HERSHY
C        + Get16 ----- Finish ---- GrWait
C        + Four -----+ Get16
C        |           + Plot -----+ GRAFLIB
C        |
C        + Two ------- Plot      + HERSHY
C        + Multi ----+ PlotCh ---+ GRAFLIB
C                    |    	+ HERSHY
C                    + Finish              
C                    |    
C                    + McOption -+ Get16 
C                    |           + Dash ------ Get16 
C                    |           + HERSHY
C                    + Get16     + Color ----- GRAFLIB
C
C Libraries:  SAM      - Screen Access Management
C             NCARSPP  - NCAR System Plot Package
C             NCARDASH - NCAR Dashed Line Routines
C             GRAFLIB  - GRAFLIB Graphics Library
C             HERSHY   - Hershey Character Plotting Utilities
C--------------------------------------------------------------------- 
C
	PROGRAM ConMain

$INCLUDE:'Conrec.dec'

CDEBUG
$INCLUDE:'Screen.str'
	LOGICAL*1 GetYesNo
CDEBUG

	LOGICAL Exists

CDEBUG	INQUIRE	(FILE = 'conrec.scr', EXIST = Exists)
CDEBUG	IF (.NOT. Exists) THEN
CDEBUG	  CALL Clear Screen
CDEBUG	  WRITE (*, 10) 
CDEBUG	  CALL FLASH (20, 2, 'Press any key to continue ... ')
CDEBUG	ELSE
          CALL Grafinit

CDEBUG    CALL Clear Screen
CDEBUG	  Debug Screen = GetYesNo (20, 2, 'Debug Screen  <Y/N>? ')
CDEBUG	  CALL Read Structure ('conrec.scr')
CDEBUG	  CALL Show Screen Storage
	  Debug Screen = .FALSE.

	  CALL Initialize
	  CALL Read System Defaults
	  IF (Intro) CALL Intro Screen
	  CALL Get Screen Fields
	  CALL Write System Defaults
	  CALL Clear Screen
CDEBUG	ENDIF


C 10	FORMAT (' CONREC cannot operate without the file conrec.scr ',
C     +	/,/,	'   This file could not be located in the default   ',
C     +	/,	' directory.  Please copy this file into the current',
C     +	/,	' directory and try again.',//)
	END

C----------------------------------------------------------------------
C Intro Screen - 	displays screen frame/label/fields
C			for the introductory screen

	SUBROUTINE Intro Screen
	CALL Clear Screen
	CALL Set Screen (1)
	CALL Display Frame
	CALL Display Labels
	CALL Locate Cursor (25, 0)
	CALL Flash (20, 2, 'Press any key to begin the program ')
	RETURN
	END

C----------------------------------------------------------------------
C Initialize - initialize CONREC global values
C
	SUBROUTINE Initialize 
	IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'conrec.dec'

        CHARACTER*10    Dummy, Field Name
        INTEGER*4       Position

	Missing  = 1.E31
	Dash Pattern = '---   ---   '
	DO 10 I = 1, 10
 10	   CharSize (I) = Float (I) * .3

 	DO 20 I = 1, 12
	   IF (Dash Pattern (I:I) .EQ. ' ') THEN
	       DashSeg (I:I) = CHAR(39)
	   ELSE
	       DashSeg (I:I) = '$'
	   ENDIF
 20	CONTINUE

	Offset     = .FALSE.	
	Nvar 	= 0
	Ndata	= 0
	Nclevel = 0
	NxGrid	= 0
	NyGrid	= 0
	Xvar	= 1
	Yvar	= 2
	Cvar	= 3

	XaxisType	= 0
	YaxisType	= 0

	ContourMin	= 0.0
	ContourMax	= 0.0
	ContourInc	= 0.0
	OldMax		= Missing
	OldMin		= Missing
	OldInc		= Missing

	XcellSize	= 0.0
	YcellSize	= 0.0 
	DashCutoff	= 0.0
	ScaleFactor	= 1.0

	Xticfmt = '(G10.3)'
	YTicFmt = '(G10.3)'

	SplineTension = 15.0
	CALL Set 4 Floating Point Value ('SPLINETENS',SplineTension,1)
	CALL Set Character Value ('XTICFORMAT', XticFmt)
	CALL Set Character Value ('YTICFORMAT', YticFmt)
						       
	CALL Set Character Value ('DASHPAT   ', Dash Pattern)
	CALL Set 4 Floating Point Value ('DASHCUTOFF', Dashcutoff, 3)

        CALL Get Toggle Value ('CLABSKIP  ',Position, Dummy)
        SkipFactor = Position+1

        CALL Get Toggle Value ('BACKFLAG  ',Position, Dummy)
        BackGround = (Position .EQ. 0)

        CALL Get Toggle Value ('CLABFRST  ', Position, Dummy)
        FirstContour = Position+1 

        CALL Get Toggle Value ('CLABOPT   ', Position, Dummy)
        Clabopt = Position

        CALL Get Toggle Value ('CLABSIZE  ', Position, Dummy) 
        SizeL = Position+1 

        CALL Get Toggle Value ('MINMAXSIZ ', Position, Dummy)
        SizeM = Position+1 

        CALL Get Toggle Value ('XTICSIZE  ', Position, Dummy) 
	XticLsize = Position+1

        CALL Get Toggle Value ('YTICSIZE  ', Position, Dummy) 
	YticLsize = Position+1

        CALL Get Toggle Value ('NDASHREP  ',Position, Dummy)
        NdashRep = Position+1

        CALL Get Toggle Value ('ANOTFLAG  ',Position, Dummy)
        AnotFlag = Position-1

        NmajorX = 1
        CALL Set Integer Value ('NMAJORXTIC', NMajorX)
        NmajorY = 1
        CALL Set Integer Value ('NMAJORYTIC', NmajorY)
        NminorX = 1
        CALL Set Integer Value ('NMINORXTIC', NMinorX)
        NminorY = 1
        CALL Set Integer Value ('NMINORYTIC', NMinorY)

        CALL Get Toggle Value ('MAINJUST  ',Position, Dummy)
	Mainjust = Position-1

        CALL Get Toggle Value ('SUBTJUST  ',Position, Dummy)
	Subtjust = Position-1

        CALL Get Toggle Value ('MAINTSIZE ',Position, Dummy)
 	Tsize (1) = Position - 1
	DO 40 I = 2, 6
	   WRITE (Field Name, '(A8,I1,1X)') 'SUBTSIZE',I-1
 	   CALL Get Toggle Value (Field Name, Position, Dummy)
	   Tsize (I) = Position	   
 40	CONTINUE

	MetaFile = 'Metacode.met'
	CALL Set Character Value ('METAFILE  ', MetaFile)

	RETURN
	END


C----------------------------------------------------------------------
C Get Screen Fields - 	displays screen frame/label/fields
C			gets the screen fields for the current screen

	SUBROUTINE Get Screen Fields 

	IMPLICIT INTEGER*4 (A-Z)
	PARAMETER (OTHER = 0, HERC = 1, CGA = 2, EGA = 3)
 
$INCLUDE:'conrec.dec'

	INTEGER*4 	Main Screen, Back Screen, Con Screen
	INTEGER*4	Fld Index
	CHARACTER*10	Edit Menu, Main Menu, Con Menu, Back Menu
	CHARACTER	Menu Choice*12, Field Name*10, TFile*14
	CHARACTER*65	File Name
	LOGICAL*1	GetYesNo, Exit, Proceed
	LOGICAL		Error, Fexist

	DATA Edit Menu	/'EDITMENU  '/
	DATA Main Menu	/'MAINMENU  '/
	DATA Con Menu	/'CP MENU   '/
	DATA Back Menu	/'BG MENU   '/
	DATA Main Screen /2/
	DATA Back Screen /3/
	DATA Con Screen  /4/

	IF (Data File (1:1) .NE. ' ') THEN
	  Tfile = Data File
	  CALL Build File Name (Tfile,     Data File, '.grd')
	  CALL Build File Name (Data File, IparmFile, '.cpf')
	  OparmFile (1:14) = IparmFile (1:14)	
	ENDIF
	CALL Get File Name (File Name, File Prefix, IparmFile, Fexist)
	IF (.NOT. Fexist) IparmFile = ' '

  5	CALL New Screen (Main Screen)
	CALL Set Character Value ('DATAFILE  ', Data File)
	CALL Set Character Value ('IPARFILE  ', IparmFile)
	CALL Set Character Value ('OPARFILE  ', OparmFile)
 10 	CALL Put Menu (Main Menu)

 20	CALL Get Menu Choice (Main Menu, Menu Choice)	
 	IF      (Menu Choice .EQ. 'Quit        ') THEN
	  Exit = GetYesNo (20, 2, 'Do you really want to QUIT? <Y/N>')
	  IF (Exit) RETURN

	ELSE IF (Menu Choice .EQ. 'Options/Exec') THEN
 25	   CALL Put Menu (Edit Menu)
 30	   CALL Get Menu Choice (Edit Menu, Menu Choice)	
	   IF      (Menu Choice .EQ. 'Quit        ') THEN
	      GOTO 10
		   
	   ELSE IF (Menu Choice .EQ. 'Graph Option') THEN
	      CALL New Screen (Back Screen)
 	      CALL Put Menu (Back Menu)
 40	      CALL Get Menu Choice (Back Menu, Menu Choice)	
	      IF      (Menu Choice .EQ. 'Quit        ') THEN
	         CALL New Screen (Main Screen)
	         GOTO 25		
	      ELSE
 	         CALL Get Field	Group
	         GOTO 40 
	      ENDIF

	   ELSE IF (Menu Choice .EQ. 'Contour Opti') THEN
 45	      CALL New Screen (Con Screen)
  	      CALL Put Menu (Con Menu)
 50	      CALL Get Menu Choice (Con Menu, Menu Choice)	
	      IF      (Menu Choice .EQ. 'Quit        ') THEN
	         CALL New Screen (Main Screen)
	         GOTO 25		
	      ELSE
	         CALL Get Field Group 
	         GOTO 50 
	      ENDIF

	   ELSE IF (Menu Choice .EQ. 'Execute     ') THEN
	      CALL Get Field Group
	      CALL Erase (20, 2, 75)
	      Field Name = 'METAFILE  '
	      CALL Get Character Value (Field Name, MetaFile)
	      CALL Left Justify (MetaFile)
	      CALL Set Character Value (Field Name, MetaFile)
	      IF (Metafile .EQ. '              ') THEN
	         CALL Flash (20, 2,
     + 	    'Blank Metafile name - operation aborted (press a key)')
	         CALL Set Character Value (Field Name, 'MetaCode.met')
	         GOTO 25
	      ENDIF
	      CALL Get File Name (File Name, File Prefix,
     +			          Meta File, Fexist)
	      IF (Fexist .AND. Metafile(1:12) .NE. 'Metacode.met') THEN
	         Proceed = GetYesNo (20, 2, 
     +		 'Specified file exists. Overwrite? <Y/N> ')
	         IF (.NOT. Proceed) GOTO 25
	      ENDIF
	      CALL Generate MetaFile (File Name, Error)
	      IF (.NOT. Error) THEN
	         IF (DevCode .EQ. OTHER) THEN
	            CALL Beep
	            CALL Flash (20, 2,
     +  'No compatible graphics hardware found! (press any key)') 
	         ELSE
	            CALL View (File Name, Error, DevCode)
	         ENDIF
	      ENDIF
	      CALL New Screen (MAIN SCREEN)
	      GOTO 25
	   ELSE
 	      CALL Get Field Group
	      GOTO 30
	   ENDIF

	ELSE  
 	   CALL Get Field Group
         GOTO 20
	ENDIF
	GOTO 20
	END	

C----------------------------------------------------------------------
C Get Field Group - get a group of fields, perform processes

	SUBROUTINE Get Field Group 
	IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

$INCLUDE:'screen.str'

 10	CALL Get Field	
	CALL Perform Process (SC Field Name (Current Field))
	IF (SC Field Type (Next Field) .NE. Main Menu Field .AND.
     +	    SC Field Type (Next Field) .NE. Menu Field) GOTO 10
	RETURN
	END

C----------------------------------------------------------------------
C New Screen (Screen number) - sets up a new screen
C
	SUBROUTINE New Screen (Screen Number)
	IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'screen.str'
$INCLUDE:'conrec.dec'

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

	CALL Clear Screen
	CALL Set Screen (Screen Number)
	CALL Display Frame
	CALL Display Labels

	DO 10 I = Begin Screen (Current Screen),
     +	         End Screen (Current Screen)
	  IF (SC Field Type (I) .NE. Main Menu Field .AND.
     +	      SC Field Type (I) .NE. Menu Field)
     +	      CALL Write Field (i, SC Attribute (i))

  10	CONTINUE

	CALL Erase (20, 2, 75)
	IF (NxGrid .NE. 0 .AND. NyGrid .NE. 0 .AND. Current Screen .NE. 1) 
     +	   CALL Display Grid Limits (Current Screen)

	RETURN
	END

C----------------------------------------------------------------------
C Read System Defaults

	SUBROUTINE Read System Defaults
	IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE:'conrec.dec'

	CHARACTER*20  String

	OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'OLD',
     +			 ERR = 999)
	READ (10, '(A)', ERR = 999) File Prefix
	READ (10, '(A)', ERR = 999) Data File
	READ (10, '(A)', ERR = 999) String
	Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')
	CALL Set Character Value ('FILEPREFIX', File Prefix)
	CLOSE (10)
	RETURN

 999	CLOSE (10)
	Intro = .TRUE.
	RETURN
	END

C----------------------------------------------------------------------
C Write System Defaults

	SUBROUTINE Write System Defaults
	IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE:'conrec.dec'
	CHARACTER*14	TFile

	OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +			ERR = 999)
	WRITE (10, '(A)', ERR = 999) File Prefix
	CALL Build File Name (Data File, TFile, '.dat')
	WRITE (10, '(A)', ERR = 999) TFile
	IF (Intro) THEN
	  WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
	ELSE
	  WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
	ENDIF
 999	CLOSE (10)
	RETURN
	END

C---------------------------------------------------------------------
C Display Grid limits - put out grid limits for the given screen

	SUBROUTINE Display Grid Limits (Screen)
	IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'conrec.dec'

	INTEGER*4	Screen, Length, Att
	CHARACTER*11	Cminx, Cmaxx, Cminy, Cmaxy
	CHARACTER*11	CincX, CincY, CnX, CnY
	CHARACTER	Ifmt*5, Rfmt*7

	DATA Ifmt /'(I11)'/
	DATA Rfmt /'(F11.3)'/
	DATA Length, Att  /11, 0/

	WRITE (CnX   , Ifmt) NxGrid
	WRITE (CnY   , Ifmt) NyGrid

	IF (Offset) THEN
	   WRITE (CminX , Rfmt) Xmin - (Xcellsize*.5)
	   WRITE (CmaxX , Rfmt) Xmax + (Xcellsize*.5)
	   WRITE (CminY , Rfmt) Ymin - (Ycellsize*.5)
	   WRITE (CmaxY , Rfmt) Ymax + (Ycellsize*.5)
	ELSE
	   WRITE (CminX , Rfmt) Xmin
	   WRITE (CmaxX , Rfmt) Xmax
	   WRITE (CminY , Rfmt) Ymin
	   WRITE (CmaxY , Rfmt) Ymax
	ENDIF
	WRITE (CincX , Rfmt) XcellSize
	WRITE (CincY , Rfmt) YcellSize

	IF (Screen .EQ. 2) THEN
	   CALL Put Str ( CnX,   Length, 15, 50, Att)
	   CALL Put Str ( CminX, Length, 16, 50, Att)
	   CALL Put Str ( CmaxX, Length, 17, 50, Att)
	   CALL Put Str ( CincX, Length, 18, 50, Att)
	   CALL Put Str ( CnY,   Length, 15, 66, Att)
	   CALL Put Str ( CminY, Length, 16, 66, Att)
	   CALL Put Str ( CmaxY, Length, 17, 66, Att)
	   CALL Put Str ( CincY, Length, 18, 66, Att)
	ELSEIF (Screen .EQ. 3) THEN
	   CALL Put Str (  CnX,   Length, 4, 55, Att)
	   CALL Put Str (  CminX, Length, 6, 55, Att)
	   CALL Put Str (  CmaxX, Length, 7, 55, Att)
	   CALL Put Str (  CincX, Length, 8, 55, Att)
	   CALL Put Str (  CnY,   Length, 4, 67, Att)
	   CALL Put Str (  CminY, Length, 6, 67, Att)
	   CALL Put Str (  CmaxY, Length, 7, 67, Att)
	   CALL Put Str (  CincY, Length, 8, 67, Att)
	ENDIF
	RETURN
	END

C----------------------------------------------------------------------
C Show screen storage
	SUBROUTINE Show Screen Storage

$INCLUDE:'Screen.str'

	CALL Clear Screen
	K = Total Number of Screens
	J = End Label (K)
	write (*,*) 'Screen Storage Requirements '
	write (*,*)
	write (*,*) '# Screens  :',K                 
	write (*,*) '# Fields   :',End Screen (K)
	write (*,*) '# Labels   :',End Label  (K)
	write (*,*) '# Frames   :',End Frame  (K)
	write (*,*) 'String size:', LB Pointer (J) + LB Length (J) - 1
	write (*,*)
	write (*,*) 'These values must be smaller that those declared'
	write (*,*) 'in Screen.str '
	CALL Flash (20, 2, 'press a key')
	RETURN
	END

C----------------------------------------------------------------------
C ***	Setfmt - return format and width for axis labels

	SUBROUTINE Setfmt (Max, Fmt, Width)
	REAL*4		Max, Dx
	CHARACTER 	Fmt *(*), Scratch*15
	INTEGER*4  	Width, Ndig, MaxDig

	Ndig = 1
        MaxDig  = Width - 3
	Dx 	= Max + 0.05 / (10 ** MaxDig)

	WRITE (Fmt, 100) 15, MaxDig
	WRITE (Scratch, Fmt) Dx - INT (Dx)

	K = 15

	J = Index (Scratch, '.')
  10	IF (Scratch (K:K) .EQ. '0' .AND. K .GT. J) THEN
	  K = K - 1
	 GOTO 10
	ENDIF
	Ndig = K - J

c ******* 4/16/90  The following if then block was added and the following
c *******          write statement is commented.
c	WRITE (Fmt, 100) Width, Ndig
	IF ( ndig .ne. 0 ) THEN
	  WRITE (Fmt, 100) Width, ndig - 5
	ELSE
	  WRITE (Fmt, 100) Width, ndig 
	ENDIF

	WRITE (Scratch, Fmt, ERR = 999) Max

  20	IF (Scratch (1:1) .EQ. '*') THEN
 999	  IF (Width .GE. 5) THEN
  	    WRITE (Fmt, 200) Width, Width-5
          ELSE
  	    WRITE (Fmt, 200) Width, 0
          ENDIF
	ENDIF 
	
 100	FORMAT ('(F',I2,'.',I2,')')
 200	FORMAT ('(E',I2,'.',I2,')')

	RETURN
	END


C----------------------------------------------------------------------
C ***	neatax - calculation of 'neat' axis parameters

	SUBROUTINE Neatax (Min, Max, Tinc, Ntic)
	REAL*4          T, Min, Max, Amin, Amax
        REAL*4          Tinc, Range, Tmin, Tmax, Tic
	INTEGER*4       Ntic, I, J 
        
        Amin = Min
        Amax = Max

        IF (Amin .GT. Amax) THEN
            T   = AMax
            AMax = AMin
            AMin = T
        ENDIF  
            
	IF (Amax .EQ. 0.0 .AND. Amin .EQ. 0) THEN
           Amin = 0.0
           Amax = 1.0
	     Tinc = 1.0
	     Ntic = 1
	     RETURN
	ENDIF

        IF (Amin .EQ. Amax) Amax = Amin + 1.0
            
C calculate tic mark spacing
        Range = (Amax - Amin)
	Tinc = Tic (Amin, Amax)	 
        IF (Tinc .EQ. 0.0) THEN
           Tinc = 1.0 
        ENDIF  
 2      Ntic = INT (Range / Tinc + 0.5) + 1 
	IF (Ntic .GT. 10) THEN
	   Tinc = Tinc * 2.0
	   GOTO 2
	ENDIF	

C adjust minimum value to nearest tic mark

  5     Tmin = 0.0
        IF (Amin .GE. 0.0) THEN
 10        IF (Tmin + Tinc .LT. Amin) THEN
               Tmin = Tmin + Tinc       
               GOTO 10
           END IF
        ELSE
 20        IF (Tmin .GT. Amin) THEN
               Tmin = Tmin - Tinc
               GOTO  20
           ENDIF
        ENDIF
        Amin = Tmin

C adjust maximum value to nearest ticmark
        Tmax = 0.0
        IF (Amax .GE. 0.0) THEN
 30        IF (Tmax .LT. Amax) THEN
               Tmax = Tmax + Tinc       
               GOTO 30
           END IF
        ELSE
 40        IF (Tmax - Tinc .GT. Amax) THEN
               Tmax = Tmax - Tinc
               GOTO  40
           ENDIF
        ENDIF
        Amax = Tmax

        Range = Amax - Amin
        Ntic = INT (Range / Tinc + 0.5) + 1 

	RETURN
	END

C-------------------------------------------------------------------
C ***	Tic (min,max) - calculates ticmark intervals
	
	REAL*4 FUNCTION Tic (Min, Max)
	implicit real*4 (a-z)
	real*4 min, max
	
      DATA log2  /.39794/
	DATA log5  /.69897/
	DATA log7  /.87506/
	
	tlog = log10(max - min)
	frac = tlog - int(tlog)
	if(tlog .lt. 0.0) then
	  if(frac .ne. 0.0) frac = frac + 1.0
	  tlog = int (tlog)
	  if(tlog .eq. 0.0) tlog = -1.0
	endif

	if(frac .le. log2) then
	  f = 1.0
	elseif(frac .gt. log2 .and. frac .lt. log5) then
	  f = 2.5
	elseif(frac .ge. log5 .and. frac .le. log7) then
	  f = 5.0
	elseif(frac .gt. log7) then
	  f = 7.5
	endif

	ttic = 10 ** (int(tlog) - 1.0) * f

      if(ttic .eq. 0.0) then
	  write(6,*) 'error in tic (tic=0) tlog, frac, f=',tlog,frac,f
	  stop
	endif
	tic = ttic 
	return
	end	

