$LARGE
C Generate.for - subroutine which calls all NCAR subroutines
C   		 to generate the graphics metafile.
C  The NCAR System Plot Package is used to do the graph background,
C and CONREC is used to generate the contour lines and labels. 
C----------------------------------------------------------------------

	SUBROUTINE Generate Metafile (File Name, Error)
	IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'conrec.dec'
$INCLUDE: 'Screen.str'

	INTEGER*4	Nset, 	ISize (10),	JSize
	INTEGER*4	IX,	IY,	IJust, 	Fld Index
	INTEGER*1	ISTRING (65), 	LenStr

	REAL*4		XWmin, 	XWmax, 	YWmin, 	YWmax
	REAL*4		XDmin, 	XDmax, 	YDmin, 	YDmax
	REAL*4		X1,    	X2,	Y1,	Y2  
	REAL*4		Dx, 	Dy, 	Ratio
	REAL*4		Xwidth,	YWidth,	Tx,	Ty
	REAL*4		THeight

	CHARACTER*4	Fmt, 	Fmt1, 	Fmt2, 	File Name*65

	LOGICAL		Error

	Data 	Fmt1 /'(I1)'/, 	Fmt2 /'(I2)'/
	Data 	X1 /.2/, 	Y1 /.10/
	Data 	Xwidth /.7/, 	Ywidth /.7/
	Data 	Nset /1/

C Check for errors
C	
	IF (NxGrid .LE. 0 .OR. NYGrid .LE. 0) THEN
	   CALL PutStr ('The program has detected that the',33,11,3,0)
	   CALL PutStr ('  number of grid cells for the X ',33,12,3,0)
	   CALL PutStr ('  or Y directions is zero.       ',33,13,3,0)
	   CALL PutStr ('A grid must be formed by giving a',33,15,3,0)
	   CALL PutStr ('  data file name, and selecting  ',33,16,3,0)
	   CALL PutStr ('coordinate and contour variables.',33,17,3,0)
	   CALL Flash (20, 2,'ERROR - No grid formed  (press any key)')
	   Error = .TRUE.
	   GOTO 555
	ELSE IF (NcLevel .LE. 0) THEN
	   CALL PutStr ('The program has detected that the',33,11,3,0)
	   CALL PutStr ('  number of contour levels set is',33,12,3,0)
	   CALL PutStr ('  zero.                          ',33,13,3,0)
	   CALL PutStr ('Set the contour levels from the  ',33,15,3,0)
	   CALL PutStr ('Contour options screen, before   ',33,16,3,0)
	   CALL PutStr ('viewing the graph.               ',33,17,3,0)
	   CALL Flash (20, 2,
     +     'ERROR - No contour levels set (press any key)')
	   Error = .TRUE.
	   GOTO 555
	ENDIF
	
C
C Initialize
C
	DO 5 I = 1, 10
 5	   ISize (I) = 5 + (I*2)

	IF (Offset) THEN
	   XWmin = Xmin - (0.5 * XcellSize)
	   XWmax = Xmax + (0.5 * XcellSize)
	   YWmin = Ymin - (0.5 * XcellSize)
	   YWmax = Ymax + (0.5 * XcellSize)

	   XDmin = 0.5
	   XDmax = NxGrid + XDmin
	   YDmin = 0.5
	   YDmax = NyGrid + YDmin
	ELSE
	   XWmin = Xmin
	   XWmax = Xmax
	   YWmin = Ymin
	   YWmax = Ymax

	   XDmin = 1
	   XDmax = NxGrid 
	   YDmin = 1
	   YDmax = NyGrid
	ENDIF	

	CALL Get AxisType (XaxisType, YaxisType, AxisType)
C
C Calculate scaling ratio (to keep true X/Y proportional scale)
C
	Dx 	= XWmax - XWmin
	Dy 	= YWmax - YWmin
	Ratio	= Dy/Dx
	IF (Ratio .LT. 1.0) THEN
	   X2 = X1 + Xwidth
	   Y2 = Y1 + (Xwidth * Ratio)
	ELSE 
	   Y2 = Y1 + Ywidth
	   X2 = X1 + (Ywidth / Ratio)	   
	ENDIF
C
C Open Metacode file
C
        OPEN (UNIT = 8, FILE = File Name, FORM = 'BINARY', RECL=1440,
     +		ERR=999)
 
	CALL PutStr (' Building Graphics MetaFile ... ',32,20,2,2)

	IF (.NOT. BackGround) GOTO 500
C
C Set Up ticmark label formats and sizes
C
	DecPos = Index (Xticfmt, '.')
	IF (DecPos .EQ. 5) THEN
	   Fmt = Fmt2
	ELSE
	   Fmt = Fmt1
	ENDIF
	READ (Xticfmt (3:DecPos-1), Fmt) NumX

	DecPos = Index (Yticfmt, '.')
	IF (DecPos .EQ. 5) THEN
	   Fmt = Fmt2
	ELSE
	   Fmt = Fmt1
	ENDIF
	READ (Yticfmt (3:DecPos-1), Fmt) NumY

C *** Set Up scaling for axes and numeric tic labels
C The coordinate system is set up with the origin offset by half
C the grid cell dimensions for each direction.  The coordinate space
C is the input grid coordinate space (expanded by .5 cellsize on all
C sides.
C
	CALL Set (X1, X2, Y1, Y2, XWmin, XWmax, YWmin, YWmax, 1)
C
C Call Axes Label modifications routine
C
	CALL LabMod (XticFmt, YTicFmt, NumX, NumY,
     +               ISize (XticLSize), ISize (YticLsize), 0, 0, 0) 	
C 
C Call Axis Plotting Routine
C
	CALL Gridal (NmajorX, NminorX, NmajorY, NMinorY,
     +               1, 1, AxisType, Xmin, Ymin)

C 
C *** Set Up scaling for axes labels and titles
C the whole plotting area is used, with coordinate space 1. by 1.
C
	CALL Set (0., 1., 0., 1., 0., 1., 0., 1., 1)
C 
C Place the Titles
C
	Ty = Y2 + .015
	DO 10 I = 4, 1, -1
	   CALL Left Justify (Title (I))
	   N = Lenstr (Title (I))
	   IF (Title (I) (1:1) .NE. ' ' .AND. N. GT. 0) THEN
	       IF (I .EQ. 1) THEN
	          Ijust = MainJust
	       ELSE
	          Ijust = SubTjust
	       ENDIF
	       IF (IJust .EQ. -1) THEN
	         Tx = X1 
	       ELSEIF (IJust .EQ. 0) THEN
	         Tx = X1 + (X2 - X1) / 2.0
	       ELSEIF (IJust .EQ. 1) THEN
	         Tx = X2
	       ELSE
	         IJust = 0
	         Tx = X1 + (X2 - X1) / 2.0
	       ENDIF
	       Jsize = Isize (Tsize (I))
	       CALL PWRIT (Tx, Ty, Title (I), N, Jsize, 0, Ijust)
	       Ty = Ty + Jsize / 1024.0 + .025 * (Y2 - Y1)
	   ENDIF
 10	CONTINUE  
C
C Label the X and Y Axes
C
	CALL Left Justify (Title (5))
	N 	= Lenstr (Title (5))
	Tx 	= X1 + (X2 - X1) / 2.0
	Ty 	= .01
	Jsize	= Isize (Tsize (5))
	CALL PWRIT (Tx, Ty, Title (5), N, Jsize, 0, 0)

	CALL Left Justify (Title (6))
	N 	= Lenstr (Title (6))
	Tx  	= .025
	Ty  	= Y1 + (Y2 - Y1) / 2.0
	Jsize	= Isize (Tsize (6))
	CALL PWRIT (Tx, Ty, Title (6), N, Jsize, 90, 0)

C
C Set up scaling for CONREC use. (each cell is 1x1 'real' units)
C The Origin is offset by 1/2 cell width.  The coordinate space is
C scaled to the indices of the grid array (NxGrid by NyGrid), expanded
C by half a cellsize (.5) on all sides.
C
 500	CONTINUE
	CALL Set (X1, X2, Y1, Y2, XDmin, XDmax, YDmin, YDmax, 1)

C
C Call CONREC to plot the contour lines
C
	CALL CONREC (Zcontour, MAXGRID, NxGrid, NyGrid,
     +               ContourMin, ContourMax, ContourInc,
     +               Nset, AnotFlag, DashSeg, 
     +		     Clabel, Clevel, Nclevel,
     +               DashCutoff, SkipFactor, FirstContour,
     +		     ClabOpt, CharSize (SizeL), CharSize (SizeM),
     +		     NdashRep, SplineTension)

	CLOSE (8) 
	CALL Erase (20, 2, 32)
	Error = .FALSE.
	RETURN

 999	CLOSE (8)
	Error = .TRUE.
	CALL PutStr ('  ERROR creating graphics file   ',33,11,3,2)
	CALL PutStr ('An ERROR was encountered trying  ',33,13,3,0)
	CALL PutStr ('  to create the graphics file.   ',33,14,3,0)
	CALL PutStr ('The disk may be full, or the     ',33,15,3,0)
	CALL PutStr ('  floppy drive (if used) door    ',33,16,3,0)
	CALL PutStr ('  may not be secured.            ',33,17,3,0)
        CALL PutStr ('Check the disk drive, try again. ',33,18,3,0)
	CALL Flash (20, 2,
     + 'ERROR - cannot create graphics metafile ... (press a key)  ') 	
 555	DO 444 I = 11, 18
 444	CALL Erase (I, 3, 33)
	RETURN
	END	

C---------------------------------------------------------------------
C Get AxisType - decodes the (full=1, half=2, grid=3) x and y axistype
C                codes into a single (NCAR GRIDAL) axistype (IGPH) 
C
	SUBROUTINE Get Axis Type (XaxisType, YaxisType, AxisType)
	INTEGER*4	XaxisType, YAxisType, AxisType

	IF (XaxisType .EQ. 3) THEN
	   IF (YaxisType .EQ. 3)  THEN
	       AxisType = 0
	   ELSE IF (YaxisType .EQ. 1) THEN
	       AxisType = 1
	   ELSE IF (YaxisType .EQ. 2) THEN
	       AxisType = 2
	   ELSE
	       AxisType = 5
	   ENDIF
	ELSE IF (XaxisType .EQ. 1) THEN
	   IF (YaxisType .EQ. 3)  THEN
	       AxisType = 4
	   ELSE IF (YaxisType .EQ. 1) THEN
	       AxisType = 5
	   ELSE IF (YaxisType .EQ. 2) THEN
	       AxisType = 6
	   ELSE
	       AxisType = 5
	   ENDIF
	ELSE IF (XaxisType .EQ. 2) THEN
	   IF (YaxisType .EQ. 3)  THEN
	       AxisType = 8
	   ELSE IF (YaxisType .EQ. 1) THEN
	       AxisType = 9
	   ELSE IF (YaxisType .EQ. 2) THEN
	       AxisType = 10
	   ELSE
	       AxisType = 5
	   ENDIF
	ELSE
	   AxisType = 5
	ENDIF

	RETURN
	END

