$LARGE
C Readdata.for - file input routines for CONREC
C
C----------------------------------------------------------------------
C ***  Read Header Data - get data from input file for program CONREC

	SUBROUTINE Read Header Data  (Error)
	IMPLICIT INTEGER*4 (A-Z)
 	
$INCLUDE: 'conrec.dec'

	LOGICAL		Error, Fexist
	CHARACTER*65	File Name
	INTEGER*4	Fld Index, Field

C Build File name
	CALL Get File Name (File Name, File Prefix,
     +			          Data File, Fexist)
	IF (.NOT. Fexist) GOTO 999

C Open File for input
	CALL PutStr ('Reading Header Data ',20, 20, 2, 2)
	Error 	= .FALSE.
	OPEN (UNIT = 10, FILE = File Name, ERR = 999)

C Read header string into header character variable
	READ(10, '(A)', ERR = 999, END = 999) Header

C Get number of variables in file
	READ(10, *, ERR = 999, END = 999) nvar

	IF (Nvar .LT. 3) THEN 
	   CALL Flash (20, 2, 
     +              'Too few variables in file  (press any key)') 
	   GOTO 999
	ELSEIF (Nvar .GT. MAXVAR) THEN
	   CALL Flash (20, 2, 
     +              'Too many variables in file  (press any key)') 
	   GOTO 999
	ENDIF

C Get variable names for each variable in file
  	DO 20 I = 1,  Nvar
 	  READ (10,'(2A10)', ERR = 999, END = 999)
     +                                Varname (i), Units (i)
 20     CONTINUE        

	CALL Load Toggle Values ('XVARIABLE ',10,Nvar,VarName,Xvar-1)
	CALL Load Toggle Values ('YVARIABLE ',10,Nvar,VarName,Yvar-1)
	CALL Load Toggle Values ('CVARIABLE ',10,Nvar,VarName,Cvar-1)

C display new field values
	CALL Display One Field  ('XVARIABLE ')
	CALL Display One Field  ('YVARIABLE ')
	CALL Display One Field  ('CVARIABLE ')
	Ndata 	= 0
	CLOSE (10)
	RETURN

C error has been found in data of file
 999	Error = .TRUE.
	CALL PutStr ('     ERROR reading data file ... ',33,11,3,2)
	CALL PutStr ('An ERROR was encountered trying  ',33,12,3,0)
	CALL PutStr ('to read the header information in',33,13,3,0)
	CALL PutStr ('the specified file. Be sure that ',33,14,3,0)
	CALL PutStr ('the file has the proper format   ',33,15,3,0)
	CALL PutStr ('for use by this program.         ',33,16,3,0)
        CALL PutStr ('Consult the Users Manual for more',33,17,3,0)
        CALL PutStr ('information about file format.   ',33,18,3,0)
	CLOSE (10)
	RETURN
	END

C----------------------------------------------------------------------
C Read Grid - read file data into grid

	SUBROUTINE Read Grid (Error) 
	IMPLICIT INTEGER*4 (A-Z)
	
$INCLUDE: 'conrec.dec'

	CHARACTER*10 	Field Name, Field Name1, Field Name2
	CHARACTER	File Name*65, Scratch*10
	REAL*4		DataRec (MAXVAR), Tx, Ty
	REAL*4		DeltaX, DeltaY
	INTEGER*4	XPosition, YPosition, MPOS, I, Nrec
	INTEGER*4	Fld Index
	LOGICAL		Error, Fexist

C Initialize Variables
	Ndata  		= 0
	NxGrid 		= 0
	NyGrid 		= 0
	DeltaX 		= 0.0
	DeltaY 		= 0.0

C Build File name
	CALL Get File Name (File Name, File Prefix,
     +			          Data File, Fexist)
	IF (.NOT. Fexist) GOTO 999
C put out message
	CALL PutStr ('Reading Data File, Building Grid ...',36,20,2,2)

C Open File for input
	Error 	= .FALSE.
	OPEN (UNIT = 10, FILE = File Name, ERR = 999)

C Read header string 
	READ(10, '(A)', ERR = 999, END = 999) Scratch

C Get number of variables in file
	READ(10, *, ERR = 999, END = 999) Nvar

	IF (Nvar .LT. 1 .OR. Nvar .GT. MAXVAR) GOTO 999

C Get variable names for each variable in file
  	DO 20 I = 1,  Nvar
 	  READ (10,'(A)', ERR = 999, END = 999) Scratch
 20     CONTINUE        

C Try to Build a Grid
C     first read all X and Y values into their arrays, and
C     determine if the spacing is equal for all X values

	CALL PutStr ('Loading coordinates, sorting...     ',36,20,2,2)
	READ (10, *, END = 900, ERR = 999) (DataRec (i), i = 1, Nvar)	
	Ndata  = 1
	NxGrid = 1
	NyGrid = 1
	X (NxGrid) = DataRec (Xvar)
	Y (NyGrid) = DataRec (Yvar)
	DO 30 Nrec = 1, MAXDATA
	   READ (10, *, END = 900, ERR = 999) (DataRec (i), i=1, Nvar)
	   Ndata = Ndata + 1
           XPosition = MPOS (X, DataRec (Xvar), NxGrid)
	   IF (XPosition .EQ. 0) THEN
	      IF (NxGrid .LT. MAXGRID) THEN
                 NxGrid = NxGrid + 1
                 X (NxGrid) = DataRec (Xvar)
	      ELSE
	         GOTO 800
	      ENDIF
	   ENDIF   	
           YPosition = MPOS (Y, DataRec (Yvar), NyGrid)
	   IF (YPosition .EQ. 0) THEN
	      IF (NyGrid .LT. MAXGRID) THEN
                 NyGrid = NyGrid + 1
                 Y (NyGrid) = DataRec (Yvar)
	      ELSE
	         GOTO 800
	      ENDIF
	   ENDIF   	
 30	CONTINUE
C
C File too large
	READ (10, *, END = 900, ERR = 999) (DataRec (i), i=1, Nvar)
	Error = .TRUE.
	CALL PutStr (' WARNING - TOO MANY DATA        ',32,11,3,2)
	CALL PutStr ('The number of data records read ',32,12,3,0)
	CALL PutStr ('is over the program limit.  The ',32,13,3,0)
	CALL PutStr ('remaining data will not be used ',32,14,3,0)
	CALL PutStr ('by this program.                ',32,15,3,0)
	GOTO 1001

C Grid Too large
 800	Error = .TRUE.
	CALL PutStr (' ERROR - GRID LIMITS EXCEEDED   ',32,11,3,2)
	CALL PutStr ('The number of row/column values ',32,12,3,0)
	CALL PutStr ('is over the program limit. This ',32,13,3,0)
	CALL PutStr ('file may not contain a grid, or ',32,14,3,0)
	CALL PutStr ('the grid is just too large.     ',32,15,3,0)
	GOTO 1001

 900	IF (NxGrid .LE. 1 .OR. NyGrid .LE. 1) GOTO 999

C Rewind and Reposition File
 905	REWIND (10)
	READ(10, '(A)') 	Dummy
	READ (10, *)	 	Nvar
  	DO 40 I = 1,  Nvar
 	  READ (10,'(2A10)') 	Dummy, Dummy
 40     CONTINUE        

C Sort Grid Coordinates into Ascending order
	CALL Qsort1 (X, NxGrid)
	CALL Qsort1 (Y, NyGrid)

C Check if coordinates are equally spaced, get Mins, Maxs
	Xmin 		= X (1)
	Xmax 		= X (NxGrid)
	XcellSize	= X (2) - X (1) 
C	DO 50 I = 2, NxGrid-1
C	   Tx 	= X (I+1) - X (I)
C	   IF (ABS (XcellSize - Tx) .GT. 0.001) GOTO 999
C 50	CONTINUE 

	Ymin 		= Y (1)
	Ymax 		= Y (NyGrid)
	YCellSize	= Y (2) - Y (1) 
C	DO 60 I = 2, NyGrid-1
C	   Ty 	= Y (I+1) - Y (I)
C	   IF (ABS (YCellSize - Ty) .GT. 0.001) GOTO 999
C 60	CONTINUE 
	
C Read file and store contour variable into grid
	CALL PutStr ('Initializing Grid ...               ',36,20,2,2)
	DO 10 I = 1, NxGrid
	   DO 10 J = 1, NyGrid
	      Zcontour (I, J) = Missing
 10	CONTINUE

	CALL PutStr ('Loading grid values from file ...   ',36,20,2,2)
	DO 70 NRec = 1, Ndata
	   READ (10, *, ERR = 999) (DataRec (i), i=1, Nvar)
           XPosition = MPOS (X, DataRec (Xvar), NxGrid)
           YPosition = MPOS (Y, DataRec (Yvar), NyGrid)
	   IF (DataRec (Cvar) .LT. Missing) THEN
	      Zcontour (Xposition, Yposition) = DataRec (Cvar)
	   ENDIF
 70	CONTINUE

 	CLOSE (10)
	CALL PutStr (' A Grid has been built ', 23, 11, 10, 2) 
	RETURN

C error has been found in data, or an error building grid
 999	Error 	= .TRUE.
	CLOSE (10)
	CALL PutStr ('        ERROR - no grid formed  ',32,11,3,2)
	CALL PutStr (' An ERROR was encountered trying',32,12,3,0)
	CALL PutStr ('  to build a grid with the given',32,13,3,0)
	CALL PutStr ('  variables.  Check if the right',32,14,3,0)
	CALL PutStr (' variables were chosen and that ',32,15,3,0)
	CALL PutStr (' the data file contains gridded ',32,16,3,0)
	CALL PutStr (' data.                          ',32,17,3,0)
 1001	NxGrid		= 0
	NyGrid		= 0
	XCellSize 	= 0.0
	YCellSize 	= 0.0
	ContourMin	= 0.0
        ContourMax	= 0.0
        ContourInc	= 0.0
	CALL Erase (20, 2, 75)
	RETURN
	END

C---------------------------------------------------------------------
C    POSITION FUNCTION - RETURNS POSITION OF ELEMENT IN VECTOR
C
C    CALLED BY - SUBROUTINE INPUT
C
C      VARIABLES:    ELEMENT - NUMBER TO SEARCH FOR
C                    VECTOR  - NUMBERS TO SEARCH THRU
C                         N  - SIZE OF VECTOR
C 
      INTEGER*4 FUNCTION MPOS (VECTOR, ELEMENT, N)
      INTEGER*4 	N
      REAL*4 		ELEMENT, VECTOR (N)
      NPOS = 0
      IF (N .LE. 0) RETURN
      DO 10 I = 1,N
           IF (ABS (VECTOR (I) - ELEMENT) .LT. .0001) THEN 
             NPOS = I
             GOTO 20
           ENDIF	     
 10   CONTINUE
 20   MPOS = NPOS
      RETURN
      END

C----------------------------------------------------------------------
C ***	NONRECURSIVE QUICK SORT SUBROUTINE

		SUBROUTINE QSORT1 (A,N)
		IMPLICIT INTEGER*4 (A-Z)
		REAL*4  A, T, T1
		INTEGER*4 N
		DIMENSION LOWER (500), UPPER (500), A  (N)

		M	= 1
		I1	= 1
		J1	= N
		I	= I1
		J	= J1

 10		IF (I .GE. J) GOTO 110
 20		K = I
		I2 = INT ((J + I)/2)
		T = A (I2)
		IF (A (I) .LE. T) GOTO 30
		A (I2) = A (I)
		A (I) = T
		T = A (I2)
 30		L = J
		IF  (A (J) .GE. T) GOTO 40
		A (I2) = A (J)
		A (J) = T
		T = A (I2)
		IF  (A (I) .LE. T) GOTO 40
		A (I2) = A (I)
		A (I) = T
		T = A (I2)
 40		L = L - 1
		IF  (A (L) .GT. T) GOTO 40
		T1 = A (L)
 50		K = K + 1
		IF  (A (K) .LT. T) GOTO 50
		IF  (K .GT. L) GOTO 60
		A (L) = A (K)
		A (K) = T1
		GO TO 40

 60		IF  (L-I .LE. J-K) GOTO 70
		LOWER (M) = I
		UPPER (M) = L
		I = K
		M = M+1
		GOTO 80
 70		LOWER (M) = K
		UPPER (M) = J
		J = L
		M = M+1
 80		IF  (J-I .GE. I1) GOTO 20
		IF  (I .EQ. I1) GOTO 10
		I = I-1
 90		I = I+1
		IF  (I .EQ. J) GOTO 110
		T = A (I+1)
		IF  (A (I) .LE. T) GOTO 90
		K = I
 100		A (K+1) = A (K)
		K = K-1
		IF  (T .LT. A (K)) GOTO 100
		A (K+1) = T
		GOTO 90
 110		M = M - 1
		IF  (M .EQ. 0 ) GOTO 999
		I = LOWER (M)
		J = UPPER (M)
		GO TO 80
 999		RETURN
		END	
