c ***********************************************************************

      subroutine spline(x,y,n)

c     fit and plot a natural cubic spline for the n pairs of points in
c     the x and y arrays.  actually two splines are fit -- the first
c     represents virtual plot units in the x-direction (longitude) as a
c     function of the knot number, while the second represents the same
c     for the y-direction (latitude).  this is done since the resultant
c     plotted curve is not in general a function.

      parameter (maxpts=100)
      real x(maxpts),y(maxpts),zx(maxpts),zy(maxpts)

c     at least two knots must be present to fit the splines.
      if(n.le.1) return

c     set up the interpolation factors for each spline.
      call zspl3(n,x,zx)
      call zspl3(n,y,zy)

c     move to the beginning point of the spline.
      call movea(x(1),y(1))

c     use the splines to interpolate the (x,y) virtual plot units at
c     increments of 1/10 the distance between successive knots, and
c     draw a line between them.
      do 20 i=1,n-1
         do 10 t=0.1,1.0,0.10
            xx=spl3(x,zx,i,t)
            yy=spl3(y,zy,i,t)
            call drawa(xx,yy)
   10    continue
   20 continue
c
      return
      end
    
c ***********************************************************************

      function spl3(knot,z,i,t)

c     evaluate a natural cubic spline at the point t between the
c     knots i and i+1.

      parameter (maxpts=100)
      real knot(maxpts),z(maxpts)
      a=t*(z(i+1)-z(i))/6.0+0.5*z(i)
      b=t*a+knot(i+1)-knot(i)-(2.0*z(i)+z(i+1))/6.0
      spl3=t*b+knot(i)

      return
      end

c ***********************************************************************
    
      subroutine tri(n,c,d,z)

c     calcualate the interpolation factors for a spline by solving the
c     symmetric n by n tridiagonal system of equations set up by the
c     subroutine zspl3.  the array d contains the diagonal elements of
c     the system, and the array c contains the subdiagonal/superdiagonal
c     elements.  the array z is the right-hand side of the tridiagonal
c     system, and on return contains the spline interpolation factors.

      parameter (maxpts=100)
      real c(maxpts),d(maxpts),z(maxpts)
      do 10 i=2,n
         xmult=c(i-1)/d(i-1)
         d(i)=d(i)-xmult*c(i-1)
         z(i)=z(i)-xmult*z(i-1)
   10 continue
      z(n)=z(n)/d(n)
      do 20 i=1,n-1
         z(n-i)=(z(n-i)-c(n-i)*z(n-i+1))/d(n-i)
   20 continue

      return
      end

c ***********************************************************************
    
      subroutine zspl3(n,knot,z)

c     set up a symmetric n by n tridiagonal system of equations which
c     will be solved to calculate the spline interpolation factors.

      parameter (maxpts=100)
      real c(maxpts),d(maxpts),knot(maxpts),z(maxpts)

c     set up the tridiagonal system so the array d contains the diagonal
c     elements, c contains the subdiagonal/superdiagonal elements, and
c     z contains the right-hand side values.
      d(1)=1.0
      c(1)=0.0
      z(1)=0.0
      do 10 i=2,n-1
         d(i)=4.0
         c(i)=1.0
         z(i)=6.0*(knot(i+1)-2.0*knot(i)+knot(i-1))
   10 continue
      d(n)=1.0
      c(n-1)=0.0
      z(n)=0.0

c     solve the tridiagonal system just set up.  on return, the array
c     z will contain the spline interpolation factors.
      call tri(n,c,d,z)

      return
      end


