$LARGE
C*********************************************************************
C*** File Name: HPPLOT.FOR (Source code for the program HPPLOT)
C*********************************************************************
C                      PROGRAM HPPLOT
C
C HPPLOT translates the device independent plotting instructions in
C metacode files into a file of HPGL plotting commands.  The output
C file can be routed to a HP plotter by setting up the serial port
C (COM1) by running the batch file HPSETUP.BAT and then using the
C DOS PRINT command to send the file to the plotter.  The HP plotters
C supported are:
C   
C          HP 7470  - 2 pen plotter
C          HP 7475  - 6 pen plotter
C          HP 7450  - 8 pen plotter
C
C If the HP 7470 is used for plotting the graph will be plotted in 
C two pen colors, the program does not prompt for pen changes. 
C
C The main program performs the following:
C   Reads in the system defaults
C   Sets up the screen
C   Calls Get Screen Fld to display the screens and process the
C    user's requests
C   Writes to the system default file
C
 	PROGRAM  HPPLOT

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'hpplot.cmn'

C *** The commented lines are used when debugging the screen
C *** definition file     
C       INQUIRE (FILE = 'HPPLOT.scr',EXIST = Exists)
C       IF (.NOT. Exists) THEN
C          CALL Clear Screen
C          WRITE (*,20)
C          CALL Flash (20,2,'Press any key to continue ... ')
C       ELSE
           CALL Clear Screen
C          CALL Read Structure ('HPPLOT.scr')
           CALL Read System Defaults
           CALL Set Up Screen
           
           IF (Intro) CALL Intro Screen

           CALL Get Screen Fld
           CALL Write System Defaults
           CALL Clear Screen
C       ENDIF
	 
 20   	FORMAT (' HPPLOT cannot operate without the file HPPLOT.scr ',
     +	/,/,	'   This file could not be located in the default   ',
     +	/,	' directory.  Please copy this file into the current',
     +	/,	' directory and try again.',//)
	 
        END

C************************************************************************
C SUBROUTINE INTRO SCREEN 
C
C  This routine displays the screen frame/labels/fields for the 
C  introductory screen.           
C
        SUBROUTINE Intro Screen

        CALL Clear Screen
        CALL Set Screen(1)
        CALL Display Frame
        CALL Display Labels
        CALL Flash (20, 2, 'Press any key to begin the program ')

        RETURN
        END

C************************************************************************
C SUBROUTINE READ SYSTEM DEFAULTS 
C 
C This routine reads the system defaults file 'GEOEAS.DEF' and 
C initializes the system defaults.  
C
        SUBROUTINE Read System Defaults

        IMPLICIT INTEGER*4 (A-Z)
        
        CHARACTER   String*80

$INCLUDE:'Hpplot.cmn' 

        OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +        ERR= 999)

        READ (10, '(A)', ERR = 999) File Prefix
        READ (10, '(A)', ERR = 999) Data File
        READ (10, '(A)', ERR = 999) String

C *** Intro flag embedded as INTRO=ON, INTRO=anything else
C *** Intro is a global logical
        Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')

        CALL Set Character Value ('PREFIX    ',File Prefix)

        CLOSE (10)
        RETURN
         
 999    CLOSE (10)
        File Prefix = ' '
        Intro = .TRUE.

        RETURN
        END

C************************************************************************
C SUBROUTINE WRITE SYSTEM DEFAULTS 
C
C This routine writes the system defaults to the system default file 
C 'GEOEAS.DEF'.
C
        SUBROUTINE Write System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'HPPLOT.CMN'


        OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +        ERR= 999)

        WRITE (10, '(A)', ERR = 999) File Prefix
        WRITE (10, '(A)', ERR = 999) Data File

        IF (Intro) THEN
           WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
        ELSE
           WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
        ENDIF

 999    CLOSE (10)

        RETURN
        END

C**********************************************************************   
C SUBROUTINE GET NEW SCREEN (SCREEN NUMBER) 
C
C Formal Arguments:
C  SCREEN NUMBER  - Stores the screen number.  
C
C This routine sets the current screen to screen number and then displays
C the frame and labels for that screen. 
C
        SUBROUTINE Get New Screen (Screen Number)
         
        IMPLICIT INTEGER*4 (A-Z)
       
        INTEGER*4   Screen Number        

        CALL Clear Screen
        CALL Set Screen (Screen Number)
        CALL Display Frame
        CALL Display Labels

        RETURN
        END

C*********************************************************************
C INTEGER*4 FUNCTION TOGGLE POSITION (FIELD NAME) 
C 
C Formal Arguments:
C  FIELD NAME  - Stores the toggle field name. 
C
C This routine returns the toggle position for the specified toggle
C field.
C
        INTEGER*4 FUNCTION Toggle Position (Field Name)
       
        IMPLICIT INTEGER*4   (A-Z)

$INCLUDE: 'screen.str'
      
        CHARACTER*(*)   Field Name
        
        f = Fld Index (Field Name)
        Toggle Position = SC Position (f)

        RETURN
        END
 
C*********************************************************************
C SUBROUTINE DISPLAY FLDS 
C
C This routine displays the current screen fields.
C
        SUBROUTINE Display Flds

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'screen.str'

        INTEGER*4   Main Menu Field, Menu Field
  
        PARAMETER (Main Menu Field = 3)
        PARAMETER (Menu Field      = 4)


        DO 10 i = Begin Screen (Current Screen),
     +            End Screen (Current Screen)
            IF (SC Field Type (i) .NE. Main Menu Field .AND.
     +         SC Field Type (i) .NE. Menu Field)
     +        CALL Write Field (i, SC Attribute (i))
10      CONTINUE

        RETURN
        END           

            
