$LARGE
C*********************************************************************
C*** File Name: PLOT.FOR (Source code for the program POSTPLOT)
C*********************************************************************
C SUBROUTINE SET PARAMS
C
C This routine sets the AUTOGRAPH parameters for the graph background
C axes parmeters, titles, and labels. It calls Set Arg to set the
C X and Y axes arguments. 
C

	SUBROUTINE Set Params

	IMPLICIT INTEGER*4    (A-Z)

$INCLUDE: 'postplot.cmn'


C *** Set the general arguments (Background, line/max, label control)

	CALL AGSETI ('BACKGROUND.',Background)
        CALL AGSETI ('LINE/MAXIMUM.',60)
	CALL AGSETI ('LABEL/CONTROL.',2)
        CALL AGSETC ('LINE/END.','')

        CALL AGSETI ('X/NICE.',0)
        CALL AGSETI ('Y/NICE.',0)

C *** Set the Axes Arguments
	
	DO 10 i = 1, 7        
	   CALL Set Arg (Xaxis Args(i), 'X', i)
	   CALL Set Arg (Yaxis Args(i), 'Y', i)
 10     CONTINUE

C *** Set the tickmarks to point outwards
	CALL AGSETF ('LEFT/MA/IN.',  0.)
	CALL AGSETF ('BOTTOM/MA/IN.',0.)
	CALL AGSETF ('TOP/MA/IN.',   0.)
	CALL AGSETF ('RIGHT/MA/IN.', 0.)
	CALL AGSETF ('LEFT/MI/IN.',  0.)
	CALL AGSETF ('BOTTOM/MI/IN.',0.)
	CALL AGSETF ('TOP/MI/IN.',   0.)
	CALL AGSETF ('RIGHT/MI/IN.', 0.)

	CALL AGSETF ('LEFT/MA/OU.',  0.015)
	CALL AGSETF ('BOTTOM/MA/OU.',0.015)
	CALL AGSETF ('TOP/MA/OU.',   0.015)
	CALL AGSETF ('RIGHT/MA/OU.', 0.015)
	CALL AGSETF ('LEFT/MI/OU.',  0.01)
	CALL AGSETF ('BOTTOM/MI/OU.',0.01)
	CALL AGSETF ('TOP/MI/OU.',   0.01)
	CALL AGSETF ('RIGHT/MI/OU.', 0.01)

C *** Set the distance of labels from the bottom tickmarks
	CALL AGSETF ('BOTTOM/OF.',0.035)

C *** Set the minimum number of major tick marks to be placed on the axes
	CALL AGSETF ('AXIS/LEFT/MAJOR/COUNT.',1.)
	CALL AGSETF ('AXIS/BOTTOM/MAJOR/COUNT.',1.)
	CALL AGSETF ('AXIS/TOP/MAJOR/COUNT.',1.)
	CALL AGSETF ('AXIS/RIGHT/MAJOR/COUNT.',1.)

C *** Set the character size for the numeric labels
	CALL AGSETF ('AXIS/LEFT/WIDTH/MANTISSA.',.03)
	CALL AGSETF ('AXIS/BOTTOM/WIDTH/MANTISSA.',.03)

	CALL AGSETF ('AXIS/LEFT/WIDTH/EXPONENT.',.025)
	CALL AGSETF ('AXIS/BOTTOM/WIDTH/EXPONENT.',.025)


C *** Set the axes Scaling

        CALL AGSETI ('X/LOGARITHMIC.',0)
        CALL AGSETI ('Y/LOGARITHMIC.',0)

C *** Set title and axes labels

	CALL AGSETC ('LABEL/NAME.','T')
	CALL AGSETI ('LINE/NUMBER.',50)
	CALL AGSETP ('LINE/TE.',' ',1)
	
	CALL AGSETI ('LINE/NUMBER.',60)
	CALL AGSETP ('LINE/TE.',' ',1)

	CALL AGSETI ('LINE/NUMBER.',70)
	CALL AGSETP ('LINE/TE.',' ',1)

C *** Subtitle 2
	la = Lenstr (STit2)
	CALL AGSETI ('LINE/NUMBER.',80)
	CALL AGSETF ('LINE/CHAR.',.04)
	CALL AGSETC ('LINE/TE.',STit2(1:la))	

C *** Subtitle 1
	la = Lenstr(STit1)
	CALL AGSETI ('LINE/NUMBER.',90)
	CALL AGSETF ('LINE/CHAR.',.045)
	CALL AGSETC ('LINE/TE.',STit1(1:la))
	
C *** Title
	la = Lenstr(Titl)
	CALL AGSETI ('LINE/NUMBER.',100)
	CALL AGSETF ('LINE/CHAR.',.05)
	CALL AGSETC ('LINE/TE.',Titl(1:la))
	
C *** X label
	la = Lenstr(XLab)
	CALL AGSETC ('LABEL/NAME.','B')
	CALL AGSETI ('LINE/NUMBER.',- 100)
	CALL AGSETF ('LINE/CHAR.',.03)
	CALL AGSETC ('LINE/TE.',XLab(1:la))

C *** Y label
	la = Lenstr(YLab)
	CALL AGSETC ('LABEL/NAME.','L')
	CALL AGSETI ('LINE/NUMBER.',100)
	CALL AGSETF ('LINE/CHAR.',.03)
	CALL AGSETC ('LINE/TE.',YLab(1:la))


	RETURN
	END

C************************************************************************
C SUBROUTINE SET ARG (ARG, XY, INDEX) 
C
C Formal Arguments:
C  ARG      -  Indicates which AUTOGRAPH axis parameter to set.
C  XY       -  Axis parameters to set.
C  Index    -  Index number of the argument in the array of 
C              axes parameters.
C
C Local Variable:
C  Acid     -  Stores the argument name as a character.
C  Acval    -  Stores the argument value as a character.
C  Afval    -  Stores the argument value as a real number.
C
C This routined sets the AUTOGRAPH axes parameters that were 
C specified by the user and then sets parameters for the top and bottom
C axes.
C
	SUBROUTINE Set Arg (Arg, XY, Index)

	IMPLICIT INTEGER*4   (A-Z)

$INCLUDE:'postplot.cmn'

        CHARACTER      XY, Fmt*7, Acid*29, Acval*13
	CHARACTER*(*)  Arg

        INTEGER*4       Index 

	REAL*4	        Afval, Type, Base

C *** Determine the length of the argument, the position of the 
C *** comma, and the length of the value
	Larg = Lenstr (Arg)
	Lcma = Lencma (Arg)
	Lval = Larg - Lcma - 1

C *** Extract the argument name and value
	Acid = Arg(5:Lcma-2)
	Acval = Arg(Lcma+1:Larg-1)
        
C *** Set the argument
        IF (Acval .EQ. '1.E36') THEN
	   CALL AGSETF (Acid,1.E36)
           Type = 1.E36
           base = 1.E36
	ELSE
	   WRITE (Fmt, '(A2,I2,A3)') '(F', Lval, '.3)'
	   READ (Acval,Fmt) Afval
	   CALL AGSETF (Acid,Afval)
           Type = Afval
           base = Afval
	ENDIF

       
C ***	Set the top or right axis parameter according to those set for
C ***   the X or Y axis 
        IF (Index .EQ. 3) THEN
           IF (XY .EQ. 'X') THEN
              CALL AGSETF ('AXIS/TOP/MAJOR/TYPE.', type)
           ELSE
              CALL AGSETF ('AXIS/RIGHT/MAJOR/TYPE.', type)
           ENDIF
        ENDIF
      
        IF (Index .EQ. 4) THEN
           IF (XY .EQ. 'X') THEN
	      CALL AGSETF ('AXIS/TOP/MAJOR/BASE.', base)
           ELSE
	      CALL AGSETF ('AXIS/RIGHT/MAJOR/BASE.', base)
           ENDIF
        ENDIF

	RETURN
	END

C************************************************************************
C SUBROUTINE CREATE PLOT 
C
C This routine sets up the graph parameters and plots the graph by 
C using calls to PWRIT, then calls Plot Blegend to plot the legend. 
C

	SUBROUTINE Create Plot
	
	IMPLICIT INTEGER*4   (A-Z)

$INCLUDE: 'postplot.cmn'

	CHARACTER*3  Colors
	CHARACTER*6  Sym

	REAL*4     Dx, Dy, Yoffset, Factor

C *** Compute character size for the symbols and labels
	Issize = 5 + (Ssize*2)
	Ivsize = 5 + (Vsize*2)

C *** Set scaling factor
	Dx = Xmax-Xmin
	Dy = Ymax-Ymin
	Factor = Dy/Dx


C *** Determine offset between symbols and values
C     (The grid size in X is 85% of plotting space .55*1024. = 563.2)
C      This code added 1/4/89 to fix bug in spacing 

	IF (Sym sel .EQ. 'Yes' .AND. Val sel .EQ. 'Yes')
     +	   Yoffset  = ((FLOAT (Ivsize + Issize) * .5) / 563.2) * Dy
     +	              + Dy * .02  

C *** Set the AUTOGRAPH parameter for the grid and window area
	CALL AGSETF ('GRID/LEFT.'  ,.10)
	CALL AGSETF ('GRID/RIGHT.' ,.95)
	CALL AGSETF ('GRID/BOTTOM.',.25)
	CALL AGSETF ('GRID/TOP.'   ,.80)
        CALL AGSETF ('GRID/SHAPE.' ,factor)

	CALL AGSETI ('WINDOW.',1)

C *** Set up the and draw the background
	CALL AGSTUP (Xcor(1),1,0,Ndata,1,Ycor(1),1,0,Ndata,1)
	CALL AGBACK
	CALL Bndary

C *** Plot data values     
	DO 10 i = 1, Ndata
           IF (Sym sel .EQ. 'One') THEN
              Sym = '#032&'
              Colors = 'YEL'

	   ELSE
   
C *** 1st quartile value
              IF (Pval (i) .LE. Cutoff(1)) THEN 
                  Sym = '#032&'
                  Colors = 'BLU'
C *** 2nd quartile value
              ELSE IF ((Pval (i) .GT. Cutoff(1)) .AND. 
     +                 (Pval (i) .LE. Cutoff(2))) THEN 
	         Sym = '#032'''
                 Colors = 'GRE'
C *** 3rd quartile value
	      ELSE IF ((Pval (i) .GT. Cutoff(2)) .AND. 
     +                 (Pval (i) .LE. Cutoff(3))) THEN 
	         Sym = '#032B'
                 Colors = 'YEL'
C *** 4th quartile value
	      ELSE 
	         Sym = '#032?'
                 Colors = 'RED'
	      ENDIF
	   ENDIF

C *** Set color
           CALL OPTN('COLOR',Colors)

C *** Plot symbols and values
	   IF (Sym sel .EQ. 'Yes' .OR. Sym sel .EQ. 'One') THEN
              IF (Val sel .EQ. 'Yes') THEN
                 la = Lenstr(Value(i))
	         CALL PWRIT (Xcor(i),Ycor(i),Sym,5,ISsize,0,0)
  	         CALL PWRIT (Xcor(i),Ycor(i)-Yoffset,Value(i)(1:la),
     +                       la,IVsize,0,0)
	      ELSE
C *** Plot symbols only
	         CALL PWRIT (Xcor(i),Ycor(i),Sym,5,ISsize,0,0)
	      ENDIF
	   ELSE 
C *** plot values only
              la = Lenstr(Value(i))
	      CALL PWRIT (Xcor(i),Ycor(i),Value(i)(1:la),
     +                   la,IVsize,0,0)
	   ENDIF
  10 	CONTINUE


C *** Plot legend  
	CALL Plot Blegend	
	   
C *** Advance frame
	CALL FRAME

	RETURN
	END

C************************************************************************
C SUBROUTINE PLOT BLEGEND
C
C This routine plots a quartile legend at the bottom of the graph.
C
	SUBROUTINE Plot Blegend

	IMPLICIT INTEGER*4   (A-Z)

$INCLUDE: 'postplot.cmn'

	CHARACTER      Color*6, Fmt*8, Sym*7, Label2*7, Quartile*15
        CHARACTER*13   Cutlabel(3), Min, Max
        CHARACTER*13   Label1, Label3, Temp
        REAL*4   Row

        Fmt = '(F13.3)'
	COLOR = 'BLACK '

C *** Set up scaling
	CALL SET (.05,.95,0.,.22,0.,6.,0.,6.,1)

C *** Plot scale factor
        IF (Scal sel .NE. ' 0') THEN
	    CALL OPTN ('COLOR',Color)
            CALL Pwrit (.10,5.,Scaled,35,16,0,-1)
	ENDIF

C *** Set up cut off label	
	IF (Sym sel .EQ. 'Yes') THEN  
 	   WRITE (Min,Fmt) Pmin
           WRITE (Max,Fmt) Pmax

	   DO 5 i = 1,3
	      WRITE (Cutlabel(i),Fmt) Cutoff(i)
5          CONTINUE

C *** Set up labels to plot
           Row = 4.
	   DO 10 i = 1, 4
	      IF (i .EQ. 1) THEN
                 Quartile = '1st Quartile: '
                 Sym = '#032&% '
                 Color = 'BLUE'
                 l1 = Lenstr(Min)
                 Temp = Cutlabel(1)
                 Call Left Justify (Temp)    
                 l3 = Lenstr(Temp)
                 Label1 = Min(1:l1)
                 Label2 = ' #010Q% '
                 Label3 = Temp(1:l3)
             
              ELSE IF (i .EQ. 2) THEN
                 Quartile = '2nd Quartile: '
	         Sym = '#032''% '
                 Color = 'GREEN'
                 l1 = Lenstr(Cutlabel(1))
                 Temp = Cutlabel(2)
                 Call Left Justify (Temp)    
                 l3 = Lenstr(Temp)
                 Label1 = Cutlabel(1)(1:l1)
                 Label2 = ' #010O% '
                 Label3 = Temp(1:l3) 
	     
              ELSE IF (i .EQ. 3) THEN
                 Quartile = '3rd Quartile: '
	         Sym = '#032B% '
                 Color = 'YELLOW'
                 l1 = Lenstr (Cutlabel(2))
                 Temp = Cutlabel(3)
                 Call Left Justify (Temp)    
                 l3 = Lenstr(Temp)
                 Label1 = Cutlabel(2)(1:l1)
                 Label2 = ' #010O% '
                 Label3 = Temp(1:l3)  
            
              ELSE 
                 Quartile = '4th Quartile: '
     	         Sym = '#032?% '
                 Color = 'RED'
                 l1 = Lenstr(Cutlabel(3))
                 Call Left Justify (Max)    
                 l3 = Lenstr(Max)
                 Label1 = Cutlabel(3)(1:l1)
                 Label2 = ' #010O% '
                 Label3 = Max(1:l3)
              ENDIF
	  
C *** Plot legend
              CALL OPTN ('COLOR',Color)
              CALL PWRIT (1.0,row,Quartile,15,15,0,-1)
	      CALL Pwrit (2.0,row,Label1,13,16,0,-1)
              CALL PWRIT (3.2,row,Label2,7,16,0,0)
              CALL PWRIT (3.5,row,Sym,7,16,0,0)     
              CALL PWRIT (3.7,row,' #010Q% ',8,16,0,0)
              CALL PWRIT (3.9,row,Label3,13,16,0,-1)
              Row = Row - 1. 
10	   CONTINUE
     
	ENDIF

	RETURN
	END		  

C***********************************************************************
C SUBROUTINE BNDARY 
C
C This routine draws the frame around the graph.
C
	SUBROUTINE Bndary

	CALL PLOTIT (    0,    0,0)
	CALL PLOTIT (32767,    0,1)
	CALL PLOTIT (32767,32767,1)
	CALL PLOTIT (    0,32767,1)
	CALL PLOTIT (    0,    0,1)

	RETURN
	END

