$LARGE
C*********************************************************************
C*** File Name: PROCESS.FOR (Source code for the program POSTPLOT)
C*********************************************************************
C SUBROUTINE GET SCREEN FLD 
C
C Local Variable:
C  Main Screen    - Stores the screen number for the main screen.
C  Back Screen    - Stores the screen number for the background screen.
C  Main Menu      - Stores the name of the main menu.
C  Back Menu      - Stores the name of the background menu.
C  Menu Choice    - Stores the user's menu selection.
C  Error          - Logical variable that is set to true if an error 
C                   occurs.
C  Exit           - Logical variable that is set to true when the user
C                   want to exit the program. 
C
C  This routine displays the current screen, accesses the  menu option
C  selected by the user and then calls Perform Process to process the 
C  screen fields or calls a routine to process the menu option. 
C
        SUBROUTINE Get Screen Fld
        
        IMPLICIT INTEGER*4 (A-Z)
   
$INCLUDE:'SCREEN.STR'
$INCLUDE:'POSTPLOT.CMN' 

        INTEGER*4      Main Screen, Back Screen
        CHARACTER*10   Main Menu, Back Menu
        CHARACTER*12   Menu Choice
   
        LOGICAL*1      GetYesNo, Exit, Defaults
        LOGICAL*1      Error

        DATA Main Menu  /'MAINMENU  '/
        DATA Back Menu  /'BG MENU   '/
        DATA Main Screen     /2/
        DATA Back Screen     /3/

        Error = .FALSE.

C *** Display Main menu
        CALL Get New Screen (Main Screen)
10      CALL Put Menu (Main Menu)
        CALL Display One Field ('PREFIX    ')

        CALL Get Menu Choice (Main Menu, Menu Choice)

C *** Quit - Main menu
        IF (Menu Choice .EQ. 'Quit        ') THEN
           Exit = GetYesNo (20, 2, 'Do you really want to QUIT? <Y/N> ')
           IF (Exit)  THEN
              CALL Clear Screen
	      RETURN
           ELSE
              GOTO 10
           ENDIF

C *** Graph option menu  
	ELSE IF (Menu Choice .EQ. 'Graph Option') THEN
           IF (Nvar .EQ. 0) THEN
             CALL Beep
             Text = 'ERROR - Data file not specified'
             CALL Flash (20,2,Text)
             GOTO 10
           ELSE

C *** Display Backgound Screen
              CALL Get New Screen (Back Screen)
              CALL Display Flds
 20           CALL Put Menu (Back Menu)
              CALL Get Menu Choice (Back Menu, Menu Choice)

C *** Quit - Graph option menu
              IF (Menu Choice .EQ. 'Quit        ') THEN
                 CALL Get New Screen (Main Screen)
                 CALL Display Flds 
                 CALL Put Str ('10^', 3, 16, 20, 0)
                 GOTO 10

C *** Reset - Graph option menu
              ELSE IF (Menu Choice .EQ. 'Reset          ') THEN
                 Text = 'Reset parameters to default values? <Y/N> '
                 Defaults= GetYesNo (20, 2,Text)
                 IF (Defaults)  THEN
                    CALL Reset Values
                    CALL Get New Screen (Back Screen)
                    CALL Display Flds
                    GOTO 20
	         ELSE
                    GOTO 20
                 ENDIF

C *** All other Graph option
              ELSE
                 CALL Perform Process (Error)  
                 GOTO 20
              ENDIF
           ENDIF
  
C *** Data - Main menu
        ELSE IF (Menu Choice .EQ. 'Data      ') THEN
           CALL Display One Field ('DATA FILE ')
           CALL Perform Process (Error)
           GOTO 10

C *** Prefix - Main menu
        ELSE IF (Menu Choice .EQ. 'Prefix     ') THEN
           CALL Perform Process (Error)
           GOTO 10

C *** Execute - Main menu
        ELSE IF (Menu Choice .EQ. 'Execute   ') THEN
           IF (Nvar .EQ. 0) THEN
              CALL Beep
              Text = 'ERROR - Data file not specified'
              CALL Flash (20,2,Text)
              GOTO 10
           ELSE
              CALL Perform Process (Error)
              IF ( .NOT. Error) THEN
                 CALL View (File Name,Error,DevCode)
                 CALL Get New Screen (Main Screen)
                 CALL Display Flds
                 CALL Put Str ('10^', 3, 16, 20, 0)
                 GOTO 10
              ELSE
                 Error = .FALSE.
                 CALL Set Character Value ('METACODE  ', 
     +                                     'Metacode.met  ')
                 GOTO 10
              ENDIF
           ENDIF
        ELSE         
           IF (Nvar .EQ. 0) THEN
              CALL Beep
              Text = 'ERROR - Data file not specified'
              CALL Flash (20,2,Text)
              GOTO 10
           ELSE
              CALL Perform Process (Error)
              GOTO 10
           ENDIF
        ENDIF
        
	RETURN
        END

C***********************************************************************
C SUBROUTINE PERFORM PROCESS
C
C Local Variables:
C  Field Name     - Stores the screen field name.
C  Fexist         - Logical variable set to true if the file exists.
C  Error          - Logical variable set to true if an error occurs.
C  Overwrite      - Logical variable set to true if the file can be
C                   overwritten.
C
C This routine identifies which screen field has been selected, performs
C any error checking and then processes the screen field.
C
        SUBROUTINE Perform Process (Error) 

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'screen.str'
$INCLUDE:'postplot.cmn'

        CHARACTER     Field Name*10, Blanks*50
	CHARACTER*14    Tdata File
        INTEGER*4     Fld Index
        LOGICAL*1     Fexist, Overwrite, GetYesNo, Error

C *** Initialize flags
	
        Error  = .FALSE.
        Fexist = .FALSE.  
        Overwrite = .TRUE.

C *** Identify fields, check for errors, perform process
        
  5     CALL Get Field
        Field Name = SC Field Name (Current Field)


C *** Set value for prefix (directory name)
	IF (Field Name .EQ. 'PREFIX    ') THEN
	   CALL Get Character Value (Field Name, File Prefix)

C *** open data file, read in header information
        ELSE IF (Field Name .EQ. 'DATA FILE ') THEN
           CALL Initialize
C The following line was added 11/22/89.
           Tdata File = Data File
           CALL Get Character Value (Field Name, Data File)
           IF (File Prefix .NE. Blanks) THEN
    	      la = Lenstr(File Prefix)
	      File Name = File Prefix(1:la)//Data File
           ELSE
	      File Name = Data File
           ENDIF
	   INQUIRE (FILE = File Name,  EXIST = Fexist)
           IF (.NOT. Fexist) THEN
              Text = 'ERROR - data file not found'
              CALL Flash ( 20, 2, Text)
	      ERROR = .TRUE.
              CALL Erase Field (Field Name)
C The following line was added 11/22/89.
              Data File = Tdata File
	   ELSE
              CALL Read Header Data (Error)
           ENDIF
	   IF (.NOT. Error) THEN
              Next Field = Fld Index ('XVARIABLE ')   
              GOTO 5
	   ENDIF
           
C *** Set value for X variable, and default label  
   	ELSE IF (Field Name .EQ. 'XVARIABLE ') THEN 
           CALL Get Field Group
 
           f = Toggle Position (Field Name) + 1
           len1 = LENSTR (Units (f))
           len2 = LENSTR (Varnam (f))
	   IF (len1 .NE. 0) THEN
              Xlab = Varnam (f)(1:len2) //'  ('//
     +			Units (f)(1:len1)//')'
           ELSE
              Xlab = Varnam (f)(1:len2) 
	   ENDIF
           CALL Set Character Value ('X LABEL   ', Xlab)

C *** Set value for Y variable, and default label  
           f = Toggle Position ('YVARIABLE ') + 1
           len1 = LENSTR (Units (f))
           len2 = LENSTR (Varnam (f))
	   IF (len1 .NE. 0) THEN
              Ylab = Varnam (f)(1:len2) //'  ('//
     +    			Units (f)(1:len1)//')'
           ELSE
              Ylab = Varnam (f)(1:len2) 
	   ENDIF
           CALL Set Character Value ('Y LABEL   ', Ylab)

C *** Set value for Post variable 
           f = Toggle Position ('PLOTVAR   ') + 1
           len1 = LENSTR (Varnam (f))
           Titl = 'Postplot of ' // Varnam (f) (1:len1) //  
     +                 ' from data file ' // Data File
           CALL Set Character Value ('TITLE     ', Titl)
           CALL Get XY Data 
	   CALL Erase (20, 2, 75)

C *** Set option values                    
 	ELSE IF ( Field Name .EQ. 'VALUES    ') THEN
           CALL Get Field Group
           f  = Toggle Position ('VALUES    ') 
           f1 = Toggle Position ('SYMBOLS   ')
           IF (f .EQ. 0  .AND. f1 .EQ. 2) THEN
             CALL Set Toggle Position ('SYMBOLS    ', 0)
             CALL Display One Field ('SYMBOLS    ')
           ENDIF
                  
C *** Set graph limits 
 	ELSE IF ( Field Name .EQ. 'MIN X     ') THEN
           CALL Get Field Group

C *** Set tick parameters
 	ELSE IF ( Field Name .EQ. 'TIC BAS X  ') THEN
           CALL Get Field Group

C *** Set titles and labels
 	ELSE IF ( Field Name .EQ. 'TITLE     ') THEN
           CALL Get Field Group

C *** Get Metcode file name and create plot
   	ELSE IF ( Field Name .EQ. 'METACODE  ') THEN
           CALL Get Character Value (Field Name, Metacode File)
	   IF (Metacode File .EQ. '               ') THEN
              Error = .TRUE.
              RETURN
c              CALL Set Character Value(Field Name,'Metacode.met')
c              CALL Display One Field (Field Name)
c              Next Field = Fld Index (Field Name)
c              GOTO 5
           ENDIF
	   IF (File Prefix .NE. Blanks) THEN
  	      la = Lenstr(File Prefix)
	      File Name = File Prefix(1:la)//Metacode File
	   ELSE
	      File Name = Metacode File
	   ENDIF
	   INQUIRE (FILE = File Name, EXIST = Fexist)
           IF (Fexist) THEN 
	      IF (Metacode File .NE. 'Metacode.met  ') THEN
	         Overwrite = GetYesNo (20, 2,
     +                     'File Exists, Overwrite?  <Y/N> ')
	      ENDIF
           ELSE
	      OPEN (UNIT=8,STATUS='NEW',FILE=File Name,
     +             FORM = 'BINARY',RECL=1440)
	   ENDIF
           IF (.NOT. Overwrite) THEN
              Next Field = Fld Index (Field Name)
              Overwrite = .TRUE.
              GOTO 5
           ELSE
	      OPEN (UNIT=8,STATUS='OLD',FILE=File Name,
     + 		     FORM = 'BINARY',RECL=1440)
           ENDIF
           CALL Put Str ( ' Preparing plot ', 15, 20, 2, 2)
           CALL Get Screen Args
           CALL Set Graph Limits
           IF (Val Sel .EQ. 'Yes') CALL Set Value
           CALL Set Params
           CALL Create Plot 
           CLOSE (8)
 	
        ELSE
           CALL Get Field
   	ENDIF

        RETURN
        END



