$LARGE
C*********************************************************************
C*** File Name: READDATA.FOR (Source code for the program POSTPLOT)
C*********************************************************************
C SUBROUTINE READ HEADER DATA (ERROR)
C
C Formal Arguments:
C  ERROR    - Logical variable set to true if an error occurs.
C 
C This routine reads in the header information from the data file
C and then sets the screen fields. If an error occurs will reading
C the file an error message is displayed.
C
        SUBROUTINE Read Header Data (Error)

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'screen.str'
$INCLUDE:'postplot.cmn'
 
	LOGICAL*1  Error

	CALL Putstr ('Reading Header Data ',20,20,2,2)
C *** Open input file
	Nunit = 10
	Stat = 'OLD'

        CALL Open File (Nunit, File Name, Stat, Openerr)
        IF (Openerr .NE. 0) THEN
           Text = 'Problem opening data file'
           CALL Flash (20, 2, Text)
           GOTO 999
        ENDIF

C *** Read header string into default title

	READ (Nunit, '(A)', ERR = 999, END = 999) Text

C *** Get Number of variables in file

   	READ (Nunit, *, ERR= 999, END = 999) Nvar
        IF (Nvar .LT. 3) THEN
           Text = 'Too few variables in file (press any key)'
           CALL Flash ( 20, 2, Text)
           GOTO 999
	ELSE IF (Nvar .GT. Maxvar) THEN
           Text = 'Too many variables in file (press any key)'
           CALL Flash ( 20, 2, Text)
           GOTO 999
        ENDIF
        
C *** Get variable names for each variable in file
	
	DO 20 i = 1, Nvar
           READ (Nunit,'(2A10)', ERR = 999, END = 999) 
     +  		 Varnam(i), Units(i)
  20    CONTINUE

C *** Load variable names into toggle fields 
	
	CALL Load Toggle Values ('XVARIABLE ', 10, Nvar, Varnam, 0)
	CALL Load Toggle Values ('YVARIABLE ', 10, Nvar, Varnam, 1)
	CALL Load Toggle Values ('PLOTVAR   ', 10, Nvar, Varnam, 2)

C *** Display screen fields

	CALL Display Flds
        CALL Put Str ('10^', 3, 16, 20, 0)

	CLOSE (Nunit)
	RETURN
   
  999   Error = .TRUE.
        CLOSE (Nunit)
        CALL Beep
	CALL Putstr ('     ERROR reading data file ... ',33,9,43,2)
	CALL Putstr ('An ERROR was encountered in the  ',33,11,43,0)
	CALL Putstr ('header section of the data file. ',33,12,43,0)
	CALL Putstr ('Check the file name, and that the',33,13,43,0)
	CALL Putstr ('file named has the standard data ',33,14,43,0)
	CALL Putstr ('file format.                     ',33,15,43,0)
	CALL Putstr ('Consult the Users Manual for more',33,16,43,0)
	CALL Putstr ('information about file format.   ',33,17,43,0)
	CALL Erase Field ('XVARIABLE ') 
	CALL Erase Field ('YVARIABLE ')
	CALL Erase Field ('PLOTVAR   ')
        CALL Erase Field ('DATA FILE ')
	CALL Flash (20,2, ' Press and key to exit')
	CALL Erase (20,2,75)
	Ndatafile = 0
	Nvar  = 0
	Ndata = 0
	CALL Erase Message

	RETURN
	END

C***********************************************************************
C SUBROUTINE GET XY DATA
C
C This routine reads in the values for the variables selected and 
C determines the minimum/maximum values.
C
        SUBROUTINE Get XY Data

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'postplot.cmn'


        CHARACTER   Blanks*50, Dummy*75

	REAL*4      Data(Maxvar), Missing, Xtype, Ytype

        CALL Putstr ('Reading Data Values ...  ',25,20,2,2)

C *** Determine postion of variables
        Xpos = Toggle Position ('XVARIABLE ') + 1
        Ypos = Toggle Position ('YVARIABLE ') + 1
        Ppos = Toggle Position ('PLOTVAR   ') + 1
             
C Get File Name
	IF (File Prefix .NE. Blanks) THEN
    	   la = Lenstr(File Prefix)
	   File Name = File Prefix(1:la)//Data File
        ELSE
	   File Name = Data File
	ENDIF
	  
C *** Open input file
	Nunit = 10
	Stat = 'OLD'

        CALL Open File (Nunit, File Name, Stat, Openerr)
        IF (Openerr .NE. 0) THEN
           Text = 'Problem opening data file'
           CALL Flash (20, 2, Text)
           GOTO 999
        ENDIF

C *** Read header string into default title

	READ (Nunit, '(A)', ERR = 999, END = 999) Dummy

C *** Get Number of variables in file

   	READ (Nunit, *, ERR= 999, END = 999) Nvar

C *** Get variable names for each variable in file
	
	DO 20 i = 1, Nvar
           READ (Nunit,'(2A10)', ERR = 999, END = 999) Dummy, Dummy 
  20    CONTINUE


C *** Read variable values
	Ndata = 0
        Ndatafile = 0
	Missing   = 1.E31

	DO 30 j = 1, MaxData
	   READ (Nunit, *,ERR=999,END = 900) (Data (i), i=1,Nvar)
	   IF ((Data (Xpos) .LT. Missing .AND.
     +	        Data (Ypos) .LT. Missing ) .AND.
     +          Data (Ppos) .LT. Missing) THEN
	      Ndatafile = Ndatafile + 1
	      Xarr(Ndatafile) = Data (Xpos)
	      Yarr(Ndatafile) = Data (Ypos)
	      Parr(Ndatafile) = Data (Ppos)
	   ENDIF
 30     CONTINUE

C *** Check if more then MaxData records
        READ (Nunit, *, ERR = 999, END = 900) (Data(i), i=1,Nvar)
	CALL Beep
	CALL Putstr(' The number of records in the data ',35,10,42,2)
	CALL Putstr(' file exceeds the program limits.  ',35,11,42,2)
	CALL Putstr(' Only the maximum number of allowed',35,12,42,0)
	CALL Putstr(' of coordinate values have been    ',35,13,42,0)
	CALL Putstr(' stored in memory, the rest will   ',35,14,42,0)
	CALL Putstr(' not be used...                    ',35,15,42,0)
	CALL Flash (20,2, 'Press any key to continue ')
	CALL Erase Message

 900    CLOSE (Nunit)

C *** Determine min/max values and set screen values
           
	Xmin = Xarr(1)
	Xmax = 0.0
        
	Ymin = Yarr(1)
	Ymax = 0.0
        
	Pmin = Parr(1)
	Pmax = 0.0
	DO 40 k = 1, Ndatafile
           Xmin = AMIN1(Xarr(k),Xmin)
           Xmax = AMAX1(Xarr(k),Xmax)

           Ymin = AMIN1(Yarr(k),Ymin)
           Ymax = AMAX1(Yarr(k),Ymax)

           Pmin = AMIN1(Parr(k),Pmin)
           Pmax = AMAX1(Parr(k),Pmax)
 40     CONTINUE

        CALL Set  4 Floating Point Value ('MIN X     ', Xmin, 3)
        CALL Set  4 Floating Point Value ('MAX X     ', Xmax, 3)

        CALL Set  4 Floating Point Value ('MIN Y     ', Ymin, 3)
        CALL Set  4 Floating Point Value ('MAX Y     ', Ymax, 3)
        

	CALL Display Flds
	
        RETURN

 999    CLOSE (Nunit)
        CALL Beep
	CALL Putstr ('     ERROR reading data file ... ',33,9,43,2)
	CALL Putstr ('An ERROR was encountered while   ',33,11,43,0)
	CALL Putstr ('reading the data file.           ',33,12,43,0)
	CALL Putstr ('Check the file name, and that the',33,13,43,0)
	CALL Putstr ('file named has the standard data ',33,14,43,0)
	CALL Putstr ('file format.                     ',33,15,43,0)
	CALL Putstr ('Consult the Users Manual for more',33,16,43,0)
	CALL Putstr ('information about file format.   ',33,17,43,0)
	CALL Erase Field ('XVARIABLE ') 
	CALL Erase Field ('YVARIABLE ')
	CALL Erase Field ('PLOTVAR   ')
        CALL Erase Field ('DATA FILE ')
	CALL Flash (20,2, ' Press and key to exit')
        CALL Erase (20,2,75)
	Ndatafile = 0
	Nvar  = 0
	Ndata = 0
        CALL Erase Message
	
        RETURN
        END

