$LARGE
c**********************************************************************
c*** File name: QSORT.FOR (Source code for the program TRANS)
c**********************************************************************
c        R a n k  V a r                        (subroutine)
c**********************************************************************

      SUBROUTINE Rank Var(Col)

c*** This subroutine ranks the variable upon which the operation is performed
c*** from low to high.  The resulting variable stores this rank.  To find
c*** the rank of a variable's value refer to the corresponding row of the 
c*** result variable column.  That value will be the rank.
c*** 
c*** FORMAL ARGUMENT:
c*** Col     - Integer identifying the column number of variable in question.
c***
c*** VARIABLES:
c*** Ndata   - Global variable containing the number of sample records.
c*** Tmpdata - Global array containing ranking of samples for one variable.
c*** A1()    - Read array containing the samples of the variable specified by
c***           'Col'.
c*** P()     - an integer array which stores the original positions of 
c***           corresponding sample datum. 
c*** Nvar    - Global variable that stores the current number of variables.
c*** Missing - Global variable that stores the missing value.
c*** MAXVAR  - Global variable that contains the maximum number of variables
c***           permitted.

$INCLUDE:'Trans.dec'

      REAL*4     A1(MAXDATA), Get Element, Subtotal, Average
      INTEGER*4  P(MAXDATA), I, J, N, Countr
      LOGICAL*1  Vmiss, Same Values, EndofData

      N = Ndata
c***  Initialize the arrays.
      DO 10 I = 1, Ndata
        A1(I) = Get Element(1,Col,I,MAXVAR)
	P(I) = I
  10  CONTINUE

c***  Sort the specified variable's data stored in A1 in the ascending
c***  order.  Upon return A1 will contain the ordered sample data, and P 
c***  will contain the original position of that sample datum.  N is 
c***  assigned the value contained in 'Ndata' before this call. 

      CALL Qsort(A1, P, N)

      Countr = 0
c***  Assign ranks to array A1.
      DO 40 I = 1, Ndata
c***    Assign rank values to non-missing data.  Assign missing 
c***    values to the 'missing' data.
	IF (.NOT. (Vmiss(A1(I)))) THEN
	   Countr = Countr + 1
           Tmpdata(P(I)) = Countr
	ELSE
	   Tmpdata(P(I)) = Missing
	ENDIF
  40  CONTINUE

c***  Determine which samples are equal in value and then assign
c***  a revised rank based on the average value of their ranks.

      EndofData   = .FALSE.
      Same Values = .FALSE.
      Countr = 0
      Subtotal = 0.0
      DO 60 I = 1, Ndata - 1
	IF (Vmiss(A1(I))) RETURN
        IF (ABS(A1(I) - A1(I+1)) .LE. .000000001) THEN
          Same Values = .TRUE.
          Subtotal = Subtotal + Tmpdata(P(I))
          Countr = Countr + 1
          IF (I+1 .EQ. Ndata) THEN
            Subtotal = Subtotal + Tmpdata(P(Ndata))
            EndofData   = .TRUE.
          ENDIF
        ELSE
          IF (Same Values) THEN
            Subtotal = Subtotal + Tmpdata(P(I))
            Average = Subtotal/REAL(Countr+1)
            DO 50 J = I, I-Countr, -1
              Tmpdata(P(J)) = Average
  50        CONTINUE
            Countr = 0
            Subtotal = 0.0
            Same Values = .FALSE.
          ENDIF
        ENDIF
  60  CONTINUE

c***  If all variable's samples are equal then assign average.

      IF (Same Values .AND. (Countr .EQ. Ndata-1)) THEN
        Average = Subtotal/REAL(Countr+1)
        DO 70 J = 1, Ndata
          Tmpdata(P(J)) = Average
  70    CONTINUE

c***  Process those samples containing the maximum value.

      ELSEIF (Same Values .AND. EndofData) THEN
        Average = Subtotal/REAL(Countr+1)
        I = Ndata - Countr 
        DO 80 J = Ndata, I, -1
          Tmpdata(P(J)) = Average
  80    CONTINUE
      ENDIF

      RETURN
      END
c**********************************************************************
c    	NONRECURSIVE QUICK SORT SUBROUTINE	 (subroutine)
c**********************************************************************

      SUBROUTINE Qsort(A,P,N)
 
c***  This routine sorts the real array 'A'(which stores sample data)
c***  in ascending order.  The original position of that sample datum 
c***  is preserved in the integer array 'P'.  After 'A' is sorted, 
c***  the original position of the sample datum contained in A(1) is
c***  maintained in P(1).

      IMPLICIT INTEGER*4 (A-Z)

      REAL*4 A, T, T1
      INTEGER*4 N, P(1)
      DIMENSION LOWER(100), UPPER(100), A(N)

		M	= 1
		I1	= 1
		J1	= N
		I	= I1
		J	= J1

 10		IF (I .GE. J) GOTO 110
 20		K = I
		I2 = INT ((J + I)/2)
C-------------------------------
		T = A(I2)
		TP = P(I2)
C-------------------------------
		IF (A(I) .LE. T) GOTO 30
C-------------------------------
		A(I2) = A(I)
		P(I2) = P(I)
		A(I) = T
		P(I) = TP
		T = A(I2)
		TP = P(I2)
C-------------------------------
 30		L = J
		IF (A(J) .GE. T) GOTO 40
C-------------------------------
		A(I2) = A(J)
		P(I2) = P(J)
		A(J) = T
		P(J) = TP
		T = A(I2)
		TP = P(I2)
C-------------------------------
		IF (A(I) .LE. T) GOTO 40
C-------------------------------
		A(I2) = A(I)
		P(I2) = P(I)
		A(I) = T
		P(I) = TP
		T = A(I2)
		TP = P(I2)
C-------------------------------
 40		L = L - 1
		IF (A(L) .GT. T) GOTO 40
C-------------------------------
		T1 = A(L)
		TP1 = P(L)
C-------------------------------
 50		K = K + 1
		IF (A(K) .LT. T) GOTO 50
		IF (K .GT. L) GOTO 60
C-------------------------------
		A(L) = A(K)
		P(L) = P(K)
		A(K) = T1
		P(K) = TP1
C--------------------------------
		GO TO 40

 60		IF (L-I .LE. J-K) GOTO 70
		LOWER(M) = I
		UPPER(M) = L
		I = K
		M = M+1
		GOTO 80
 70		LOWER(M) = K
		UPPER(M) = J
		J = L
		M = M+1
 80		IF (J-I .GE. I1) GOTO 20
		IF (I .EQ. I1) GOTO 10
		I = I-1
 90		I = I+1
		IF (I .EQ. J) GOTO 110
C-------------------------------
		T = A(I+1)
		TP = P(I+1)
C-------------------------------
		IF (A(I) .LE. T) GOTO 90
		K = I
C-------------------------------
 100		A(K+1) = A(K)
		P(K+1) = P(K)
C-------------------------------
		K = K-1
		IF (T .LT. A(K)) GOTO 100
C--------------------------------
		A(K+1) = T
		P(K+1) = TP
C--------------------------------
		GOTO 90
 110		M = M - 1
		IF (M .EQ. 0 ) GOTO 999
		I = LOWER(M)
		J = UPPER(M)
		GO TO 80
 999		RETURN
		END	

c**********************************************************************

