/*
 *	vstats.c -- Program to determine status of various computers. This
 * program returns the various monitor types found on the system, the
 * current video mode, and if the machine is a AT-class system. It
 * also checks for the existance of a second adaptor in the system.
 * The main routine that is used is a modified version of a program
 * found in "Programmer's Guide to PC & PS/2 Video Systems" by Richard
 * Wilton. The program (VideoID) is a assembly language routine for
 * determining the primary and secondary adaptors in the system. 
 */

unsigned char far *mem_at = (char far *)0xf000fffe;	
                                               /* BIOS computer type info */
char far *mem_mode = (char far *)0x00400049;   /* current adaptor mode */
char far *mem_ega  = (char far *)0x00400087;   /* EGA info */

vstats(c_adap,ega_mon,mode,at,sec_mon)

int *c_adap;  /* current adaptor found. May by any of the following */
              /*     0 = other (ie - MDA, Compaq, etc...)           */
              /*     1 = Hercules                                   */
              /*     2 = CGA                                        */
              /*     3 = EGA                                        */
              /*     4 = MCGA     (future use, set to 0)            */
              /*     5 = VGA      (future use, set to 0)            */
              /*     6 = HGC+     (future use     "    )            */
              /*     7 = InColor	 (future use     "    )            */

int *ega_mon; /* monitor type connected to primary adaptor:      	  */
              /* check for EGA monitor only. Values 1-5 will be for */
              /* future expansion.											  */
              /*     0 = colour monitor                             */
              /*     1 = monochrome monitor                         */
	      /*     2 = Color Graphics Card (with EGA)             */
              /* For future use                                     */
              /*     1 = MDA                                        */
              /*     2 = CGA                                        */
              /*     3 = EGA                                        */
              /*     4 = PS/2 Monochrome                            */
              /*     5 = PS/2 Color                                 */

int *at;          /* AT computer flag (0 = non-AT, 1 = AT)              */

int *mode;        /* current video mode (0 - 19)                        */

int *sec_mon;     /* type of second adaptor found in system, if any,    */
                  /* using number system mentioned above. However, if   */
                  /* no second adaptor is found then a 4 is returned.   */

{
	char	*SubsystemName();
	char  *DisplayName();

	static struct {
		char Subsystem;
		char Display;
	}
	VIDstruct[2];

	VideoID(VIDstruct);

	*sec_mon = 4;	/* assume no second adaptor */

	if (VIDstruct[0].Subsystem & 0x80)
	  switch(VIDstruct[0].Subsystem & 0x7f) {
	    case 0:
		 case 1:
		 case 2:
		 		*c_adap = 1;		/* HGC found */
				break;
	  }
	else 
	switch (VIDstruct[0].Subsystem) {
	  	case 1:
            *c_adap = 0;		/* MDA found */
 				break;
		case 2:
				*c_adap = 2;		/* CGA found */
				break;
		case 3:
				*c_adap = 3;		/* EGA found */
				break;
		case 4:
		case 5:
		case 6:
		case 7:
				*c_adap = 0;		/* future expansion */
				break;
	}

	if (*c_adap == 3 && VIDstruct[0].Display == 2)
	  *ega_mon = 2;		/* EGA card with CGA monitor */
	  
	if (VIDstruct[1].Subsystem) {
	if (VIDstruct[1].Subsystem & 0x80)
	switch (VIDstruct[1].Subsystem & 0x7f) {
	   case 0:
		case 1:
		case 2:
		     *sec_mon = 1;      /* HGC found */
			  break;
	 }
    else
	 switch(VIDstruct[1].Subsystem) {
	 case 1:
  	        *sec_mon = 0;		/* MDA found */
           break;
      case 2:
           *sec_mon = 2;		/* CGA found */
           break;
		case 3:
           *sec_mon = 3;		/* EGA found */
           break;
      case 4:
      case 5:
           *sec_mon = 0;		/* special found */
           break;
      }
	}
   *mode = *mem_mode;		/* return current video mode */
   *at = 0;
   if (*mem_at == 0xfc)
     *at = 1;					/* AT-type computer found    */
   if (*c_adap == 3 && VIDstruct[0].Display != 2) 
     *ega_mon = ((*mem_ega & 0x01) >> 1);
	  
}

