$LARGE
C*********************************************************************
C*** File Name: PROCESS.FOR (Source code for the program XYGRAPH)
C*********************************************************************
C SUBROUTINE GET SCREEN FLD
C
C Local Variable:
C  Main Screen    - Stores the screen number for the main screen.
C  Edit Screen    - Stores the screen number for the options screen.
C  Back Screen    - Stores the screen number for the background screen.
C  Main Menu      - Stores the name of the main menu.
C  Edit Menu      - Stores the name of the options menu.
C  Back Menu      - Stores the name of the background menu.
C  Menu Choice    - Stores the user's menu selection.
C  Error          - Logical variable that is set to true if an error 
C                   occurs.
C  Exit           - Logical variable that is set to true when the user
C                   want to exit the program. 
C
C  This routine displays the current screen, accesses the  menu option
C  selected by the user and then calls Perform Process to process 
C  the screen fields or calls a routine to process the menu option. 
C
        SUBROUTINE Get Screen Fld
        
        IMPLICIT INTEGER*4 (A-Z)
   
$INCLUDE: 'SCREEN.STR'
$INCLUDE: 'XYGRAPH.CMN'

        INTEGER*4      Main Screen, Edit Screen, Back Screen
        CHARACTER*10   Main Menu, Edit Menu, Back Menu
        CHARACTER*15   Menu Choice
        LOGICAL*1      GetYesNo, Exit, Defaults, FEXIST, Error

        DATA Edit Menu  /'EDITMENU  '/
        DATA Main Menu  /'MAINMENU  '/
        DATA Back Menu  /'BG MENU   '/
        DATA Main Screen     /2/
        DATA Edit Screen     /3/
        DATA Back Screen     /4/

          
	Error = .FALSE.


C *** Initialize file names
        CALL Build File Name (Data File, Param File, '.xpf') 
        CALL Build File Name (Data File, User Pfile, '.xpf') 
        INQUIRE (FILE = Param File, EXIST = FEXIST)
        IF (FEXIST) CALL Set Character Value ('PARAMETER ',Param File)
        CALL Set Character Value ('OPARAMETER',User Pfile)
        
C *** Display main menu
        CALL Get New Screen (Main Screen)
10      CALL Put Menu (Main Menu)
        CALL Display One Field ('PREFIX    ')

        CALL Get Menu Choice (Main Menu, Menu Choice)

C *** Quit - main menu
        IF (Menu Choice .EQ. 'Quit          ') THEN
           Exit = GetYesNo (20, 2, 'Do you really want to QUIT? <Y/N> ')
           IF (Exit)  THEN
              CALL Clear Screen
              RETURN
           ELSE
              GOTO 10
           ENDIF

C *** Options menu
        ELSE IF (Menu Choice .EQ. 'Options/Execute') THEN
           CALL Get New Screen (Edit Screen)
           CALL Display Flds
20         CALL Put Menu (Edit Menu)
           CALL Get Menu Choice (Edit Menu, Menu Choice)

C *** Quit - Options menu
           IF (Menu Choice .EQ. 'Quit           ') THEN
              CALL Get New Screen (Main Screen)
              CALL Display Flds 
              GOTO 10

C *** Graph options menu
           ELSE IF (Menu Choice .EQ. 'Graph Options  ') THEN
              CALL Get New Screen (Back Screen)
              CALL Display Flds
30            CALL Put Menu (Back Menu)
              CALL Get Menu Choice (Back Menu, Menu Choice)

C *** Quit - Graph options menu
              IF (Menu Choice .EQ. 'Quit           ') THEN
                 CALL Get New Screen (Edit Screen)
                 CALL Display Flds 
                 GOTO 20

C *** Reset - Graph options menu
              ELSE IF (Menu Choice .EQ. 'Reset          ') THEN
                 Text = 'Reset parameters to default values? <Y/N> '
                 Defaults= GetYesNo (20, 2,Text)
                 IF (Defaults)  THEN
                    CALL Reset Values
                    CALL Get New Screen (Back Screen)
                    CALL Display Flds
                    GOTO 30
	         ELSE
                    GOTO 30
                 ENDIF

C *** All other Graph Option options
              ELSE
                  CALL Perform Process (Error)  
                  GOTO 30
              ENDIF

C *** Data - Options menu
           ELSE IF (Menu Choice .EQ. 'Data           ') THEN
              CALL Set Character Value ('DATA FILE ',Data File)
              CALL Display One Field ('DATA FILE ')
              CALL Perform Process (Error)
              GOTO 20

C *** Execute - Options menu
           ELSE IF (Menu Choice .EQ. 'Execute        ') THEN
              IF (Nvar .EQ. 0  .AND. Num Curves .EQ. 0) THEN
                 CALL BEEP
                 Text = 'ERROR - Data file not specified'
                 CALL Flash (20,2,Text)
                 GOTO 10
              ELSE
                 CALL Perform Process (Error)  
                 IF ( .NOT. Error) THEN
                    Text = ' Preparing plot  '
                    l = LENSTR (Text)
                    CALL Put Str (Text(1:l) ,l, 20, 2, 2)
	            CALL XYPLOT
	            CALL Get New Screen (Edit Screen)
	            CALL Display Flds
	            GOTO 20
                 ELSE
                    Error = .FALSE.
                    CALL Set Character Value ('METACODE  ', 
     +                                        'Metacode.met  ')
                    GOTO 20
                 ENDIF
              ENDIF

C *** All other Options menu options
           ELSE
              CALL Perform Process (Error)  
              GOTO 20
           ENDIF

C *** Read Parameters - Main menu
        ELSE IF (Menu Choice .EQ. 'Read Parameters') THEN
           CALL Perform Process (Error) 
           IF (.NOT. RPError) THEN
              CALL Get New Screen (Edit Screen)
              CALL Display Flds
              GOTO 20
           ELSE
              GOTO 10
           ENDIF

C *** All other Main menu options
        ELSE         
           CALL Perform Process (Error)  
           GOTO 10
        ENDIF

        RETURN
        END

C***********************************************************************
C SUBROUTINE PERFORM PROCESS 
C
C Local Variables:
C  Field Name     - Stores the screen field name.
C  Fexist         - Logical variable set to true if the file exists.
C  Error          - Logical variable set to true if an error occurs.
C  Overwrite      - Logical variable set to true if the file can be
C                   overwritten.
C  Set Screen     - Logical variable set to true if the screen fields
C                   are to be set.
C
C This routine identifies which screen field has been selected, performs
C any error checking and then processes the screen field.
C
        SUBROUTINE Perform Process (Error)

        IMPLICIT INTEGER*4 (A-Z)		

$INCLUDE: 'screen.str'
$INCLUDE: 'XYGRAPH.CMN'

        CHARACTER*10  Field Name
        CHARACTER*14  Tdata File
        CHARACTER*50  Blanks
       
	INTEGER*4     Fld Index

        LOGICAL*1     Fexist, Overwrite, GetYesNo, Error, Set Screen

        Overwrite = .TRUE.

 5      CALL Get Field
        Field Name = SC Field Name (Current Field)
       
C *** Identify fields, perform process

C *** enter file prefix (directory name)
	IF (Field Name .EQ. 'PREFIX    ') THEN
	   CALL Get Character Value (Field Name, File Prefix) 
           CALL Left Justify (File Prefix)
	   CALL Set Character Value (Field Name, File Prefix)

C *** read parameter file, and read in the data for parameters set
        ELSE IF (Field Name .EQ. 'PARAMETER ') THEN
           CALL Get Character Value (Field Name, Param File)
	   IF (File Prefix .NE. Blanks) THEN
   	      ls = Lenstr(File Prefix)
 	      File Name = File Prefix(1:ls)//Param File
           ELSE
	      File Name = Param File
	   ENDIF
	   INQUIRE (FILE = File Name, EXIST = FEXIST)
	   IF (.NOT. FEXIST) THEN
	      Text = 'ERROR - parameter file not found'
              RPError = .TRUE.
	      CALL Flash (20, 2, Text)
              CALL Erase Field (Field Name)
           ELSE
	      Nunit = 11
              Stat = 'OLD'
              CALL Open File (Nunit, File Name, Stat, Openerr)
              IF (Openerr .NE. 0) THEN
                 Text = 'Problem opening parameter file'
                 CALL Flash (20, 2, Text)
                 CALL Erase Field (Field Name)
                 RPError = .TRUE.
	         RETURN
              ENDIF
              CALL Read Param File (Error)
              IF (.NOT. Error) THEN
C *** Added following line on 11/20/89
                 Tdata File = Data File
                 CALL Get Character Value ('DATA FILE ', Data File)
	         IF (File Prefix .NE. Blanks) THEN
	            ls = Lenstr(File Prefix)
	            File Name = File Prefix(1:ls)//Data File
	         ELSE
	            File Name = Data File
	         ENDIF
	         INQUIRE (FILE = File Name, EXIST = FEXIST)
	         IF (.NOT. Fexist) THEN
	            Text = 'ERROR - data file not found'
	            CALL Flash (20,2,Text)
                    CALL Erase Field ('DATA FILE')
C *** Added following line on 11/20/89
                    Data File = Tdata File
	         ELSE 
                    Error = .FALSE.
                    Set Screen = .FALSE.
	            CALL Read Header Data (Error, Set Screen)
	            IF (.NOT. Error) CALL Get Data (Set Screen) 
                    CALL Build File Name (Param File, User PFile,
     +                                    '.xpf')
                    CALL Set Character Value ('OPARAMETER', User Pfile)
	            CALL Erase (20,2,75)
                 ENDIF
              ELSE
                 RPError = .TRUE.
	      ENDIF
           ENDIF

C *** read in data
        ELSE IF (Field Name .EQ. 'DATA FILE ') THEN
C *** Added following line on 11/20/89
           Tdata File = Data File
	   CALL Get Character Value (Field Name, Data File)
	   IF (File Prefix .NE. Blanks) THEN
 	      ls = Lenstr(File Prefix)
	      File Name = File Prefix(1:ls)//Data File
	   ELSE
	      File Name = Data File
	   ENDIF
	   INQUIRE (FILE = File Name, EXIST = FEXIST)
	   IF (.NOT. Fexist) THEN
	      Text = 'ERROR - data file not found'
	      CALL Flash (20,2,Text)
              Error = .TRUE.
              CALL Erase Field (Field Name)
C *** Added following line on 11/20/89
              Data File = Tdata File
           ELSE
              ERROR = .FALSE.
              Set Screen = .TRUE.
              Num Curves  = 1
              CALL Reset Values
              CALL Read Header Data (Error,Set Screen)
	      CALL Erase (20,2,75)
	   ENDIF
           IF (.NOT. Error) THEN
              CALL Build File Name (Data File, User PFile, '.xpf')
              CALL Set Character Value ('OPARAMETER', User Pfile)
cccc              Next Field = Fld Index ('XVAR      ')
cccc              GOTO 5
           ENDIF

C *** determine the X and Y variables
        ELSE IF (Field Name .EQ. 'XVAR       ') THEN
           CALL Get Field Group
           f = Toggle Position ('XVAR       ') + 1 
           len1 = Lenstr(Vunit (f))
           len2 = Lenstr(Vname (f))
	   IF (len1 .NE. 0) THEN
              Xlab = Vname (f)(1:len2) //'  ('//
     +	             Vunit (f)(1:len1)//')'
           ELSE
              Xlab = Vname (f)(1:len2) 
	   ENDIF
           CALL Set Character Value ('X LABEL   ', Xlab)

           f = Toggle Position ('Y VAR C1  ')
           len1 = Lenstr (Vunit (f))
           len2 = Lenstr (Vname (f))
	   IF (len1 .NE. 0) THEN
              Ylab = Vname (f)(1:len2) //'  ('//
     +	             Vunit (f)(1:len1)//')'
           ELSE
              Ylab = Vname (f)(1:len2) 
	   ENDIF
           CALL Set Character Value ('Y LABEL   ', Ylab)
           Set Screen = .TRUE.
	   CALL Get Data (Set Screen)
	   CALL Erase (20,2,75)

C *** select the line type, symbol, and color for curves
        ELSE IF (Field Name .EQ. 'STYPE C1   ') THEN
	   CALL Get Field Group

C *** create the parameter file    
        ELSE IF (Field Name .EQ. 'OPARAMETER') THEN
           CALL Get Character Value (Field Name, User Pfile)
           IF (User Pfile .EQ. Blanks(1:14) .OR. 
     +         User Pfile .EQ. '              ') RETURN
           IF (File Prefix .NE. Blanks) THEN
              ls = Lenstr(File Prefix)
              File Name = File Prefix(1:ls)//User Pfile
   	   ELSE
	      File Name = User Pfile
	   ENDIF
	   INQUIRE (File = File Name, EXIST = Fexist)
           IF (Fexist) THEN
	      Text = 'File Exists, Overwrite?  <Y/N) '
	      Overwrite = GetYesNo(20,2,Text)
	   ENDIF
	   IF (.NOT. Overwrite) THEN
	      Next Field = Fld Index (Field Name)
	      Overwrite = .TRUE.
	      GOTO 5
	   ELSE
              CALL Write Param File                
	   ENDIF

C *** get metacode file name and create the parameter file. 
        ELSE IF (Field Name .EQ. 'METACODE  ' ) THEN
	   CALL Get Character Value (Field Name, Metacode File)
	   IF (Metacode File .EQ. '               ') THEN
              Error = .TRUE.
              RETURN
c              CALL Set Character Value (Field Name, 'Metacode.met')
c              CALL Display One Field (Field Name)
c              Next Field = Fld Index (Field Name)
c              GOTO 5
           ENDIF
           IF (File Prefix .NE. Blanks) THEN
	      ls = Lenstr(File Prefix)
              File Name = File Prefix(1:ls)//Metacode File
	   ELSE
	      File Name = Metacode File
	   ENDIF
	   INQUIRE (File = File Name, EXIST = Fexist)
	   IF (Fexist) THEN
              IF (Metacode File .NE. 'Metacode.met') THEN		   
	         Text = 'File Exists, Overwrite?  <Y/N) '
	         Overwrite = GetYesNo(20,2,Text)
	      ENDIF
	   ENDIF
	   IF (.NOT. Overwrite) THEN
	      Next Field = Fld Index (Field Name)
              Overwrite = .TRUE.
              GOTO 5
	   ELSE
	      CALL Write Param File
	   ENDIF

C *** set the min/max and scale
        ELSE IF (Field Name .EQ. 'MIN X     ') THEN 
	   CALL Get Field Group
           Xpos  = Toggle Position ('SCALE X   ')
           Ypos  = Toggle Position ('SCALE Y   ')
             
           IF (Xpos .EQ. 1 .AND. .NOT. Xlog) THEN
              Text = 'ERROR - X coordinate data  zero'
              CALL Flash (20,2,Text)
              CALL Erase (20,2,75)
              CALL Set Toggle Position ('SCALE X   ',0)
              CALL Display One Field ('SCALE X   ')
           ENDIF 
           IF (Scaly .EQ. 1) THEN 
              DO 20  i= 1,Num Curves
                 IF (.NOT. Ylog(i)) THEN 
                    Text = 'ERROR - Y coordinate data  zero'
         	    CALL Flash (20,2,Text)
                    CALL Erase (20,2,75)
                    CALL Set Toggle Position ('SCALE Y   ',0)
                    CALL Display One Field ('SCALE Y   ')
                 ENDIF
  20          CONTINUE
           ENDIF

C *** set the tick parameters       
        ELSE IF (Field Name .EQ. 'TIC BAS X ') THEN
  	   CALL Get Field Group

C *** set the title and labels     
        ELSE IF (Field Name .EQ. 'TITLE     ') THEN
	   CALL Get Field Group
        ENDIF

        RETURN
        END

            

