$LARGE
C*********************************************************************
C*** File Name: XYGRAPH.FOR (Source code for the program XYGRAPH)
C*********************************************************************
C                      PROGRAM XYGRAPH
C
C This program produces line and/or scatter plots for upt to 6 
C variables in a Geo-EAS data file.  plot of up to 6 dependent  
C with one independent variable can be obtained.  Up to 6 colors, 
C symbols, and line types may be used to identify the data.
C Options allow a regression line to be calculated and control
C of axes parameters and titles.  A file called a 'metacode' file
C is created for redisplay or to produce a hardcopy.
C
C The main program performs the following:
C   Reads in the system defaults
C   Initializes program variables
C   Sets up the screen
C   Calls Get Screen Fld to display the screens and process the
C    user's requests
C   Writes to the system default file
C
C**************************************************************************
C Interface  statements for video status function

	interface to integer*2 function vstats[C]
     1		(c_adap[reference],ega_mon[reference],mode[reference],
     1		 at[reference],sec_mon[reference])
	integer*2 c_adap,ega_mon,mode,at,sec_mon
	end
C**************************************************************************

        PROGRAM XYGRAPH

$INCLUDE:'xygraph.cmn'
c$INCLUDE:'screen.str'

      	LOGICAL Exists

C ***  The commented lines are used when debugging the screen 
C ***  definition file.
c      	INQUIRE	(FILE = 'xygraph.scr', EXIST = Exists)
c      	IF (.NOT. Exists) THEN
c	   CALL Clear Screen
c	   WRITE (*, 10) 
c	   CALL FLASH (20, 2, 'Press any key to continue ... ')
c       ELSE
           CALL Clear Screen
           CALL Grafinit

c           Debug Screen = .TRUE.
c           CALL Read Structure ('xygraph.scr')
c           CALL Clear Screen
c          k = tOTAL NUMBER OF SCREENS
c          WRITE (*, *) '# fields: ',End Screen (K)
c          WRITE (*, *) '# labels: ',End Label  (K)
c          K = End Label  (Total number of Screens)
c          WRITE (*, *) 'Pointer: ',LB Pointer (K)+LBLength(K)-1
c          WRITE (*, *)
c          CALL Flash (20, 2, '*****')
 
           CALL Read System Defaults
           CALL Initialize
           CALL Set Up Screen
           IF (Intro) CALL Intro Screen
           CALL Get Screen Fld
           CALL Write System Defaults
           CALL Clear Screen
c       ENDIF


 10   	FORMAT (' XYGRAPH cannot operate without the file xygraph.scr ',
     +	/,/,	'   This file could not be located in the default   ',
     +	/,	' directory.  Please copy this file into the current',
     +	/,	' directory and try again.',//)
	 
        END

C**************************************************************************
C SUBROUTINE GRAFINIT 
C
C  This routine calls Vstats to initialize graphics parameters and
C  sets the  global variable Devcode to the type of graphics device
C  in use.
C
	SUBROUTINE GrafInit 
	
        IMPLICIT INTEGER*4 (A-Z)
	
        PARAMETER (OTHER = 0, HERC = 1, CGA = 2, EGA = 3)

$INCLUDE: 'xygraph.cmn'

	INTEGER*2     Vstats, Adaptor, EgaMon, Mode
	INTEGER*2     ATflag, Mon2, Rcode

C Get device type , and graphics mode
        RCode = Vstats (Adaptor, Ega Mon, Mode, Atflag, Mon2)
	IF (Adaptor .EQ. CGA) THEN
           Devcode = CGA
        ELSE IF (Adaptor .EQ. EGA) THEN
	   Devcode = EGA
        ELSE IF (Adaptor .EQ. HERC) THEN
	   Devcode = HERC
        ELSE
           Devcode = OTHER
	ENDIF

	RETURN
	END

C************************************************************************
C SUBROUTINE INTRO SCREEN 
C
C  This routine displays the screen frame/labels/fields for the 
C  introductory screen.           
C
        SUBROUTINE Intro Screen

        CALL Clear Screen
        CALL Set Screen(1)
        CALL Display Frame
        CALL Display Labels
        CALL Flash (20, 2, 'Press any key to begin the program ')

        RETURN
        END

C************************************************************************
C SUBROUTINE READ SYSTEM DEFAULTS 
C 
C This routine reads the system defaults file 'GEOEAS.DEF' and  
C initializes the system defaults.
C
        SUBROUTINE Read System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'XYGRAPH.CMN'

        CHARACTER   String*80

        OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +        ERR= 999)

        READ (10, '(A)', ERR = 999) File Prefix
        READ (10, '(A)', ERR = 999) Data File
        READ (10, '(A)', ERR = 999) String

C *** Intro flag embedded as INTRO=ON, INTRO=anything else
C *** Intro is a global logical
        Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')

        CALL Set Character Value ('PREFIX    ',File Prefix)

        CLOSE (10)
        RETURN
         
 999    CLOSE (10)
        File Prefix = ' '
        Intro = .TRUE.

        RETURN
        END

C************************************************************************
C SUBROUTINE WRITE SYSTEM DEFAULTS 
C
C This routine writes the system defaults to the system default file 
C 'GEOEAS.DEF'.
C
        SUBROUTINE Write System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'XYGRAPH.CMN'


        OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +        ERR= 999)

        WRITE (10, '(A)', ERR = 999) File Prefix
        WRITE (10, '(A)', ERR = 999) Data File

        IF (Intro) THEN
           WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
        ELSE
           WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
        ENDIF

 999    CLOSE (10)

        RETURN
        END

C************************************************************************
C SUBROUTINE INITIALIZE 
C
C This routine initializes global varaibles to their default values and
C then calls Reset Values to set the screen fields.
C
        SUBROUTINE Initialize
        
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'XYGRAPH.CMN'
       
	DO 20  i = 1, Maxvar
           Var Args (i,2) = 'I (''DASH/ADDRESS.'',LINARR(1))'
           Var Args (i,3) = 'I (''DASH/NUMBER.'',6)'
           Var Args (i,4) = 'I (''DASH/INDEX.'',1)'
           Var Args (i,5) = 'S (XARRAY,YARRAY,''0445'',1,0,0,0)'
           Var Args (i,6) = 'O (''COLOR'',''BLACK'')'
   20   CONTINUE 
        
        Gen Args(1) = 'I (''BACKGROUND.'',3)'
        Gen Args(2) = 'I (''LINE/MAXIMUM.'',60)'
 	Gen Args(3) = 'I (''LABEL/CONTROL.'',2)'

        Xaxis Args (1) = 'F (''X/MINIMUM.'',0.)'
        Xaxis Args (2) = 'F (''X/MAXIMUM.'',0.)'
        Xaxis Args (3) = 'F (''AXIS/BOTTOM/MAJOR/TYPE.'',1.E36)'
        Xaxis Args (4) = 'F (''AXIS/BOTTOM/MAJOR/BASE.'',1.E36)'
        Xaxis Args (5) = 'F (''AXIS/BOTTOM/NUMERIC/TYPE.'',1.E36)'
        Xaxis Args (6) = 'F (''AXIS/BOTTOM/NUMERIC/EXPONENT.'',1.E36)'
        Xaxis Args (7) = 'F (''AXIS/BOTTOM/NUMERIC/FRACTION.'',1.E36)'
        Xaxis Args (8) = 'I (''X/LOGARITHMIC.'',0)'
        
	Yaxis Args (1) = 'F (''Y/MINIMUM.'',0.)'
        Yaxis Args (2) = 'F (''Y/MAXIMUM.'',0.)'
        Yaxis Args (3) = 'F (''AXIS/LEFT/MAJOR/TYPE.'',1.E36)'
        Yaxis Args (4) = 'F (''AXIS/LEFT/MAJOR/BASE.'',1.E36)'
        Yaxis Args (5) = 'F (''AXIS/LEFT/NUMERIC/TYPE.'',1.E36)'
        Yaxis Args (6) = 'F (''AXIS/LEFT/NUMERIC/EXPONENT.'',1.E36)'
        Yaxis Args (7) = 'F (''AXIS/LEFT/NUMERIC/FRACTION.'',1.E36)'
        Yaxis Args (8) = 'I (''Y/LOGARITHMIC.'',0)'
        
        Lablin Args (1,1) = 'C (''LABEL/NAME.'',''T'')'
        Lablin Args (1,2) = 'I (''LINE/NUMBER.'',100)'
        Lablin Args (1,3) = 'C (''LINE/TE.'','' $'')'  
    
        Lablin Args (2,1) = 'C (''LABEL/NAME.'',''T'')'
        Lablin Args (2,2) = 'I (''LINE/NUMBER.'',90)'
        Lablin Args (2,3) = 'C (''LINE/TE.'','' $'')'  
    
        Lablin Args (3,1) = 'C (''LABEL/NAME.'',''T'')'
        Lablin Args (3,2) = 'I (''LINE/NUMBER.'',80)'
        Lablin Args (3,3) = 'C (''LINE/TE.'','' $'')'  
    
        Lablin Args (4,1) = 'C (''LABEL/NAME.'',''B'')'
        Lablin Args (4,2) = 'I (''LINE/NUMBER.'',-100)'
        Lablin Args (4,3) = 'C (''LINE/TE.'',''X$'')'  
    
        Lablin Args (5,1) = 'C (''LABEL/NAME.'',''L'')'
        Lablin Args (5,2) = 'I (''LINE/NUMBER.'',100)'
        Lablin Args (5,3) = 'C (''LINE/TE.'',''Y$'')'  

        CALL Reset Values

        RETURN

        END
   
C**********************************************************************   
C SUBROUTINE RESET VALUES 
C
C This routine resets the parameters for the graph options menu to 
C their default values.
C

        SUBROUTINE Reset Values
        
 
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'xygraph.cmn'

        CALL Set Character Value ('TIC BAS X', '   Auto   ')
        CALL Set Character Value ('TIC BAS Y', '   Auto   ')

        CALL Set Toggle Position ('TIC TYP X  ', 4)
        CALL Set Toggle Position ('TIC TYP Y  ', 4)
        
        CALL Set Toggle Position ('TIC LAB X  ', 0)
        CALL Set Toggle Position ('TIC LAB Y  ', 0)
        
        CALL Set Character Value ('TIC EXP X', '   Auto   ')
        CALL Set Character Value ('TIC EXP Y', '   Auto   ')
        
        CALL Set Character Value ('TIC FRC X', '   Auto   ')
        CALL Set Character Value ('TIC FRC Y', '   Auto   ')
	  
        CALL Set Toggle Position ('AXES     ', 0)
        
        CALL Set Toggle Position ('SCALE X  ', 0)
        CALL Set Toggle Position ('SCALE Y  ', 0)

        CALL Set Character Value ('METACODE  ', 'Metacode.met  ')

        RETURN
        END

C**********************************************************************   
C SUBROUTINE GET NEW SCREEN (SCREEN NUMBER) 
C
C Formal Arguments:
C  SCREEN NUMBER  - Stores the screen number.  
C
C This routine sets the current screen to screen number and then displays
C the frame and labels for that screen. 
C

        SUBROUTINE Get New Screen (Screen Number)
         
        IMPLICIT INTEGER*4 (A-Z)
       
       
        INTEGER*4   Screen Number        

        CALL Clear Screen
        CALL Set Screen (Screen Number)
        CALL Display Frame
        CALL Display Labels

        RETURN
        END

C*********************************************************************
C INTEGER*4 FUNCTION TOGGLE POSITION (FIELD NAME) 
C 
C Formal Arguments:
C  FIELD NAME  - Stores the toggle field name. 
C
C This routine returns the toggle position for the specified toggle
C field.
C
        INTEGER*4 FUNCTION Toggle Position (Field Name)
       
        IMPLICIT INTEGER*4   (A-Z)

$INCLUDE: 'screen.str'
      
        CHARACTER*(*)   Field Name
        
        f = Fld Index (Field Name)
        Toggle Position = SC Position (f)

        RETURN
        END
 
C*********************************************************************
C SUBROUTINE DISPLAY FLDS 
C
C This routine displays the current screen fields.
C

        SUBROUTINE Display Flds

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'screen.str'

        INTEGER*4   Main Menu Field, Menu Field
  
        PARAMETER (Main Menu Field = 3)
        PARAMETER (Menu Field      = 4)


        DO 10 i = Begin Screen (Current Screen),
     +            End Screen (Current Screen)
           IF (SC Field Type (i) .NE. Main Menu Field .AND.
     +         SC Field Type (i) .NE. Menu Field)
     +        CALL Write Field (i, SC Attribute (i))
10      CONTINUE

        RETURN
        END           

C**********************************************************************
C SUBROUTINE ERASE MESSAGE (ROW, COL, LEN)
C
C Formal Arguments:
C  ROW           - Row to start erasing.
C  COL           - Column to start erasing.
C  LEN           - Number of characters to erase.
C
C This routine erases messages displayed in the message area of the 
C screen.
C
  	SUBROUTINE Erase Message (Row, Col, Len)
	  
        IMPLICIT INTEGER*4 (A-Z)

        INTEGER*4  Row, Col, Len

        DO 5 i = Row, 18
 	   CALL Erase (i,Col,Len)
 5      CONTINUE
        
        RETURN
        END
             
C********************************************************************
C SUBROUTINE GET FIELD GROUP 
C
C This routine gets the value for a group of screen fields.
C
  	SUBROUTINE Get Field Group 
	IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

$INCLUDE:'screen.str'
	
 10	CALL Get Field	
	IF (SC Field Type (Next Field) .NE. Main Menu Field .AND.
     +      SC Field Type (Next Field) .NE. Menu Field) GOTO 10

	RETURN
	END
            
C************************************************************************
C SUBROUTINE REMOVE BLANKS (ARG)
C
C Formal Arguments:
C  ARG  - Character string to remove blanks.
C
C This routine checks for blanks in a given argument and shifts to
C the left as needed. 
C
	SUBROUTINE Remove Blanks (Arg)

	IMPLICIT INTEGER*4 (A-Z)

	CHARACTER*(*)  Arg

	ls = Lenstr (Arg)

	DO 10  k = 3,ls
	   m = Lenstr (Arg)

	   IF (Arg(k:k) .eq. ' ') THEN
              DO 15 l = k,m
                 IF (Arg(l:l) .ne. ' ') Arg(k:ls) = Arg(l:ls)
 15          CONTINUE
	   ENDIF

 10     CONTINUE

	RETURN
	END

C************************************************************************
C SUBROUTINE OPEN FILE (NUNIT, NAMFIL, STAT, OPENERR)
C
C Formal Arguments:
C  NUNIT     -  File unit number.
C  NAMFIL    -  File name.
C  STAT      -  File status.
C  OPENERR   -  Logical variable set to ture if error occurs.
C
C This routine opens Nunit with Namfil and Stat parameters and returns
C  Openerr = 0, if successful.
C

	SUBROUTINE Open File (Nunit, Namfil, Stat, Openerr)

	IMPLICIT INTEGER*4 (A-Z)

	CHARACTER Namfil*(*), Stat*3
 
	Openerr = 0

	IF (Stat .eq. 'OLD') THEN
	   OPEN (UNIT = Nunit,
     +           FILE = Namfil,
     +           STATUS = Stat,
     +           ERR = 99)
	ELSE
	   OPEN (UNIT = Nunit,
     +           FILE = Namfil,
     +           STATUS = Stat,
     +           ERR = 99)
	ENDIF

	RETURN
 
  99    Openerr = 1
 
	RETURN
	END

