#include <stdio.h>

/* ------------------------------------------------------------------------- */
/* variables global to this module */

/* even though tektronix devices keep track of the current position on the
   graphics screen, I maintain an independant record of this position to 
   avoid loss of generality and much hassle in these routines. 
*/
int CURX=0,CURY=0,CURANGLE=0,CURBASE=14,CURLEG=0;

/* the following variables are used to perform the transformation between
   scale and graphics units. the formulea are :
   sx=gx*GSMX+GSSX
   sy=gy*GSMY+GSSY
   gx=sx*SGMX+SGSX
   gy=sy*SGMY+SGSY
   sorry about the cryptic names.
*/
float GSMX,GSMY,SGMX,SGMY;
float GSSX,GSSY;
int SGSX,SGSY;

/* The following are for drawing axis on a graph 
*/
int ITPOS=2; /* ticsize on possitive side of x or y axis */
int ITNEG=2; /* ticsize on negative side of x or y axis */

float worldxmin;
float worldxmax;
float worldymin;
float worldymax;

/* ------------------------------------------------------------------------- */
/* macros for internal use only */

#define ESC 27
#define CTRL_L 12
#define CTRL_C 3

/* scale to graphics conversion macros */

#define MAXX 1023
#define MAXY 768

#define SCRX_TO_WOR(gx,sx) (sx=((float) gx)*GSMX+GSSX)
#define SCRY_TO_WOR(gy,sy) (sy=((float) gy)*GSMY+GSSY)
#define WORX_TO_SCR(sx,gx) (gx=((int) (sx*SGMX))+SGSX)
#define WORY_TO_SCR(sy,gy) (gy=((int) (sy*SGMY))+SGSY)

/* tektronix bit packing macros */

#define LOWER(w) (w&255)
#define HIGHER(w) ((w&65280)>>8)

#define LOWY(word) ((LOWER(word)&31)|96)
#define HIGHY(word) ((((HIGHER(word)<<3)&24)|((LOWER(word)>>5)&7))|32)

#define LOWX(word) ((LOWER(word)&31)|64)
#define HIGHX(word) ((((HIGHER(word)<<3)&24)|((LOWER(word)>>5)&7))|32)

/* ------------------------------------------------------------------------- */
/* non-graphics routines for internal use only */

/* sqr - takes a square root
*/
float sqr(arg)
float arg;
{
    float g1;

    g1=arg/2.0;
    if (!g1) return 0.0;
    if (!(g1=(arg/g1+g1)/2.0)) return 0.0;
    if (!(g1=(arg/g1+g1)/2.0)) return 0.0;
    if (!(g1=(arg/g1+g1)/2.0)) return 0.0;
    if (!(g1=(arg/g1+g1)/2.0)) return 0.0;
    if (!(g1=(arg/g1+g1)/2.0)) return 0.0;
    if (!(g1=(arg/g1+g1)/2.0)) return 0.0;
    return g1;
 
}/* sqr */

/* lsine - performes a linear aproximation of the sine that looks
          like this /\/\/\/\/\/\.
          angle is in degrees
*/
float lsine(angle)
float angle;
{
    float sign=1.0;

    angle=angle-((float) ((int) (angle/360.0))*360.0);
    if (angle<0) {
        sign=(-1.0);
        angle=(-angle);
        }/* if */
    if (angle>180) {
        angle=angle-180;
        sign=(-sign);
        }/* if */        
    if (angle>90.0) angle=180.0-angle;
    return sign*(angle/90.0);

}/* lsine */

/* lcosine - given the sine of the angle to be used aproximates the cosine
*/
float lcosine(angle,sine)
float angle,sine;
{
    float temp;

    temp=sqr((float)(1-sine*sine));
    angle=angle+90.0;
    angle=angle-((float)((int)(angle/360.0))*360.0);
    if (angle>180.0) temp*=(-1.0);
    
    return temp;

}/* lcosine */

/* ------------------------------------------------------------------------- */
/* graphics routines for internal use only */

void line(x,y,x1,y1)
register int x,y,x1,y1;
{
  static char buf[11];
  
  buf[0]=29;
  buf[1]=HIGHY(y);
  buf[2]=LOWY(y);
  buf[3]=HIGHX(x);
  buf[4]=LOWX(x);
  buf[5]=HIGHY(y1);
  buf[6]=LOWY(y1);
  buf[7]=HIGHX(x1);
  buf[8]=LOWX(x1);
  buf[9]=31;
  buf[10]=0;

  puts(buf);

  CURX=x1;
  CURY=y1;

}/* line */

void move(x,y)
register int x,y;
{
  static char buf[11];

  buf[0]=29;
  buf[1]=HIGHY(y);
  buf[2]=LOWY(y);
  buf[3]=HIGHX(x);
  buf[4]=LOWX(x);
  buf[5]=31;
  buf[6]=0;

  puts(buf);

  CURX=x;
  CURY=y;

}/* move */

/* ------------------------------------------------------------------------- */
/* these are two routines that are declared internal by the graflib library  */
/* but are used externally anyway */

/* init - initially put the hercules graphics card in graphics mode .
          this routine will probably take over a couple of functions.
          1) open up graphics windows on the sun.
          2) set the retrographics vt100 to tek4014 mode.
*/
void init_()
{
  putchar(ESC);
  putchar(91);
  putchar(63);
  putchar(51);
  putchar(56);
  putchar(104);
}/* init */

/* reset_ - return to text mode.
*/
void reset_()
{
    putchar(ESC);
    putchar(CTRL_C);
}/* reset_ */

/* ------------------------------------------------------------------------- */
/* routines for external fortran calling */

/* inkey - Gets a character from the keyboard. 
           Notes: In Unix this function works correctly only in
                  cbreak or raw mode.
                              
                  The keycode section is really only different
                  than keychar on the IBM PC's
*/
void inkey_(keychar, keycode)
char *keychar;
int  *keycode;
{
    *keychar=getchar();
    *keycode=(int) *keychar;

}/* inkey */

/* zclear - clear the graphics screen
*/
void zclear_()
{
  putchar(ESC);
  putchar(91);
  putchar(63);
  putchar(51);
  putchar(56);
  putchar(104);
  putchar(ESC);
  putchar(CTRL_L);
  }/* zclear */

/* gmode - formerly an internal routine to enter  
           graphics mode. now this routine simply clears
           the tektronix graphics screen.
*/
void gmode_(fidiot,sidiot)
int *fidiot,*sidiot;
{

    init_();
    zclear_();

}/* gmode */

/* zplot_ - plot from current position to the given position.
*/
void zplot_(x,y)
float *x,*y;
{
    int p,q;

    WORX_TO_SCR((*x),p);
    WORY_TO_SCR((*y),q);
    line(CURX,CURY,p,q);

}/* zplot_ */

zsplot_(x,y,ink,splottype)
float *x,*y;
int *ink,*splottype;
{
  zplot_(x,y);
}


/* zmove_ - change the current position of the graphics cursor.
*/
void zmove_(x,y)
float *x,*y;
{
   int p,q;

    WORX_TO_SCR((*x),p);
    WORY_TO_SCR((*y),q);
    move(p,q);

}/* zmove_ */

/* zwhere - return the location of the graphics cursor.
            truncated from the original defintion to only return
            scale units.
*/
void zwhere_(x,y)
float *x,*y;
{

    SCRX_TO_WOR(CURX,(*x));
    SCRY_TO_WOR(CURY,(*y));

}/* zwhere */

zbox_(x1,y1,x2,y2)
float *x1, *y1, *x2, *y2 ;
{
  zmove_(x1,y1);
  zplot_(x1,y2);
  zplot_(x2,y2);
  zplot_(x2,y1);
  zplot_(x1,y1);
}

/* 
zline_ - draw a line. 
*/
zline_(x1,y1,x2,y2)
float *x1,*y1,*x2,*y2;
{
  zmove_(x1,y1);
  zplot_(x2,y2);
}

void zxline_(x1,x2,y)
float *x1,*x2,*y;
{
    register int p1,p2,q;

    WORX_TO_SCR((*x1),p1);
    WORX_TO_SCR((*x2),p2);
    WORY_TO_SCR((*y),q);
    line(p1,q,p2,q);

}/* zxline_ */

/* zyline_ - draw a vertical line. 
   if the routines were only to be used in c this would be better to 
   implement as a macro.
*/
void zyline_(y1,y2,x)
float *y1,*y2,*x;
{
    register int q1,q2,p;

    WORX_TO_SCR((*x),p);
    WORY_TO_SCR((*y1),q1);
    WORY_TO_SCR((*y2),q2);
    line(p,q1,p,q2);

}/* zyline_ */

/* zscale_ - set the size of the world cooridinate plane
*/
void zscale_(xmin,xmax,ymin,ymax)
float *xmin,*xmax,*ymin,*ymax;
{
    GSMX=(*xmax-*xmin)/(float) MAXX;
    GSSX=(*xmin);
    GSMY=(*ymax-*ymin)/(float) MAXY;
    GSSY=(*ymin); 
    SGMX=((float) MAXX)/(*xmax-*xmin);
    SGSX=(-(*xmin*SGMX));
    SGMY=((float) MAXY)/(*ymax-*ymin);
    SGSY=(-(*ymin*SGMY));
    
}/* zscale_ */

zgetworld_(xmin,xmax,ymin,ymax) 
float *xmin,*xmax,*ymin,*ymax;
{
  *xmin=worldxmin;
  *xmax=worldxmax;
  *ymin=worldymin;
  *ymax=worldymax;
}


void move_pen(x,y)
register int x,y;
{
  static char buf[11];

  buf[0]=29;
  buf[1]=HIGHY(y);
  buf[2]=LOWY(y);
  buf[3]=HIGHX(x);
  buf[4]=LOWX(x);
  buf[5]=31;
  buf[6]=0;

  puts(buf);

  CURX=x;
  CURY=y;

}/* move_pen */

void label(text)
char *text;
{
   int i,p,q,p1=CURX,q1=CURY;
   static char buf[11];

   for(i=0;text[i];i++) {
       p=p1+CURBASE*i;
       q=q1+CURLEG*i;
       buf[0]=29;
       buf[1]=HIGHY(q);
       buf[2]=LOWY(q);
       buf[3]=HIGHX(p);
       buf[4]=LOWX(p);
       buf[5]=31;
       buf[6]=text[i];
       buf[7]=0;
       puts(buf);
       CURX=p;
       CURY=q;
       }/* for */

   /* this part is not mentioned in the documentation but it seems natural */
   move_pen(p1,q1); 

}/* label */

/* zlabel_ - Write text to the current position at the current angle
            ??? I AM NOT SURE ABOUT FORTRAN ARRAY CALLING CONVENTIONS ???
*/
void zlabel_(text,text_len)
char *text;
int  text_len;
{
    char temp;
    temp = text[text_len];
    text[text_len] = NULL;
    label(text);
    text[text_len] = temp;
}/* zlabel_ */


zcenterlabel_(label,label_length)
     char *label;
     int  label_length;
{
  zlabel_(label,label_length);
}



zrightlabel_(label,label_length)
     char *label;
     int  label_length;
{
  zlabel_(label,label_length);
}


/* zlbdir_ - set the direction in which the next label will be written 
*/
void zlbdir_(angle)
float *angle;
{
    float x,y,ax,ay,f=1.0;

    y=lsine(*angle);
    x=lcosine(*angle,y);
    ax=(x>0)?(x):(-x);
    ay=(y>0)?(y):(-y);
    if (ax>ay)
        f=1.0/ax;
    else f=1.0/ay;
    x*=f;
    y*=f;
    CURANGLE=(int) (*angle);
    CURBASE=(int)(16.0*x);
    CURLEG=(int)(23.0*y);

}/* zlbdir_ */


/* ztcsiz - specify the tick size on x and y axis to be drawn later 
            pos specifies how far the tick exends on the possitive side.
            neg specifies how far the tick exends on the negative side.
*/
void ztcsiz(pos,neg)
int *pos,*neg;
{

    ITPOS=(*pos);
    ITNEG=(*neg);

}/* ztcsiz */

/* zytick_ - plot a horizontal tick w/o moving the current location
*/
void zytick_()
{
    int p,q;

    p=CURX;
    q=CURY;
    line(p-ITNEG,q,p,q);
    line(p+ITPOS,q,p,q);
    zmove_(&p,&q);

}/* zytick_ */

/* zxtick_ - plot a vertical tick w/o moving the current location
*/
void zxtick_()
{
    int p,q;

    p=CURX;
    q=CURY;
    line(p,q-ITNEG,p,q);
    line(p,q+ITPOS,p,q);
    zmove_(&p,&q);

}/* zxtick_ */

/* zxaxis_ - draw an xaxis line from xmin to xmax at yint placing tics
            every ticspc interval
*/
void zxaxis_(xmin,xmax,yint,ticspc)
float *xmin,*xmax,*yint,*ticspc;
{
    float t,p,q;

    p=CURX;
    q=CURY;
    zxline_(xmin,xmax,yint);
    for(t=(*xmin+*ticspc);t<=(*xmax);t+=(*ticspc)) {
        zmove_(&t,yint);
        zxtick_();
      }/* for */
    zmove_(&p,&q);

}/* zxaxis_ */

/* zyaxis_ - draw a yaxis line from ymin to ymax at xint placing tics
            every ticspc interval
*/
void zyaxis_(ymin,ymax,xint,ticspc)
float *ymin,*ymax,*xint,*ticspc;
{
    float t,p,q;

    p=CURX;
    q=CURY;
    zyline_(ymin,ymax,xint);
    for(t=(*ymin+*ticspc);t<=(*ymax);t+=(*ticspc)) {
        zmove_(&xint,&t);
        zytick_();
      }/* for */
    zmove_(&p,&q);

}/* zyaxis_ */

void grwait_()
{
char keychar;
	keychar = getchar();
        putchar(keychar);
}/* grwait      */

/* 
  getkey - Gets a character from the Window: 
*/
void getkey_(keynum)
     int  *keynum;
{
  int cur_char ;
  cur_char = getchar() & 0177;
  *keynum = cur_char;
}/* getkey */

void getkeyall_(keynum)
     int  *keynum;
{
  char keychar;
  keychar = getchar();
  *keynum = (int) keychar;
}/* getkeyall */


void nomovelabel(text)
char *text;
{
   int i,p,q,p1=CURX,q1=CURY;
   static char buf[11];

   for(i=0;text[i];i++) {
       p=p1+CURBASE*i;
       q=q1+CURLEG*i;
       buf[0]=29;
       buf[1]=HIGHY(q);
       buf[2]=LOWY(q);
       buf[3]=HIGHX(p);
       buf[4]=LOWX(p);
       buf[5]=31;
       buf[6]=text[i];
       buf[7]=0;
       puts(buf);
       CURX=p;
       CURY=q;
       }/* for */

}/* nomovelabel */

/* nogolabel_ - Write text to the current position at the current angle
            ??? I AM NOT SURE ABOUT FORTRAN ARRAY CALLING CONVENTIONS ???
*/
void nogolabel_(text,text_len)
char *text;
int  text_len;
{
    char temp;
    temp = text[text_len];
    text[text_len] = NULL;
    nomovelabel(text);
    text[text_len] = temp;
}/* zlabel_ */


/* 
                dummy subroutines for compatibility with fake_x.c 
*/

zfont_(font,font_length)
char *font;     
int font_length;
{
}

zlinewidth_(pwide)
     int *pwide;
{
}

zlinetype_(ptype)
     int *ptype;
{
}

zcaptype_(ptype)
     int *ptype;
{
}

zjointype_(ptype)
     int *ptype;
{
}

size_(x,y)
float *x, *y ;
{
}


quitx_()
{
  reset_();
}

dir_text_(theta,spacing,label,label_len)
char *label;
int  label_len;
float *theta,*spacing;
{
  float angle;
  zlbdir_(theta);
  zlabel_(label,label_len);
  angle = 0.0;
  zlbdir_(&angle);
}


color_(col,col_length)
char *col ;
int  col_length;
{
}



colornumber_(num)
int *num ;
{
}


void getstring_(label,label_length)
char *label ;
int  label_length;
{
  int i, j;
  char abc;
  j=-1;
  i=50; /* initialize i, to get into the loop */
  while(((i >= 32) && (i <= 126)) || i == 16) 
    {
      getkeyall_(&i);
      if (j == label_length-2) break;
      if ((i >= 32) && (i <= 126)) 
	{
	  j++;
	  abc = (char) i;
	  label[j] = abc;
	  fprintf(stderr,"%s",&abc);
	}
      else
	{
	  if (i == 16) 
	    {
	      label[j] = NULL;
	      fprintf(stderr,"\n%s",label);
	      j--;
	      if (j < 0) { j=0;break;}
	    }
	  else 
	    {
	      j++;
	      label[j] = ' ';
	    }
	}
    }
  for (i=j+1;i<label_length;i++)
    {
      label[i] = ' ';
    }
  label[label_length] = NULL;
}

zdump_(coloring,coloring_length)
char *coloring;     
int coloring_length;
{
}

zzdump_()
{
}



turn_ps_on_(infile_name,infile_length)
     char *infile_name;
     int  infile_length;
{
}

int graphics_message__()
{
}

int graphics_message_()
{
}


