C*********************************************************************
C*** File Name: \GEOEAS\CORRES\FCREATE2.FOR 
C*** Source code for the program CORRES
C*********************************************************************
C SUBROUTINE Open Scratch Files
C
C Local Variable:
C  File Name - contains a scratch file name.
C
C  This routine opens three scratch files called, ZZSCTCH1.FIL,
C  ZZSCTCH2.FIL and ZZSCTCH3.FIL.
C
        SUBROUTINE Open Scratch Files

		INCLUDE 'corres.inc'

        CHARACTER*14 Scratch File

C *** Open scratch file to contain 'Data1' structure. 
        Scratch File = 'ZZSCTCH1.FIL  '
cclong        OPEN (UNIT=1,FILE=Scratch File,ACCESS='DIRECT',RECL=10,
        OPEN (UNIT=1,FILE=Scratch File,ACCESS='DIRECT',RECL=20,
     +        FORM='UNFORMATTED',status='unknown')

C *** Open scratch file to contain 'Data2' structure. 
        Scratch File = 'ZZSCTCH2.FIL  '
cclong        OPEN (UNIT=2,FILE=Scratch File,ACCESS='DIRECT',RECL=10,
        OPEN (UNIT=2,FILE=Scratch File,ACCESS='DIRECT',RECL=20,
     +       FORM='UNFORMATTED',status='unknown')
		
C *** Open scratch file to contain 'Data2' structure. 
        Scratch File = 'ZZSCTCH3.FIL  '
cclong        OPEN (UNIT=3,FILE=Scratch File,ACCESS='DIRECT',RECL=10,
        OPEN (UNIT=3,FILE=Scratch File,ACCESS='DIRECT',RECL=20,
     +       FORM='UNFORMATTED',status='unknown')

        RETURN
        END

C**********************************************************************
C SUBROUTINE Read Numerical Data (UnitNmbr, FName, NVarFile, RecStart,
C                                 NData, Type)
C
C Formal Arguments:
C  UnitNmbr  - Unit number of the 'data' matrix simulated by scratch
C              file, Unit number = 1 for the results file and
C              Unit number = 2 for the secondary file.
C  FName     - Name of file from which to read data.
C  NVarFile  - Number of variables to be contained in the scratch file.
C  RecStart  - Number of the first record that contains sample data. 
C  NData     - Returns number of samples in the file.
C  Type      - Flag to indicate whether the file has an "ID variable"
C              Type = 1 for file with "ID variable", Type = 2 no 
C              "ID variable" in file.
C
C Reads in only the samples and ID variable for the variables from the
C Geo-EAS input data file whose name is contained in 'FName' and writes 
C this data to the Scratch File specified by 'UnitNmbr'.
C
        SUBROUTINE Read Numerical Data (UnitNmbr, FName, NVarFile,
     +                                  RecStart, NData, Type)

		INCLUDE 'corres.inc'

        INTEGER*4   UnitNmbr, NvarFile, RecStart, NData, Type
        CHARACTER   FName*(*), Ch*1, ID*10
        REAL*8      TempData (MaxFileVar)
        LOGICAL     ERROR

C *** Open data file. ***
        OPEN (UNIT = 10, FILE = FNAME, status='unknown',ERR = 899)
      
C *** Skip over to first sample data record.
        DO 15 I = 1, RecStart - 1
           READ (10, '(A)', END = 999,ERR=999) Ch
  15    CONTINUE

C *** Get data
C *** N tallies the number of sample data records.
        N = 1
        
 30     IF (Type .EQ. 1) THEN
C *** A sample data record is assigned to the array 'TempData'.
cclong           READ (10,*,END=40,Err=999) (TempData(I),I=1,NVarFile-1), ID 
           READ (10,*,END=40,Err=999) (TempData(I),I=1,NVarFile-1)
cc
C *** The following DO 35 loop contains the sample data record 
C *** of a Geo-EAS data file into a Scratch File.
           DO 35 J = 1, NVarFile - 1
	      I = J + (NVarFile * (N-1)) 
	      WRITE(UnitNmbr,REC=I,ERR=699) TempData(J)
  35       CONTINUE
           I = (NVarFile) + (NVarFile * (N-1))
           WRITE (UnitNmbr, REC = I, ERR = 699) ID
        ELSE

C *** A sample data record is assigned to the array 'TempData'.
           READ (10, *, END=40, Err=999) (TempData(I), I = 1, NVarFile)  
cclong
ccc	   write(*,*) (TempData(I), I = 1, NVarFile)  
cclong
       
C *** The following DO 35 loop contains the sample data record 
C *** of a Geo-EAS data file into a Scratch File.
           DO 36 J = 1, NVarFile
	      I = J + (NvarFile * (N-1)) 
	      WRITE(UnitNmbr, REC=I, ERR=699) TempData(J)
  36       CONTINUE
        ENDIF
        N = N + 1

C *** If the next sample data record to be read exceeds the maximum
C *** number of sample data records permitted then display a warning
C *** and cease reading any more records.
        IF (N .GT. MAXSAMPLES) THEN
	   GOTO 40
        ENDIF
        GOTO 30

  40    CONTINUE
        NData = N - 1
        CLOSE (10)
        RETURN

C *** The following are specific error messages.

  699   Error = .TRUE.
        CLOSE (10)
        CALL Close Scratch Files
        CALL Flash (20, 2, Message)
        Message = 'ERROR - not enough space to write scratch file'
        CALL Flash (20, 2, Message)
        RETURN

  899   Error = .TRUE.
        CLOSE (10)
        CALL Close Scratch Files
        Message = 'ERROR - Error opening data file'
        CALL Flash (20, 2, Message)
        RETURN

  999   Error = .TRUE.
        CLOSE (10)
        CALL Close Scratch Files
        Message ='Error - reading data file'
        CALL Flash (20, 2, Message)
        RETURN

        END

C**********************************************************************
C SUBROUTINE Get Element (DataUnit, Col, Index, VarNum, Type, Value,
C                         ID)
C
C Formal Arguments:
C  DataUnit - Scratch file unit number. 
C  Col      - Position of the variable requested from file.
C  Index    - Number of the sample data record requested of 'Col'.
C  VarNum   - Number of variables in scratch file.
C  Type     - Flag to indicate whether the file has an "ID variable"
C             Type = 1 for file with "ID variable", Type = 2 no 
C             "ID variable" in file.
C  Value    - Returns the data value if requested.
C  ID       - Returns the ID variable if requested.
C
C This routine extracts a data value or ID variable from the
c specified scratch file.
C
        SUBROUTINE Get Element (DataUnit, Col, Index, VarNum, Type,
     +                          Value, ID)

        CHARACTER*10 ID
        REAL*8       Value
        INTEGER*4    VarNum, DataUnit, J, Index, I, Col, Type
         
C *** Compute record number of sample
        J = Col + ((Index - 1) * VarNum)

C *** Read in data value or ID variable 
        IF (Type .EQ. 1) THEN
           READ (DataUnit, REC = J) ID
        ELSE
           READ (DataUnit, REC = J) Value
        ENDIF

        RETURN
        END

C**********************************************************************
C SUBROUTINE Put Element (DataUnit, Col, Index, VarNum, Value, ID,
C                         Type)
C 
C Formal Arguments:
C  DataUnit - Scratch file unit number. 
C  Col      - Position to store variable in the file.
C  Index    - Number of the sample data record requested of 'Col'.
C  VarNum   - Number of variables in scratch file.
C  Type     - Flag to indicate whether the file has an "ID variable"
C             Type = 1 for file with "ID variable", Type = 2 no 
C             "ID variable" in file.
C  Value    - Sample value to store in the file.
C  ID       - ID variable to store in the file.
C
C This routine stores a data value or ID variable into the specified
C scratch file.
C
        SUBROUTINE Put Element (DataUnit, Col, Index, VarNum, Value,
     +                          ID, Type)
 
        CHARACTER*10 ID
        REAL*8       Value
        INTEGER*4    VarNum, DataUnit, Col, J, Index, Type

C *** Calculate the record number
        J = Col + ((Index - 1) * VarNum)
 
C *** Write data value or ID variable 
        IF (Type .EQ. 1) THEN
           WRITE (DataUnit, REC = J) ID
        ELSE
           WRITE (DataUnit, REC = J) Value
        ENDIF

        RETURN
        END

C**********************************************************************
C SUBROUTINE Wrt Output File (Outfil, FVName, FUnits, FFormats, TNVar,
C                             TNData, Title)
C
C Formal Arguments:
C  Outfil   - Name of the output data file.
C  FVName   - Array containing the variable names.
C  FUnits   - Array containing the measurement descriptions.
C  FFormats - Array containing the output format of each variable.
C  TNVar    - Number of variables.
C  TNData   - Number of samples.
C  Title    - Title header for data file.
C
C This routine opens the output file then writes the data to the 
C file.  Scratch File 3 contains the sample data.
C
        SUBROUTINE Wrt Output File (Outfil, FVName, FUnits, FFormats,
     +                              TNVar, TNData, Title)

        IMPLICIT INTEGER*4 (A-Z)

		INCLUDE 'corres.inc'

        CHARACTER*10  FVName (MaxFileVar), FUnits (MaxFileVar), 
     +                FFormats (MaxFileVar), ID 
        CHARACTER     Outfil*(*), Title*(*), Quote*3, Fmt*200
        INTEGER*4     TNVar, TNData
        REAL*8        TempData (MaxFileVar)

        Quote = ''''

C *** Open output file
 	OPEN (UNIT=11, FILE=Outfil, ERR=999, STATUS='UNKNOWN')

C *** Print file heading 
        WRITE (11, '(A)', ERR = 999) Title

C *** Print number of variables
        WRITE (11, '(I5)', ERR = 999) TNVar - 1

C *** Print variable name, measurement description and output format for
C *** each variable.
        DO 20 i = 1, TNVar - 1
           WRITE (11, 25, ERR=999) FVName(i), FUnits(i), FFormats (i)
  25       FORMAT(3A10)
  20    CONTINUE

C *** Construct the array 'Fmt'	which is the format specifier used for
C *** for output.
        CALL Make Format (Fmt, FFormats, TNVar-1)

C *** The following do loop reads all sample data records from 
C *** Scratch File 3 and writes them to the output file.
        DO 30 i = 1, TNData

C *** The following do loop extracts one record of sample data and
C *** assigns it to the array .
           DO 40 j = 1, TNVar - 1
              CALL Get Element(3, j, i, TNVar, 2, TempData(j), ID )
  40	   CONTINUE
           CALL Get Element (3, TNVar, i, TNVar, 1, TempData(1), ID)

C ***    Write the sample data record to the output file.
           CALL Left Justify (ID)
	
cclong The vax can't handle these long formats.
cc           WRITE (11, Fmt, ERR=999) (TempData(K), K = 1, TNVar-1), 
           WRITE (11,fmt, ERR=29) (TempData(K), K = 1, TNVar-1), 
     +                              Quote, ID, Quote
           goto 30
 29        WRITE (11,*, ERR=999) (TempData(K), K = 1, TNVar-1), 
     +                              Quote, ID, Quote
 30     CONTINUE

c***  Close output file.
        CLOSE (11)
        RETURN                                 

 999    CLOSE (11)
cclong        Messages= 'ERROR - writing data file '
        Message= 'ERROR - writing data file '
cclong added flash
                 CALL Flash (20, 2, Message)
cc
        RETURN
        END

C**********************************************************************
C SUBROUTINE Close Scratch Files
C
C This routine closes the scratch files called ZZSCTCH1.FIL,
C ZZSCTCH2.FIL and ZZSCTCH3.FIL.
C        
        SUBROUTINE Close Scratch Files

        CLOSE (1, STATUS='DELETE')
        CLOSE (2, STATUS='DELETE')
        CLOSE (3, STATUS='DELETE')

        RETURN
        END

