C*********************************************************************
C*** File Name: \GEOEAS\CORRES\FILES.FOR 
C*** Source code for the program CORRES
C*********************************************************************
C SUBROUTINE Create Files 
C
C This routine creates the samples, variables, reconstructed data
C and the residuals files.
C
        SUBROUTINE Create Files 
      
        IMPLICIT INTEGER*4  (A-Z)

		INCLUDE 'corres.inc'

        CHARACTER    CNum*1  
        CHARACTER*10 Variable Name (MaxFileVar), Formats (MaxFileVar)
        CHARACTER*60 Title
        INTEGER*4    Num of Var
        LOGICAL      FExist

        WRITE (CNum, '(I1)') NFactors
        l1 = LENSTR (DataFile)
 
C *** Samples File    
        CALL Put Str (' Creating Samples File ',23,20,2,2)
        CALL Variable Names (1, Num of Var, Variable Name, Formats)
        CALL Get F Name (FileName, FilePrefix,SampFile, FExist)
        Title = 'CORRES Samples Results from '//DataFile(1:l1)//
     +           ' # Factors = '//CNum
        CALL WRT Data (1, Title, Num of Var, Variable Name, Formats)
        CALL Ers Line (20, 2, 75)

C *** Variables File    
        CALL Put Str (' Creating Variables File ',25,20,2,2)
        CALL Variable Names (1, Num of Var, Variable Name, Formats)
        CALL Get F Name (FileName, FilePrefix,VarFile, FExist)
        Title = 'CORRES Variables Results from '//DataFile(1:l1)//
     +          ' # Factors = '//CNum
        CALL WRT Data (2, Title, Num of Var, Variable Name, Formats)
        CALL Ers Line (20, 2, 75)

C *** Reconstructed Data File    
        IF (RFLAG) THEN
        CALL Ers Line (20, 2, 75)
        CALL Put Str (' Creating Reconstructed Data File ',34,20,2,2)
        CALL Variable Names (2, Num of Var, Variable Name, Formats)
        CALL Get F Name (FileName, FilePrefix, ReconFile, FExist)
        Title = 'CORRES Reconstructed Results from '//DataFile(1:l1)//
     +           ' # Factors = '//CNum
        CALL WRT Data (3, Title, Num of Var, Variable Name, Formats)
        CALL Ers Line (20, 2, 75)

C *** Residuals File    
        CALL Put Str (' Creating Residuals File ',25,20,2,2)
        CALL Variable Names (3, Num of Var, Variable Name, Formats)
        CALL Get F Name (FileName, FilePrefix, ResidFile, FExist)
        Title = 'CORRES Residuals Results from '//DataFile(1:l1)//
     +           ' # Factors = '//CNum
        CALL WRT Data (4, Title, Num of Var, Variable Name, Formats)
        CALL Ers Line (20, 2, 75)
        ENDIF

        RETURN
        END

C*********************************************************************
C SUBROUTINE Variable Names (Type, Num of Var, Variable Name, Formats)
C
C Formal Arguments:
C  Type           -  Flag to indicate which file to create  
C                    variables names for.
C  Num of Var     -  Returns then number of variables in the file.
C  Variable Name  -  Returns the array of variable names created.
C  Formats        -  Returns the array of output formats for each
C                    variable.
C
C This routine determines the number of variables, the variable
C names and the output format for the samples, variables, residuals
C and reconstructed data files.
C

        SUBROUTINE Variable Names (Type, Num of Var, Variable Name,
     +                             Formats)

        IMPLICIT INTEGER*4  (A-Z)

		INCLUDE 'corres.inc'

        CHARACTER    CNum*1  
        CHARACTER*10 Variable Name (MaxFileVar), Formats (MaxFileVar)
        INTEGER*4    Type, Num of Var

C *** Samples and Variable file variable names
        IF (Type .EQ. 1) THEN
           j = 1
           k = 2*NFactors
           l = 2*NFactors + 1
           m = 1
           Num of Var = NFactors * 4 + 2
           DO 5 i = 1, NFactors * 4 +1
              IF (j .LE. NFactors) THEN
                 WRITE (CNum, '(I1)') j
                 Variable Name (i) = 'FACTOR'// CNum
                 Formats (i) = 'F10.5'
                 Variable Name (i + NFactors) = 'COORD'// CNum
                 Formats (i+NFactors) = 'F10.5'
                 Variable Name (1 + k) = 'WEIGHT'
                 Formats (1 + k) = 'F8.5'
                 Variable Name (i + m + k) = 'CA'// CNum 
                 Formats (i + m + k) = 'F7.1'
                 Variable Name (i + m + l) = 'CR'// CNum 
                 Formats (i + m + l) = 'F7.1'
                 j = j + 1
                 m = m + 1
              ENDIF
 5         CONTINUE
           Variable Name (NFactors * 4 + 2) = 'SUMCR'
           Formats (NFactors * 4 + 2) = 'F7.1'

C *** Reconstructed Data File variable names
        ELSEIF (Type .EQ. 2) THEN
           Num of Var = NVar
           DO 10 i = 1, Num of Var
              Variable Name (i) = '*' // VarNam (i)
              IF (FileFormats (i) .EQ. '          ') THEN
                 Formats (i) =  'G16.9'
              ELSE
                 Formats (i) = FileFormats (i)
              ENDIF
 10      CONTINUE

C *** Residuals File variable names
        ELSEIF (Type .EQ. 3) THEN
           Num of Var = NVar
           DO 15 i = 1, Num of Var
              Variable Name (i) = '-'//VarNam (i)
              IF (FileFormats (i) .EQ. '          ') THEN
                 Formats (i) =  'G16.9'
              ELSE
                 Formats (i) = FileFormats (i)
              ENDIF
 15        CONTINUE
        ENDIF

        RETURN
        END

C*********************************************************************
C SUBROUTINE Wrt Data (Type, Title, Num of Var, Variable Name, 
C                      Formats)
C
C Formal Arguments:
C  Type           -  Flag to indicate which file to write the  
C                    data to.
C  Title          -  Title header for file.
C  Num of Var     -  Number of variables in the file.
C  Variable Name  -  Array of variable names.
C  Formats        -  Array of output formats for each variable.
C
C This routine opens the output file then writes the data to the 
C specified file.
C
        SUBROUTINE Wrt Data (Type, Title, Num of Var, Variable Name, 
     +                       Formats)

        IMPLICIT INTEGER*4   (A-Z)

		INCLUDE 'corres.inc'

        CHARACTER     CNum*1, Quote*3, Row ID*10, IDNum*4, CNVar*2 
        CHARACTER*10  Variable Name (MaxFileVar), Formats (MaxFileVar)
        CHARACTER*60  Title
        CHARACTER*200 Fmt
        INTEGER*4     Num of Var, Type, LenStr
        LOGICAL       Fexist, Error
        REAL*8        Missing, SumCR
     
        Quote = ''''
        Missing = 1E31

        OPEN (UNIT = 11, FILE = FileName, status='unknown')
 
C *** Write header data to the file      
        WRITE (11, '(A60)') Title
        WRITE (11, '(I3)') Num of Var
        DO 15 i = 1, Num of Var
           WRITE (11, '(A10, 10X, A10)')  Variable Name (i), Formats (i)
 15     CONTINUE


C *** Write data for Samples File
        WRITE (CNum, '(I1)') NFactors
        IF (Type .EQ. 1) THEN
           Fmt = '('//Cnum//'F10.5, 1X, '//Cnum//'F10.5, 1X, F8.5, 1X, '
     +            //CNum//'(F7.1, 1X, F7.1), 1X, F7.1, 1X, A1, A10, A1)'
           DO 20 i = 1, NSamples
              WRITE (IDNum, '(I4)') i
              CALL Left Justify (IDNum)
              Row ID = 'S'//IDNum
              SumCR = 0.0
              DO 21 l = 1, NFactors
                 SumCR = SumCR + CRI (i,l)
 21           CONTINUE
cclong              WRITE (11, Fmt) (FACI  (i,j), j = 1, NFactors),
              WRITE (11, *) (FACI  (i,j), j = 1, NFactors),
     +                        (COORI (i,j), j = 1, NFactors),
     +                        PI (i),
     +                        (CAI (i,j), CRI (i,j), j = 1, NFactors), 
     +                        SumCr
cc     +                        SumCr,
cc     +                        Quote,Row Id,Quote
cclong

 20        CONTINUE

C *** Write data for Variables File
        ELSEIF (Type .EQ. 2) THEN
           Fmt = '('//Cnum//'F10.5, 1X, '//Cnum//'F10.5, 1X, F8.5, 1X, '
     +            //CNum//'(F7.1, 1X, F7.1), 1X, F7.1, 1X, A1, A10, A1)'
           DO 25 i = 1, NVar
              SumCR = 0.0
              DO 26 l = 1, NFactors
                 SumCR = SumCR + CRJ (i,l)
 26           CONTINUE
cclong              WRITE (11, Fmt) (FACJ (i,j), j = 1, NFactors),
              WRITE (11, *) (FACJ (i,j), j = 1, NFactors),
     +                        (COORJ (i,j), j = 1, NFactors),
     +                        PJ (i),
     +                        (CAJ (i,j), CRJ (i,j), j = 1, NFactors), 
     +                        SumCR
cc     +                        SumCR,
cc     +                        Quote, VarNam (i), Quote
cclong
 25        CONTINUE
           IF (NSuppVar .NE. 0) THEN
              Fmt = '('//Cnum//'G10.1, 1X, '//
     +               Cnum//'F10.5, 1X, G8.1, 1X, '//CNum//
     +               '(G7.1, 1X, F7.1), 1X, F7.1, 1X, A1, A10, A1)'
              DO 30 i = 1, NSuppVar
                 SumCR = 0.0
                 DO 31 l = 1, NFactors
                    SumCR = SumCR + CRJS (i,l)
 31              CONTINUE
cclong                 WRITE (11, Fmt) (Missing, j = 1,NFactors),
                 WRITE (11, *) (Missing, j = 1,NFactors),
     +                           (COORJS (i,j), j = 1, NFactors),
     +                           Missing,
     +                           (Missing,CRJS (i,j), j = 1, NFactors), 
     +                           SumCR
cc     +                           SumCR,
cc     +                           Quote, SVarNam(i), Quote
cclong
 30           CONTINUE
           ENDIF               

C *** Write data for Reconstructed Data File
        ELSEIF (Type .EQ. 3) THEN
           CALL Cal Recons Table
           CALL Make Format (Fmt, Formats, NVar)
           DO 45 i = 1, NSamples
              WRITE (IDNum, '(I4)') i
              CALL Left Justify (IDNum)
              Row ID = 'S'//IDNum
cclong              WRITE (11, Fmt) (RTable (i,j), j = 1, NVar), 
              WRITE (11, *) (RTable (i,j), j = 1, NVar)
cc              WRITE (11, *) (RTable (i,j), j = 1, NVar), 
cc     +                         Quote, Row ID, Quote
cclong
 45        CONTINUE
	    
C *** Write data for Residuals File
        ELSEIF (Type .EQ. 4) THEN
           CALL Get F Name (FileName, FilePrefix, DataFile, Fexist)
           CALL ReadData (Error)
           CALL Make Format (Fmt, Formats, NVar)
           DO 50 i = 1, NSamples
              WRITE (IDNum, '(I4)') i
              CALL Left Justify (IDNum)
              Row ID = 'S'//IDNum
cclong              WRITE (11, Fmt) (RTable (i,j) - PIJ (i,j), j = 1,NVar),
              WRITE (11, *) (RTable (i,j) - PIJ (i,j), j = 1,NVar)
cc              WRITE (11, *) (RTable (i,j) - PIJ (i,j), j = 1,NVar),
cc     +                         Quote, Row ID, Quote
cclong
 50        CONTINUE
        ENDIF
                 
        CLOSE (11)
        RETURN
        END

C*********************************************************************
C SUBROUTINE Cal Recons Table
C
C This routine computes the reconstructed data table.
C 
       SUBROUTINE Cal Recons Table
         
        IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'corres.inc'

        CALL Ers Line (20, 2, 75)
        CALL Put Str (' Computing Reconstructed Data Table ',
     +               36,20,2,2)
        DO 10 i = 1, NSamples
           DO 10 j = 1, NVar
              RTable (i,j) = 0.0
              DO 20  k = 1, NFactors
                 RTable (i,j) = RTable (i,j) + FACI (i,k) * 
     +                FACJ (j,k) * dsqrt(VALPR (K+1))
 20           continue
              RTable (i,j) = PI (i) * PJ (j) * (1.0 + RTable (i,j)) 
 10        continue
        DO 30 i = 1, NSamples
           DO 30 j = 1, NVar
 30     RTable (i,j) = SOMIJ * RTable (i,j)

        CALL Ers Line (20,2, 75)

        RETURN
        END

C*********************************************************************
C SUBROUTINE Make Format (Fmt, Formats, Num Var)
C
C Formal Arguments:
C  Fmt     - Returns the format specifier.
C  Formats - Array for output formats for each variable.
C  Num Var - Number of variables.
C
C This routine constructs the FORTRAN format specifier used for output 
C files.
C

        SUBROUTINE Make Format (Fmt, Formats, Num Var)
       
        IMPLICIT INTEGER*4 (A-Z)

		INCLUDE 'corres.inc'
 
        CHARACTER   CNum*2
        CHARACTER*10  Formats (MaxFileVar)
        CHARACTER*200  Fmt
        INTEGER*4     Num Var, Num

        DO 5 i = 1, 200
           Fmt (i:i) = ' '
 5      CONTINUE

        
        Fmt (1:1) = '('
        j = 1
        i = 2 
        num = 1
        index = 0       
 10     IF (i-1 .LT. Num Var) THEN
           IF (Formats (i) .EQ. Formats (i-1)) THEN
              Num = Num + 1
              WRITE (CNum, '(I2)') Num
              i = i + 1
              IF (i .LT. Num Var) THEN
                 GOTO 10
              ELSE
                 IF (i .GT. Num Var) num = num - 1
                 Index = i - 1
                 GOTO 20
              ENDIF
           ELSE
              Index = i - 1
              GOTO 20
           ENDIF
  20       IF (Num .NE. 1) THEN
              j = j + 1 
              Fmt (j:j+2) = CNum // '(' 
              j = j + 2
           ENDIF
           k = LenStr (Formats (Index))       
           DO 25 l = 1, k
              j = j + 1
              Fmt (j:j) = Formats(Index) (l:l)
  25       CONTINUE
           IF (Num .EQ. 1) THEN
              j = j + 1
              Fmt (j:j+3) = ',1X,'
              j = j + 3
           ELSE
              j = j + 1
              Fmt (j:j+4) = ',1X),'
              j = j + 4
           ENDIF
           i = Index + 2
           Num = 1
           GOTO 10
        ELSE 
           k = LenStr (Formats (Num Var))
           DO 30 l = 1, k
              j = j + 1
              Fmt (j:j) = Formats (Num Var) (l:l)
 30        CONTINUE
           j = j + 1         
           Fmt (j:j+13) = ',1X,A1,A10,A1)'
        ENDIF

        RETURN
        END

