C*********************************************************************
C*** File Name: \GEOEAS\CORRES\WRTSCR.FOR 
C*** Source code for the program CORRES
C*********************************************************************
C SUBROUTINE Wrt Menu
C
C Local Variables:
C  Wrtermenu          - Name of the results menu in the screen definition 
C                     file.
C  WrtScreen        - Number of the results screen in the screen 
C                     definition file.
C  Outfile          - Output file name.
C  ResultFile       - Results file name.
C  SecondaryFile    - Secondary file name.
C
C  FVName, FUnits,  - Arrays for variable names, unit measurements, and  
C  FFormats           format specifiers from the results or secondary 
C                     file.
C
C  FNVar1           - Number of variables in the results file.
C  NData1           - Number of sample in the results file.
C  FNVar2           - Number of variables in the secondary file.
C  NData2           - Number of sample in the secondary file.
C
C  VName1, Units1,  - Arrays for variable names, unit measurements and
C  Formats1, NVar1,   format specifiers and the number of variables 
C                     selected from the results file.
C
C  VName2, Units2,  - Arrays for variable names, unit measurements and
C  Formats2, NVar2,   format specifiers and the number of variables 
C                     selected from the secondary file.
C   
C  VPos1, Vpos2     - Arrays to store the position of the variables 
C                     selected from the result and secondary files.
C  TNVar, TNData    - Total number of variables and samples in the
C                     output file.
C  UnitNum          - File unit number.
C  Type             - Flag to indicate whether the file has an 
C                     "ID variable".  Type = 1 for file with 
C                     "ID variable",  Type = 2 no "ID variable" 
C                     in file.
C 
C  FExist           - Logical variable set to .TRUE. if the file 
C                     exists.
C  OverWrite        - Logical variable set to .TRUE. if the file can   
C                     be overwritten.
C 
C This routine displays the write screen, accesses the menu option
C selected by the user then calls the routine to process the menu
C option. 
C
        SUBROUTINE Wrt Menu 

        IMPLICIT INTEGER*4 (A-Z)

		INCLUDE 'corres.inc'
		INCLUDE 'screen.str'
        
        CHARACTER*10  Wrtermenu, Varcod, ID, Blanks
        CHARACTER*10  FVName(MaxFileVar), FUnits(MaxFileVar),
     +                FFormats(MaxFileVar) 
        CHARACTER*10  VName1(MaxFileVar), VName2(MaxFileVar)
        CHARACTER*10  Units1 (MaxFileVar),  Units2 (MaxFileVar)
        CHARACTER*10  Formats1 (MaxFileVar), Formats2 (MaxFileVar)
        CHARACTER*14  SecondaryFile, ResultFile
        CHARACTER*75  Dummy
        INTEGER*4     WrtScreen, Position, Fld Index, LenStr, UnitNum
        INTEGER*4     FNVar1, FNVar2, NVar1, NVar2, NData1, NData2
        INTEGER*4     VPos1 (MaxFileVar), VPos2 (MaxFileVar)  
        INTEGER*4     TNVar, TNData, Type, CR, Down
        LOGICAL       FExist, OverWrite, Get Yes No
        REAL*8        Value, Missing

        DATA Wrtermenu    /'WRT MENU  '/
        DATA WrtScreen  /6/
        Missing = 1E31
        OverWrite = .TRUE.
        
C *** Set up the screen
        CALL Get New Screen (WrtScreen)
        CALL Put Str (FilePrefix, 50, 2, 10, 0)
        
        IF (RFLAG) THEN
           CALL Set T Value ('RESULTFILE',0, SampFile)
           CALL Set T Value ('RESULTFILE',1, VarFile)
           CALL Set T Value ('RESULTFILE',2, ResidFile)
           CALL Set T Value ('RESULTFILE',3, ReconFile)
           CALL Set T Value ('RESULTFILE',4, '(Row+Column)')
        ELSE
           f = Fld Index ('RESULTFILE')
           SC Members (f) = 3
           CALL Set T Value ('RESULTFILE',0, SampFile)
           CALL Set T Value ('RESULTFILE',1, VarFile)
           CALL Set T Value ('RESULTFILE',2, '(Row+Column)')
        ENDIF
   
 5      CALL Put Menu (Wrtermenu)
        CALL Get Menu Choice (Wrtermenu, MenuChoice)
   
C *** QUIT
        IF (MenuChoice .EQ. 'Quit          ') THEN
           f = Fld Index ('RESULTFILE')
           SC Members (f) = 5
           CALL Close Scratch Files   
           ResultFile = '              '
           SecondaryFile = '              '
           RETURN

C *** RESULTS VARIABLES
        ELSEIF (MenuChoice .EQ. 'Results File') THEN
 6         CALL Get Field
           CALL Get Toggle Value ('RESULTFILE', Position, Dummy)
           IF (Dummy (1:14) .EQ. '(Row+Column)  ')
     +        Dummy (1:14) = SampFile
           IF (Dummy (1:14) .NE. ResultFile) THEN
              ResultFile = Dummy (1:14)
              CALL Get F Name (FileName, FilePrefix, ResultFile, FExist)
              IF (.NOT. FExist) THEN
                 Message = 'Type - File not found '
                 CALL Flash (20, 2, Message)
                 CLOSE (10)
                 Next Field = Fld Index ('RESULTFILE')
                 GOTO 6
 	      ELSE
                 OPEN (UNIT = 10, FILE = FileName,status='unknown')
	         READ (10, '(A)') Header
	         READ(10, *) FNVar1
                 DO 10 i = 1, MaxFileVar
                   IF (i .GT. FNVar1) THEN
                      FVName (i) = '          '
                   ELSE
                      READ (10,'(3A10)') FVName (i), FUnits (i), 
     +                                   FFormats(i)
                   ENDIF
 10              CONTINUE
                 CLOSE (10)

C *** Display variable names
                 CALL Dsp Var Names (Wrt Screen, FVName)
                 DO 16 j = 1, MaxVar
                    IF (j .GT. FNVar1) THEN
                       WRITE (Varcod, '(A5,I2)') 'RVAR  ', j
                       CALL Set Toggle Position (Varcod, 0)
                    ELSE
                       WRITE (Varcod, '(A5,I2)') 'RVAR  ', j
                       CALL Set Toggle Position (Varcod, 1)
                    ENDIF
                    IF (j .EQ. FNVar1) THEN
                       WRITE (Varcod, '(A5,I2)') 'RVAR  ', j
                       f = Fld Index (Varcod)
                       m = Fld Index ('WRT MENU  ')
                       CR = SC Carriage Return (f)
                       Down = SC Down (f)
                       SC Carriage Return (f) = m - f
                       SC Down (f) = m - f
                    ENDIF
 16              CONTINUE
                CALL Display Flds
             ENDIF
          ENDIF
          CALL Get FGroup 
 
          NVar1 = 0
          DO 17 i = 1, FNVar1 
             WRITE (Varcod, '(A5,I2)') 'RVAR  ', i
             CALL Get Toggle Value (Varcod, Position, Dummy)
             IF (Position .EQ. 1) THEN
                NVar1 = NVar1 + 1
                VName1 (NVar1) = FVName (i)
                Units1 (NVar1) = FUnits (i)
                Formats1 (NVar1) = FFormats (i)
                VPos1 (NVar1) = i
             ENDIF
 17       CONTINUE
          VPos1 (NVar1+1) = FNVar1 + 1 
          SC Carriage Return (f) = CR
          SC Down (f) = Down

          
C *** PASSTHUR VARIABLES
        ELSEIF (MenuChoice .EQ. 'Secondary Fi') THEN
 18        CALL Get Field
           CALL Get Character Value ('PTVFILE   ', Dummy)
           IF (Dummy (1:14) .EQ. '              ' .AND. 
     +         SecondaryFile .EQ. '              ' ) THEN
               CALL Ers Field ('PTVAR  1  ')
               CALL Ers Field ('PTVAR  2  ')
               CALL Ers Field ('PTVAR  3  ')
               CALL Ers Field ('PTVAR  4  ')
               CALL Ers Field ('PTVAR  5  ')
               FNVar2 = 0
               NVar2 = 0
               NData2 = 0
               GOTO 5
           ENDIF
           IF (Dummy (1:14) .NE. SecondaryFile) THEN
              SecondaryFile = Dummy (1:14)
              CALL Get F Name (FileName, FilePrefix, 
     +                        SecondaryFile, Fexist)
              IF (.NOT. FExist) THEN
                 Message = 'Error - File not found '
                 CALL Flash (20, 2, Message)
                 CLOSE (10)
                 Next Field = Fld Index ('PTVFILE   ')
                 GOTO 18
 	      ELSE
 	         OPEN (UNIT = 10, FILE = FileName,status='unknown')
	         READ (10, '(A)') Header
	         READ(10, *) FNVar2
                 DO 20 i = 1, FNVar2
                    READ (10,'(3A10)') FVName (i), FUnits (i),
     +                                 FFormats (i)
 20              CONTINUE
                 CLOSE (10)
                 FVName (FNVar2+1) = 'None'
                 CALL Load Toggle Values ('PTVAR  1  ', 10, FNVar2+1,
     +                                    FVName, 0)
                 CALL Load Toggle Values ('PTVAR  2  ', 10, FNVar2+1,
     +                                    FVName, 1)
                 CALL Load Toggle Values ('PTVAR  3  ', 10, FNVar2+1,
     +                                    FVName, 2)
                 CALL Load Toggle Values ('PTVAR  4  ', 10, FNVar2+1,
     +                                    FVName, 3)
                 CALL Load Toggle Values ('PTVAR  5  ', 10, FNVar2+1,
     +                                    FVName, 4)
                 CALL Display Flds
              ENDIF  
           ENDIF 
           CALL Get FGroup
           NVar2 = 0
           DO 25 i = 1, 5
             WRITE (Varcod, '(A6,I2)') 'PTVAR ', i
             CALL Get Toggle Value (Varcod, Position, Dummy)
             IF (Dummy (1:4) .NE. 'None') THEN
                NVar2 = NVar2 + 1
                VName2 (NVar2) = FVName (i)
                Units2 (NVar2) = FUnits (i)
                IF (FFormats (i) .EQ. '          ') THEN
                   Formats2 (NVar2) = 'G16.9'
                ELSE
                  Formats2 (NVar2) = FFormats (i)
                ENDIF
                VPos2 (NVar2) = i
             ENDIF
 25       CONTINUE
  
C *** WRITE
        ELSEIF (MenuChoice .EQ. 'Write       ') THEN
           CALL Get Field
           CALL Get Character Value ('OUTPUTFILE', OutFile)
           IF (OutFile .EQ. '              ') THEN
              CALL Beep
              Message = 'ERROR - Output file not specified'
              CALL Flash (20,2,Message)
              GOTO 5      
           ENDIF
           IF (OutFile .EQ. SampFile  .OR.
     +         OutFile .EQ. VarFile   .OR.
     +         OutFile .EQ. ResidFile .OR.
     +         OutFile .EQ. ReconFile     ) THEN
              Message = 'ERROR - Output file name cannot be same '//
     +                           'as a results file name.'
              CALL Flash (20, 2, Message)
              GOTO 5
           ENDIF   
  	   CALL Get F Name (FileName, FilePrefix, OutFile, Fexist)
           IF (FExist) THEN
              Message = 'ERROR - File already exists'//
     +                 ' Overwrite (Y/N) ?'
              OverWrite = Get Yes No (20, 2, Message)
           ENDIF
           IF (.NOT. OverWrite ) THEN
              OverWrite = .TRUE.
              GOTO 5
           ENDIF 
           CALL Get Toggle Value ('RESULTFILE', Position, Dummy)
           CALL Open Scratch Files
           IF (Dummy (1:14) .NE. '(Row+Column)  ') THEN
              CALL Put Str (' Reading Results file ', 22, 20, 2, 2)
              Type = 1
              UnitNum = 1
              NData1 = 0
              CALL Get F Name (FileName, FilePrefix, ResultFile, FExist)
              CALL Read Numerical Data (UnitNum, FileName, FNVar1+1,
     +                                  FNVar1+3, NData1, Type)
              IF (FNVar2 .NE. 0) THEN
                 CALL Ers Line (20, 2, 75)  
                 CALL Put Str (' Reading Secondary file ', 24, 20, 2, 2)
                 Type = 2
                 UnitNum = 2
                 NData2 = 0
                 CALL Get F Name (FileName, FilePrefix, SecondaryFile,
     +                           FExist)
                 CALL Read Numerical Data (UnitNum, FileName, FNVar2,
     +                                  FNVar2+3, NData2, Type)
              ENDIF
           ELSE
              CALL Ers Line (20, 2, 75)  
              CALL Put Str (' Reading Samples file ', 22, 20, 2, 2)
              Type = 1
              UnitNum = 3
              NData1 = 0
              CALL Get F Name (FileName, FilePrefix, SampFile, FExist)
              CALL Read Numerical Data (UnitNum, FileName, FNVar1+1,
     +                                  FNVar1+3, NData1, Type)

              CALL Ers Line (20, 2, 75)  
              CALL Put Str (' Reading Variables file ', 24, 20, 2, 2)
              UnitNum = 2
              NData2 = 0
              CALL Get F Name (FileName, FilePrefix, VarFile,
     +                         FExist)
              CALL Read Numerical Data (UnitNum, FileName, FNVar1+1,
     +                                  FNVar1+3, NData2, Type)
              CALL Ers Line (20, 2, 75)  
              CALL Put Str (' Creating scratch Results file ',
     +                      31, 20, 2, 2)
              TNVar = NVar1 
              TNData = NData1 + Ndata2
              DO 26 i = 1, TNData     
                 DO 27 j = 1, TNVar + 1
                    IF (i .LE. Ndata1) THEN
                       IF (j .LE. TNVar) THEN         
                          Type = 2
                          Unitnum = 3
                          CALL Get Element (UnitNum, VPos1 (j), i, 
     +                                   FNVar1+1, Type, Value, ID)
                          Unitnum = 1
                          CALL Put Element (UnitNum, j, i, TNVar+1, 
     +                                  Value, ID, Type)
                       ELSE
                          Type = 1
                          Unitnum = 3
                          CALL Get Element (UnitNum, VPos1 (j), i, 
     +                                   FNVar1+1, Type, Value, ID)
                          Unitnum = 1
                          CALL Put Element (UnitNum, j, i, TNVar+1, 
     +                                  Value, ID, Type)
                       ENDIF
                    ELSE
                       IF (j .LE. TNVar) THEN         
                          Type = 2
                          UnitNum = 2
                          CALL Get Element (UnitNum, VPos1(j), i-Ndata1,
     +                                      FNVar1+1, Type, Value, Id)
                          IF (Value .EQ. Missing) Formats1 (j) = 'G16.9'
                          Unitnum = 1
                          CALL Put Element (UnitNum, j, i, TNVar+1,
     +                                      Value, ID, Type)
                       ELSE  
                          Type = 1
                          UnitNum = 2
                          CALL Get Element (UnitNum, VPos1(j), i-Ndata1,
     +                                      FNVar1+1, Type, Value, Id)
                          Unitnum = 1
                          CALL Put Element (UnitNum, j, i, TNVar+1,
     +                                      Value, ID, Type)
                       ENDIF
                    ENDIF
  27             CONTINUE
 26           CONTINUE 
              IF (FNVar2 .NE. 0) THEN
                 CALL Ers Line (20, 2, 75)  
                 CALL Put Str (' Reading Secondary file ', 24, 20, 2, 2)
                 Type = 2
                 UnitNum = 2
                 NData2 = 0
                 CALL Get F Name (FileName, FilePrefix, SecondaryFile,
     +                         FExist)
                 CALL Read Numerical Data (UnitNum, FileName, FNVar2,
     +                                     FNVar2+3, NData2, Type)
              ENDIF
              NData1 = TNData
           ENDIF   
           CALL Ers Line (20, 2, 75)  
           CALL Put Str (' Creating scratch output file ', 30, 20, 2, 2)
           TNVar = NVar1 + NVar2 + 1 
cclong incompatible data tyype for amax1
cc           TNData = AMAX1(NData1, NData2)
           TNData = AMAX1(NData1*1., 1.*NData2)
cc
           l1 = LenStr (ResultFile)
           l2 = LenStr (SecondaryFile)
           IF (Dummy (1:14) .EQ. '(Row+Column)  ') THEN 
              ResultFile = '(Row+Column)  '
              Header = 'CORRES - Custom output file from '//
     +                  ResultFile (1:l1) // ' .'
           ELSE     
              IF (NData2 .EQ. 0) THEN
                 Header = 'CORRES - Custom output file from '//
     +                     ResultFile (1:l1) // ' .'
              ELSE
                 Header = 'CORRES - Custom output file from '//
     +                  ResultFile (1:l1) // ' and ' // 
     +                  SecondaryFile (1:l2) // ' .'
              ENDIF
           ENDIF
           DO 30 i = 1,TNVar - 1
              IF ( i .LE. NVar1 ) THEN
                 FVName (i) = VName1 (i)
                 FUnits (i) = Units1 (i)
                 FFormats (i) = Formats1 (i)
              ELSE
                 FVName (i) = VName2 (i-NVar1)
                 FUnits (i) = Units2 (i-NVar1)
                 FFormats (i) = Formats2 (i-NVar1)
              ENDIF
 30        CONTINUE
           DO 35 i = 1, TNData     
              DO 40 j = 1, TNVar - 1 
                 Type = 2
                 IF (j .LE. NVar1) THEN
                    IF (i .LE. Ndata1) THEN
                       Unitnum = 1
                        CALL Get Element (UnitNum, VPos1 (j), i, 
     +                                FNVar1+1, Type, Value, ID)
                       Unitnum = 3
                       CALL Put Element (UnitNum, j, i, TNVar, 
     +                                   Value, ID, Type)
                    ELSE
                       Unitnum = 3
                       FFormats (j) = 'G16.9'
                       CALL Put Element (UnitNum, j, i, TNVar, Missing,
     +                                   ID, Type)
                    ENDIF
                 ELSE
                    IF (Ndata2 .NE. 0) THEN
                       IF (i .LE. Ndata2) THEN
                          UnitNum = 2
                          CALL Get Element (UnitNum, VPos2 (j-NVar1),
     +                                   i, FNVar2, Type, Value, Id)
                          
                          Unitnum = 3
                          CALL Put Element (UnitNum, j, i, TNVar,
     +                                      Value, ID, Type)
                       ELSE
                          FFormats (j) = 'G16.9'
                          Unitnum = 3
                          CALL Put Element (UnitNum, j, i, TNVar,
     +                                      Missing, ID, Type)
                       ENDIF
                    ENDIF
                 ENDIF
  40          CONTINUE
              IF ( I .LE. NData1) THEN
                 Type = 1
                 UnitNum = 1
                 CALL Get Element (UnitNum, VPos1(NVar1+1), i, FNVar1+1,
     +                            Type, Value, ID)
                 UnitNum = 3
                 CALL Put Element (UnitNum, TNVar, i, TNVar, Value, ID, 
     +                             Type)
              ELSE
                 Type = 1
                 UnitNum = 3
                 CALL Put Element (UnitNum, TNVar, i, TNVar, Value,  
     +                             Blanks,Type)
              ENDIF
              CONTINUE
 35        CONTINUE 
  	   CALL Get F Name (FileName, FilePrefix, OutFile, Fexist)
           CALL Ers Line (20, 2, 75)  
           CALL Put Str (' Writing to Output file ', 24, 20, 2, 2)
           CALL Wrt Output File (FileName, FVName, FUnits, FFormats,
     +                           TNVar, TNData, Header)
           SecondaryFile = '              '
           outfile_on = .true.
        ENDIF 
        CALL Ers Line (20, 2, 75)  
        
        GOTO 5      
        END     



