clarge
c**********************************************************************
c*** File name: PREP3.FOR (Source code for the program DATAPREP)
c**********************************************************************
c        L o a d  A l l  T o g g l e  F i e l d s     (subroutine)
c**********************************************************************
      SUBROUTINE Load All Toggle Fields(Count) 

c***  This routine loads the variable names into the corresponding
c***  toggle fields.
c***
c***  VARIABLES:
c***  Count    - contains the number of variable names to be loaded.
c***  Vtoggle1 - contains the name of the toggle field in the 
c***             Dataprep.scr file.
c***  Varnam   - global character array containing the variable
c***             names to be loaded.

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER*10  Vtoggle1
      INTEGER*4     Count

      Vtoggle1 = 'VTOGGLE0  '
      CALL Load Toggle Values(Vtoggle1, 10, Count, Varnam, 0)
      Vtoggle1 = 'VTOGGLE1  '
      CALL Load Toggle Values(Vtoggle1, 10, Count, Varnam, 0)
      Vtoggle1 = 'VTOGGLE2  '
      CALL Load Toggle Values(Vtoggle1, 10, Count, Varnam, 0)
      Vtoggle1 = 'VTOGGLE3  '
C*** Added 11/9/89 to provide for an additional blank variable 
C*** variable name.
      Varnam(Count+1) = '          '
      CALL Load Toggle Values(Vtoggle1, 10, Count+1, Varnam, 0)
      Vtoggle1 = 'VTOGGLE4  '
      CALL Load Toggle Values(Vtoggle1, 10, Count+1, Varnam, 0)
   
      RETURN
      END
c**********************************************************************
c        F i l e   S e t   U p		       (subroutine)
c**********************************************************************
      SUBROUTINE File Set Up (Error, Nvar temp)

c*** This routine prompts the user for a filename using the routine 
c*** 'Get File Name'.  If the file exists then a call to 'Get Data' 
c*** reads the data.  The Variable box is cleared and the new variables
c*** are displayed.  The appropriate toggle fields are 're'loaded.
c*** If the file does not exist then a warning is issued and control
c*** returned to the calling subroutine.
c***
c*** VARIABLES:
c*** Error     - Returns .true. if error occurs in opening or reading
c***             Geo-EAS data file.
c*** Nvar temp - Returns the number of variables in the Geo-EAS data file.
c*** Infil     - Global variable containing the name of the Geo-EAS input
c***             data file.

       include 'dataprep.dec'
       include '../common/screen.str'

      LOGICAL     Exist, Error
      INTEGER*4     Nvar temp

      Error = .FALSE.
c***  concatenate prefix to filename ***
      CALL Build Prefix Filename (Infil)
      INQUIRE (FILE = Prefile, EXIST = Exist)
      IF (.NOT. Exist) THEN
	 Error = .TRUE.
	 CALL Flash (20, 2,
     +   'ERROR - No such File Exists ... (Press any key)')
      ELSE
	 CALL Get Header Data(Infil, Error, Nvar temp)
	 IF (.NOT. Error) THEN
	    CALL Load All Toggle Fields (Nvar temp)
	 ENDIF
      ENDIF
   
      RETURN
      END
c**********************************************************************
c        W r i t e   t o   O u t p u t   F i l e     (subroutine)
c**********************************************************************
      SUBROUTINE Write to Output File 

c*** This subroutine prompts the user for the name of an output
c*** file.  If the file does not exist, one is opened.  The data
c*** is then written to that output file.  The source of the info
c*** for the output file is as follows:
c***      Varnam  - global character array containing the variable 
c***                names.
c***      Units   - global character array containing the measurement
c***                descriptions.
c***      Formats - global character array containing the output format
c***                of each varible.
c***      Scratch File 1 contains the sample data.
c***
c*** VARIABLES:
c*** Outfil   -	 This variable contains the name of the Geo-EAS
c***             output data file.
c*** Exist    -  Logical that indicates if file exists.
c*** Formats  -  Global character array containing the output format 
c***             of each variable.
c*** Fmt      -  Global character array which is used as a format
c***             specifier for sample data output.
c*** Nvar     -  Global variable containing the number of variables.
c*** Vector48 -  Global array which contains up to 48 real values.

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER  Writemess*16
      LOGICAL  Exist
      INTEGER*4  I, J, K, L
      REAL*4     Get Element

      Writemess = 'Writing data ...'
      Exist = .FALSE.
c***  Concatenate prefix(if any) to file name.
c***  The global variable 'Prefile' will contain the prefix + filename.
      CALL Build Prefix Filename (Outfil)
c***  If file exists then open it else create a new file.
      INQUIRE (FILE = Prefile, EXIST = Exist)
      IF (Exist) THEN
 	 OPEN (UNIT = 11, FILE = Prefile, ERR = 999, STATUS = 'OLD')
      ELSE
	 IF (Outfil .EQ. '              ') THEN
	   CALL Flash (20,2,
     +          'ERROR - Blanks not allowed...(press any key)')
	   RETURN
	 ELSE
            OPEN (UNIT = 11, FILE = Prefile, ERR = 999, STATUS = 'NEW')
	 ENDIF
      ENDIF
c***  Display 'writing' message.
      CALL Put Str (Writemess, 16, 20, 2, 2)
c***  Print file heading 
      WRITE (11, '(A)',ERR = 999) Title
c***  Print number of variables
      WRITE (11, '(I5)',ERR = 999) Nvar
c***  Print variable name, measurement description and output format for
c***  each variable.
      DO 20 I = 1, Nvar
        WRITE (11, 25,ERR=999) Varnam(I), Units(I), Formats(I)
  25    FORMAT(2A10,A5)

  20  CONTINUE
c***  Construct the array 'Fmt'	which is the format specifier used for
c***  for output.
      Fmt(1:1) = '('
      J = 1
      DO 10 I = 1, Nvar - 1
	K = LENSTR (Formats(I))
	DO 15 L = 1, K
	  J = J + 1
	  Fmt(J:J) = Formats(I)(L:L)
  15	CONTINUE
        J = J + 1
        Fmt(J:J+3) = ',1X,'
	J = J + 3
  10  CONTINUE
      K = LENSTR (Formats(Nvar))
      DO 11 L = 1, K
        J = J + 1
	Fmt(J:J) = Formats(Nvar)(L:L)
  11  CONTINUE
      J = J + 1
      Fmt(J:J) = ')'

c***  The following do loop reads all sample data records from 
c***  Scratch File 1 and writes them to the output file.
      DO 30 I = 1, Ndata
c***    The following do loop extracts one record of sample data and
c***    assigns it to the array 'Vector48'.
	DO 40 J = 1, Nvar
           Vector48(J) = Get Element(1,J,I,Nvar)
  40	CONTINUE
c***    Write the sample data record to the output file.
	Write(11,Fmt,ERR=999) (Vector48(K), K = 1, Nvar)
  30  CONTINUE

c***  Erase message line.
      CALL Erase (20, 1, 78)
c***  Close output file.
      CLOSE (11)
      CALL Flash (20, 2,
     + 'Data has been sent to output file...(press any key)')
      RETURN

 999  CALL Beep
      CLOSE (11)
      CALL Flash (20, 2,
     +   'ERROR - writing data file ... (Press any key)')
      RETURN

      END
c**********************************************************************
c        R e a d   N u m e r i c a l   D a t a       (subroutine)
c**********************************************************************
      SUBROUTINE Read Numerical Data (Unitnmbr,Filename,Nvar temp,
     +                                Nvar file,Error,Recstart)

c***  Reads in only the samples of the variables from the Geo-EAS input
c***  data file whose name is contained in 'Filename' and contains this
c***  data to the Scratch File specified by the 'Unitnmbr'.
c***  The Unit number, Infil, Nvar temp, Ndata, Recskip must have been
c***    specified. 
c***
c***  VARIABLES:
c***  Unitnmbr  - unit number represents the 'data' matrix simulated by
c***              scratch file, Unit number = 1 for 'data1' and
c***              Unit number = 2 for 'data2'.
c***  Filename  - Name of file from which to read data.
c***  Nvar temp - number of variables in Geo-EAS input data file.
c***  Nvar file - number of variables to be contained in the Scratch file.
c***  Error     - logical that is .true. if a data file could not be
c***              accessed or read from or if a Scratch File could not be
c***              written.
c***  Recstart  - global variable, contains the number of the first record 
c***              that contains	sample data.
c***  Prefile   - global variable, contains the filename preceded by a path
c***              if the path was specified.
c***  Infil     - global variable, contains the filename only, no path.
c***  Ndata     - global variable, contains the number of samples in the file.
c***  Vector48  - global variable, declared as 'real*4 Vector48(48)'.
 
       include 'dataprep.dec'
       include '../common/screen.str'

      INTEGER*4   Nvar temp, I, N, J, Unitnmbr, Nvar file
      LOGICAL   Error, force
      CHARACTER   Filename*(*), Ch*1, Readmess*16

      Error = .FALSE.
      Readmess = 'Reading Data ...'
      CALL Put Str (Readmess, 16, 20, 2, 2)
      CALL Build Prefix Filename (Filename)
c***  Open data file. ***
      OPEN (UNIT = 10, FILE = Prefile, ERR = 899)
c***  Skip over to first sample data record.
      DO 15 I = 1, Recstart - 1
        READ (10, '(A)', END = 999,ERR=999) Ch
  15  CONTINUE

c***  Get data
c***  N tallies the number of sample data records.
      N = 1

c***  A sample data record is assigned to the array 'Vector48'.
  30  READ (10,*,END=40,Err=999) (Vector48(I),I=1,Nvar temp) 

c***  The following DO 35 loop contains the sample data record 
c***  of a Geo-EAS data file into a Scratch File.
      DO 35 J = 1, Nvar temp
	I = J + (Nvar file * (N-1)) 
	WRITE(Unitnmbr,REC=I,ERR=699) Vector48(J)
  35  CONTINUE
      N = N + 1

c***  If the next sample data record to be read exceeds the maximum
c***  number of sample data records permitted then display a warning
c***  and cease reading any more records.
      IF (N .GT. MAXDATA) THEN
	CALL Beep
	CALL Flash (20, 2, 
     +     'Warning - Too many samples in data file...(press any key)')
	GOTO 40
      ENDIF
      GOTO 30

  40  CONTINUE
      Ndata = N - 1
      CLOSE (10)
      RETURN

c***  The following are specific error messages.

  699 CALL Beep
      Error = .TRUE.
      CLOSE (10)
      CALL Close Scratch Files
      CALL Flash (20, 2,
     +'ERROR-not enough space to write scratch file...(Press any key)')
      RETURN

  899 CALL Beep
      Error = .TRUE.
      CLOSE (10)
      CALL Close Scratch Files
      CALL Flash (20, 2,
     +     'ERROR - Error opening data file...(Press any key)')
      RETURN

 999  if (force) goto 40
      CALL Beep

c***  Clear message line.
      CALL Erase (20, 1, 78)
      CALL Flash (20, 2, 
     +      'Error - reading data file...(press any key)')

      CALL putstr ('Type "c" to continue anyway:     ',33,20, 2,0)
      call getthekey(i)
      if (char(i) .eq. 'c' .or. char(i) .eq. 'C') then
         CALL putstr ('Type "f" to suppress error report:',33,20, 2,0)
         call getthekey(i)
         if (char(i) .eq. 'f' .or. char(i) .eq. 'F') force = .true.
         error = .false.
         CALL Erase (20, 2, 75)
         goto 40
      end if

      Error = .TRUE.
      CLOSE (10)
      CALL Close Scratch Files

      RETURN
      END
C**********************************************************************

