clarge
c**********************************************************************
c*** File name: PREP5.FOR (Source code for the program DATAPREP)
c**********************************************************************
c        C o n v e r t  I n t  t o  C h a r  F o r m a t  (subroutine)
c**********************************************************************
      SUBROUTINE Convert Int to Char Format(Fwidth, Fdecimals, Col)

c***  This routine converts the integer values in the variables 
c***  Fwidth and Fdecimals into a FORTRAN output format.  This format
c***  is then inserted into the global 'Formats' array element 
c***  corresponding to the variable indicated by 'Col'. 
c***    
c***  VARIABLES:
c***  Fwidth	- width of the FORTRAN output format.
c***  Fdecimals - number of decimals to be included in the output format. 
c***  Col	- variable (column) identifier.
c***  W1	- temporary variable equivalent to Fdecimals.
c***  F1	- temporary variable equivalent to Fwidth.
c***  Formats	- global array containing the output formats for each
c***              variable.

       include 'dataprep.dec'

      INTEGER*4 Col, Fwidth, Fdecimals, I

c***  Determine if format entered by user is within legal range.
c***  If not then issue warning and use G15.7 by default,
c***  else use the format specified by the user.  An F2.0 format or
c***  less always produces asterisks.
      W1 = Fwidth
      F1 = Fdecimals
      IF ((W1 .GT. 15).OR.(F1 .GT. 7).OR.(W1 .LT. (F1+1)).OR.
     +    (W1 .LT. 2).OR.(F1 .LT. 0)) THEN
	CALL Flash(20, 2, 
     +      'Illegal format, G15.7 used by default...(press any key)')
	Fwidth = 15
	Fdecimals = 7
      ENDIF
c***  Convert the format output width and decimal variables(integer) to
c***  character type and insert into the array 'Formats'.  The 
c***  formats always use the G format.

      Formats(Col) = '          '
      Formats(Col)(1:1) = 'G'
c***  If width field is < than 9, process. (Eg. G9.6)
      IF (Fwidth .LT. 10) THEN
	Formats(Col)(2:2) = CHAR(ICHAR('0') + Fwidth)
	Formats(Col)(3:3) = '.'
	Formats(Col)(4:4) = CHAR(ICHAR('0') + Fdecimals)
c***  If width field is > than 9, process. (Eg. G15.6)
      ELSE
	I = INT(Fwidth/10)
	Formats(Col)(2:2) = CHAR(ICHAR('0') + I)
	I = MOD (Fwidth, 10)
	Formats(Col)(3:3) = CHAR (ICHAR('0') + I)
	Formats(Col)(4:4) = '.'
	Formats(Col)(5:5) = CHAR(ICHAR('0') + Fdecimals)
      ENDIF
      RETURN
      END
c**********************************************************************
c	 I n p u t  F o r m a t  C h e c k     (subroutine)
c**********************************************************************
      SUBROUTINE Input Format Check(Col)

c***  This routine checks if the format listed in the input file is
c***  acceptable.  The acceptable formats are 'F' or 'G' where the
c***  width field is not greater than 15 and the decimal positions is
c***  not greater than 7.  The width field cannot fall below (decimal  
c***  positions + 1) and the decimal positions cannot fall below 0.  
c***  If they do then the default format of G15.7 is used.
c***
c***  VARIABLES:
c***  Col	- contains an integer value that indicates the
c***              variable (column) index. 
c***  Fwidth	- contains the width field of the output format.
c***  Fdecimals - contains the decimal field of the output format.
c***  Formats	- global array that contains the output formats for
c***              each variable.
c***  C1,C2,C3	- temporary variables that contain one character each.


       include 'dataprep.dec'

      CHARACTER*1  C1, C2, C3
      INTEGER*4    Col
      INTEGER*4    Fwidth, Fdecimals

      Fwidth = 0
      Fdecimals = 0
c***  Assign either a 'G' or an 'F' for the format.
      IF ((Formats(Col)(1:1) .EQ. 'F').OR.(Formats(Col)(1:1).EQ. 'f'))
     +   THEN 
            Formats(Col)(1:1) = 'F'
      ELSEIF((Formats(Col)(1:1).EQ.'G').OR.(Formats(Col)(1:1).EQ.'g'))
     +   THEN
            Formats(Col)(1:1) = 'G'
      ENDIF

c***  If the first character of the format if not a 'G' or 'F' then
c***  assign the default format of 'G15.7' for the variable in question
c***  and return to the calling subroutine.
      IF(.NOT.((Formats(Col)(1:1).EQ. 'F').OR.(Formats(Col)(1:1).EQ.'f')
     +   .OR.(Formats(Col)(1:1).EQ.'G').OR.(Formats(Col)(1:1).EQ.'g')))
     +   THEN
            Formats(Col) = 'G15.7     '
	    RETURN
      ELSE
	 IF (Formats(Col)(3:3).EQ.'.') THEN
c***        Process format with a one digit width value, eg., G9.5.
     	    C1 = Formats(Col)(2:2)
	    C2 = Formats(Col)(4:4)
c***        Make sure that the numeric characters are in fact numeric and
c***        not another type character else assign the default format
c***        and return to the calling subroutine.
	    IF ((C1.LT.'0').OR.(C1.GT.'9').OR.
     +          (C2.LT.'0').OR.(C2.GT.'9'))THEN
	        Formats(Col) = 'G15.7     '
		RETURN
	    ELSE
c***           Convert character values to integer values.
 	       Fwidth =  ICHAR(Formats(Col)(2:2)) - ICHAR('0')
	       Fdecimals = ICHAR(Formats(Col)(4:4)) - ICHAR('0')
	    ENDIF
	 ELSE IF (Formats(Col)(4:4).EQ. '.') THEN
c***        Process format with a two digit width value, eg., G15.7.
	    C1 = Formats(Col)(2:2)
	    C2 = Formats(Col)(3:3)
	    C3 = Formats(Col)(5:5)
c***        Make sure that the numeric characters are in fact numeric and
c***        not another type character else assign default format and
c***        return to calling subroutine..
	    IF ((C1.LT.'0').OR.(C1.GT.'9').OR.
     +	        (C2.LT.'0').OR.(C2.GT.'9').OR.
     +	        (C3.LT.'0').OR.(C3.GT.'9'))THEN
	        Formats(Col) = 'G15.7     '
		RETURN
	    ELSE
c***           Convert character values to integer values.
	       Fwidth = ICHAR(Formats(Col)(2:2)) - ICHAR('0')
	       Fwidth = ICHAR(Formats(Col)(3:3)) - ICHAR('0') +
     +               (10 * Fwidth)
	       Fdecimals = ICHAR(Formats(Col)(5:5)) - ICHAR('0')
	    ENDIF
         ELSE
c***        If the format has neither a one or two digit width field
c***        then assign the default value of 'G15.7' to the 'Formats'
c***        array and return to the calling subroutine.
	    Formats(Col) = 'G15.7     '
            RETURN
	 ENDIF
	 D1 = Fdecimals
	 W1 = Fwidth
c***     If the 'F' formats are used then check if formats are 
c***     in correct ranges.  F2.0 will produce asterisks.
	 IF ((Formats(Col)(1:1).EQ.'F').OR.(Formats(Col)(1:1).EQ.'f'))
     +      THEN
	      IF ((W1.GT.15).OR.(D1.GT.7).OR.(W1.LT.D1+1).OR.(W1.LT.2)
     +           .OR.(D1.LT.0))THEN
		   Formats(Col) = 'G15.7     '
	      ELSE
c***             Convert integers to character strings and replace the
c***             the 'G' inserted during the conversion with an 'F' since
c***             this was originally and 'F' format.
		 CALL Convert Int to Char Format(Fwidth, Fdecimals,Col)
		 Formats(Col)(1:1) = 'F'
	      ENDIF
c***     If the 'G' formats are used then check if formats are 
c***     in correct ranges.  G5.0 will produce asterisks.
	 ELSE IF((Formats(Col)(1:1).EQ.'G')
     +               .OR.(Formats(Col)(1:1).EQ.'g')) THEN
	      IF ((W1.GT.15).OR.(D1.GT.7).OR.(W1.LT.D1+1).OR.
     +            (W1.LT.6).OR.(D1.LT.0))THEN
		  Formats(Col) = 'G15.7     '
	      ELSE
c***              Convert integers to character strings.
		  CALL Convert Int to Char Format(Fwidth,Fdecimals,Col)
	      ENDIF
	 ENDIF
      ENDIF
      RETURN
      END
c**********************************************************************


