clarge
c**********************************************************************
c*** File name: PREP9.FOR (Source code for the program DATAPREP)
c**********************************************************************
c        C o l u m n  E x t r a c t  P r o m p t s     (subroutine)
c**********************************************************************
      SUBROUTINE Column Extract Prompts

c***  This routine extracts those variables(and sample data) selected
c***  by the user and stores them in a Geo-EAS data file.  This
c***  routine constructs the Column Extract screen and displays
c***  the Column Extract menu.  The user is prompted for a menu 
c***  selection which is then processed.  
c***
c***  VARIABLES:
c***  Menu	   - contains the field name of the Column Extract menu. 
c***  Menu Choice  - contains the user's menu selection.
c***  Nvar	   - contains the number of variables in the Geo-EAS output
c***                 data file.
c***  Nvar2	   - contains the number of variables in the Geo-EAS input
c***                 data file.  
c***  Vardone	   - logical that is true if the Variable List option
c***                 has been exercised.
c***  Var	   - contains the variable(column) number of a variable
c***                 to be extracted.
c***  Toggle Field - contains the field name of the toggle field used to
c***                 select the variable.
c***  Infil	   - global variable that contains the name of the Geo-EAS
c***                 input data file.
c***  Outfil	   - global variable that contains the name of the Geo-EAS
c***                 output data file.
c***  Recskip	   - contains the number of the first sample data record in
c***                 the Geo-EAS input data file.
c***  Varnam	   - global array that contains the variable names.
c***  Units	   - global array that contains the measurement descriptions
c***                 of each variable.
c***  Formats	   - global array that contains the output formats for each
c***                 of the variables.
c***  Varnam2	   - same as 'Varnam' but is used as a temporary array.
c***  Units2	   - same as 'Units' but is used as a temporary array.
c***  Formats2	   - same as 'Formats' but is used as a temporary array.
c***  Startcol	   - global array which maintains the variable(column)
c***                 of the variables to be extracted.

       include 'dataprep.dec'
       include '../common/screen.str'
		   
      CHARACTER   Menu Choice*20, Menu*10, Blank*14, File*8
      CHARACTER   Toggle Field*8, Label0*16, Readmess*17, Label4*23
      CHARACTER   Label1*11,Label2*12,Label3*20,Oldfile*14
      CHARACTER   Label6*30, Label5*17, Choice*10, Prompt*46
      INTEGER*4   Var Count, FLD Index, Var, Toggle, J, Key
      REAL*4      Value, Get Element
      LOGICAL   Vardone, Yes, Get Yes No, Error

c***  Erase the File Operations screen.
      CALL Erase Message Divider
      CALL ClearRows(1,18)
      CALL Draw Middle Horizontal Line(4)
      CALL Draw Middle Horizontal Line(6)
      CALL Draw Middle Horizontal Line(9)
      Blank = '              '
      Menu  = 'CE MENU   '
      Vardone = .FALSE.
      Error = .TRUE.

c***  Initialize and display labels upon the screen.
      Readmess = ' Processing data '
      Label0 = 'Output Variables'
      Label1 = 'Input File:'
      Label2 = 'Output File:'
      Label3 = 'Variable to extract:'
      Label4 = 'COLUMN EXTRACT FUNCTION'
      Label5 = 'Data File Prefix:'
      Label6 = 'Total Variables in input file:'
      CALL Put Str (Prefix, 55,  3, 20, 0)
      CALL Put Str (Label0, 16, 10, 31, 0)
      CALL Put Str (Label1, 11,  5,  2, 0)
      CALL Put Str (Label2, 12,  5, 39, 0)
      CALL Put Str (Label3, 20,  8, 39, 0)
      CALL Put Str (Label4, 23,  2, 29, 0)
      CALL Put Str (Label5, 17,  3,  2, 0)
      CALL Put Str (Label6, 30,  8,  2, 0)
c***  Initialize and display file names.
      CALL Set Character Value ('INFIL1C ', Infil)
      CALL Set Character Value ('OUTFIL1C ', Outfil)
cccc      CALL Display One Field('INFIL1C ')
cccc      CALL Display One Field('OUTFIL1C')
c***  Display Column extract menu and prompt user for choice ***
      CALL Put Menu (Menu)

  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Quit                ') THEN
c***    Erase the Column Extract screen.
	CALL Erase Middle Horizontal Line(4)
	CALL Erase Middle Horizontal Line(6)
	CALL Erase Middle Horizontal Line(9)
	CALL Clear Rows(1,18)
c***    Erase the variables heading.
	CALL Erase (1, 35, 9)
  	RETURN
      ELSE IF (Menu Choice .EQ. 'Files               ') THEN
	CALL Erase (8, 60, 15)
        CALL Erase Variable List
	Nvar = 0
c***    Initialize and display the number of variables in the Geo-EAS
c***    input data file.
	Choice = 'VAR TOTAL '
        CALL Set Integer Value (Choice, Nvar)
        CALL Display One Field (Choice)
c***    Initialized vardone to false to be used later in determining if
c***    variable list option was accessed before execution option.
	Vardone = .FALSE.
c***    Prompt user for input file ***
  	File = 'INFIL1C '
	CALL Get Name of File(File,Infil,Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    Prompt user for outfile.
	File = 'OUTFIL1C'
	CALL Get Name of File(File, Outfil, Error,2)
      ELSE IF (Menu Choice .EQ. 'Variables           ') THEN
c***    If legitimate files have been specified by the user then proceed
c***    with the data processing else issue an error message and return
c***    to the menu.
	IF (.NOT. Error) THEN
c***      Indicate the variable list has been accessed before 
c***      the execution option.
	  Vardone = .TRUE.
	  Oldfile = Infil
c***      Read title, number of variables, and variable names from
c***      the Geo-EAS input data file and store in Scratch File 1.
	  CALL File Set Up (Error, Nvar2)
	  Recskip = Nvar2 + 3
c***      If no error in reading header list of input file then load 
c***      toggle fields.
          IF (.NOT. Error) THEN
	    Choice = 'VAR TOTAL '
            CALL Set Integer Value (Choice, Nvar2)
            CALL Display One Field (Choice)
 	    CALL Load All Toggle Fields (Nvar2)
	    CALL Erase Variable List
c***        Copy data of input file to Second data structure set.
	    DO 30 I = 1,Nvar2
	      Varnam2(I) = Varnam(I)
	      Units2(I)  = Units(I)
	      Formats2(I) = Formats(I)
  30        CONTINUE
c***        Prompt user for the list of variables to be extracted.
c***        The following is a loop which is exited when either the
c***        user indicates he wishes no more variables, or if the
c***        maximum number of variables(MAXVAR) has been exceeded.
	    Yes = .TRUE.
	    Var Count = 0
  20	    IF (Yes) THEN
c***          Increment variable counter.
	      Var Count = Var Count + 1
c***          Prompt user for variable from toggle field.
	      Toggle Field = 'VTOGGLE3'
	      Var = Toggle (Toggle Field) + 1
c*** Added next line on 11/9/89 to check for blank variable name in toggle
c*** field.
              IF (Var .EQ. (Nvar2+1)) THEN
                Var Count = Var Count - 1
                GOTO 25
              ENDIF
c***          Insert variable number into array that tracks those
c***          variables to be extracted.
	      Startcol(Var Count) = Var
	      Varnam(Var Count) = Varnam2(Var)
	      Units(Var Count) = Units2(Var)
	      Formats(Var Count) = Formats2(Var)
c***          Display the additional variable name on the screen.
CCCC	      CALL Display Chosen Variables(Var Count)
	      CALL Display Variables(Var Count)

	      IF (Var Count .EQ. MAXVAR) THEN
	        CALL Flash(20, 2, 
     +  'WARNING - Variable maximum has been reached...(press any key)')
	      ELSE
C Changed 9/22/89 to require am explicit "N" answer to exit the option
C	        Yes = Get Yes No
C     +	      (20, 2, 'Do you wish to include another variable? <Y/N>')
 25            Prompt = 'Do you wish to include another variable? <Y/N>'
                 CALL Put Str (Prompt, 46, 20, 2, 0)
                 CALL getthekey (Key)
                 IF (Key .NE. 110 .AND. Key .NE. 78) THEN
                    Yes = .TRUE.
                 ELSE
                    Yes = .FALSE.
                 ENDIF 
                 CALL Erase (20, 2, 46)
                 CALL Increment Variable Toggle Position (Toggle Field,
     +                                                   Var, Nvar2+1)
	         GOTO 20
	      ENDIF
            ENDIF
          ENDIF
	ELSE
c***    Inform user that legitimate filenames have not been provided 
c***    and return to the menu. 
	   CALL Beep
	   CALL Flash(20, 2, 
     +     'ERROR - Filenames must be entered...(press any key)')
        ENDIF
      ELSE IF (Menu Choice .EQ. 'Execute             ') THEN
c***    If filenames have not been entered then inform the user and return
c***    to the menu.
	IF (Error) THEN
	  CALL Beep
	  CALL Flash(20, 2, 
     +       'ERROR - Filenames must be entered...(press any key)')
	  GOTO 10
c***    Check if variable list option's infile is same as 
c***    file option's infile.  This check is to determine if the
c***    variable list option has been opted before the execution process.
	ELSE IF ((Oldfile .NE. Infil).OR.(.NOT. Vardone)) THEN
	  CALL Beep
	  CALL Flash(20, 2, 
     +     'ERROR - Choice of variables not provided...(press any key)')
	  GOTO 10
	ENDIF
        CALL Put Str (Readmess, 17, 20, 2, 2)
        Vardone = .FALSE.
c***    Open scratch file to contain 'Data2' 'structure'. ***
        CALL Open Scratch Files
c***    Read the samples of the input file.
cccc        CALL Read Numerical Data(1,Infil,Nvar2,Nvar2,Error,Recskip)
        CALL Read Numerical Data(2,Infil,Nvar2,Nvar2,Error,Recskip)
c***    If no error in reading 'sample' data then 
cccc***    Copy the samples over to the second data structure set.
	IF (.NOT. Error) THEN
          CALL Put Str (Readmess, 17, 20, 2, 2)
 	  Ndata2 = Ndata
cccc	  DO 35 I = 1,Nvar2
cccc	    DO 40 J = 1,Ndata2
cccc	      Value = Get Element(1,I,J,Nvar2)
cccc	      CALL Put Element(2,I,J,Nvar2,Value)
cccc  40	    CONTINUE
cccc  35	  CONTINUE
c***      Copy over those samples that are to be extracted over to the
c***      Scratch File 1 which is used by the Write to Output File routine.
	  DO 50 J = 1,Var Count 
	    DO 45 I = 1, Ndata2
              Value = Get Element(2,Startcol(J),I,Nvar2)
              CALL Put Element(1,J,I,Var Count,Value)
  45        CONTINUE
  50	  CONTINUE
c***      Set global variable counter to actual number of variables
c***      to be stored into the output file.
	  Nvar = Var Count
	  CALL Write to Output File 
        ENDIF
        CALL Close Scratch Files
      ENDIF

      GOTO 10
      RETURN
      END
c**********************************************************************
c        M e r g e   P r o m p t s     (subroutine)
c**********************************************************************
      SUBROUTINE Merge Prompts

c***  This routine constructs the Merge screen and displays the Merge
c***  menu.  The user is prompted for a menu selection.  The selection
c***  is then processed.
c***
c***  VARIABLES:
c***  Menu	   - contains the field name of the Merge menu.
c***  Menu Choice  - contains the user's menu selection.
c***  Infil	   - global variable which contains the Geo-EAS input
c***                 data file.
c***  Outfil	   - global variable which contains the Geo-EAS output
c***                 data file.
c***  Error	   - logical which is true if a Geo-EAS data file cannot
c***                 be accessed.
c***  Infile1	   - global variable which contains the name of the 
c***                 Geo-EAS input data file to be merged with Infile2.
c***  Infile2	   - global variable which contains the name of the 
c***                 Geo-EAS input data file to be merged with Infile1.
c***  Outfile1	   - global variable which contains the name of the 
c***                 Geo-EAS output data file whose contents are the
c***                 merged Infile1 and Infile2.

       include 'dataprep.dec'
       include '../common/screen.str'
		   
      CHARACTER   Menu Choice*20, Menu*10, File*8
      CHARACTER   Readmess*17
      CHARACTER   Label1*28,Label2*28, Label3*28,Label4*14,Label5*17
      CHARACTER   Label6*16
      INTEGER*4   FLD Index
      LOGICAL   Error

c***  Erase the File Operations screen.
      CALL Erase Message Divider
      CALL ClearRows(1,18)
      CALL Draw Middle Horizontal Line(5)
      CALL Draw Middle Horizontal Line(9)

      Menu  = 'MG MENU   '
      Error = .TRUE.

c***  Initialize and display labels.
      Readmess = ' Processing data '
      Label1 = 'Name of file 1 to be merged:'
      Label2 = 'Name of file 2 to be merged:'
      Label3 = 'Name of output file        :'
      Label4 = 'MERGE FUNCTION'
      Label5 = 'Data File Prefix:'
      Label6 = 'Output Variables'
      CALL Put Str (Prefix, 55, 4,20, 0)
      CALL Put Str (Label1, 28, 6, 2, 0)
      CALL Put Str (Label2, 28, 7, 2, 0)
      CALL Put Str (Label3, 28, 8, 2, 0)
      CALL Put Str (Label4, 14, 2,33, 0)
      CALL Put Str (Label5, 17, 4, 2, 0)
      CALL Put Str (Label6, 16,10,31, 0)
c***  Initialize and display file names.
      CALL Set Character Value ('FILE3A ', Infil)
      CALL Set Character Value ('FILE4A ', Infile2)
      CALL Set Character Value ('OUTFILE3 ', Outfil)
cccc      CALL Display One Field('FILE3A  ')
cccc      CALL Display One Field('FILE4A  ')
cccc      CALL Display One Field('OUTFILE3')
c***  Display merge menu and prompt user for choice.
      CALL Put Menu (Menu)
  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Quit                ') THEN
	CALL Erase Middle Horizontal Line(5)
	CALL Erase Middle Horizontal Line(9)
	CALL Clear Rows(1,18)
c***    Erase the variables heading.
	CALL Erase (1, 35, 9)
  	RETURN
      ELSE IF (Menu Choice .EQ. 'Files               ') THEN
	CALL Erase Variable List
c***    Prompt user for file 1 to be merged.
  	File = 'FILE3A  '
	CALL Get Name of File(File, Infile1, Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    Prompt user for file 2 to be merged with file 1.
	File = 'FILE4A  '
	CALL Get Name of File(File, Infile2, Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    If input files are the same then issue message and return to menu.
	IF (Infile1 .EQ. Infile2) THEN
          CALL Flash (20, 2,
     +    'ERROR - Input file is same as output file...(Press any key)')
	  GOTO 10
	ENDIF
c***    Prompt user for the output file that the appended file is 
c***    written to.
	File = 'OUTFILE3  '
	CALL Get Name of File(File, Outfile1, Error,2)
c***    Assign filenames to global variables.
 	Infil  = Infile1
	Outfil = Outfile1
      ELSE IF (Menu Choice .EQ. 'Execute             ') THEN
c***    If error in input and output files then do not proceed with 
c***    Merge option.
	IF (Error) THEN
	  CALL Beep
          CALL Flash (20, 2,
     +       'ERROR - Filenames must be entered...(press any key)')
	ELSE
          CALL Put Str (Readmess, 17, 20, 2, 2)
          CALL Merge (Infile1, Infile2, Outfile1)
	ENDIF
      ENDIF

      GOTO 10
      RETURN
      END

c**********************************************************************
c        M e r g e     (subroutine)  
c**********************************************************************
      SUBROUTINE Merge (InfileA, InfileB, OutfileA)

c***  This routine merges the two files, InfileA and InfileB, and stores
c***  the results into OutfileA.
c***
c***  VARIABLES:
c***  InfileA	 - contains the name of file to be merged.  It is the primary
c***               file. 
c***  InfileB	 - contains the name of the file to be merged with InfileA.
c***  OutfileA	 - contains the name of the file whose contents are 
c***               InfileA and InfileB merged.
c***  Infil	 - global variable that contains the name of the Geo-EAS
c***               input data file.
c***  Outfil	 - global variable that contains the name of the Geo-EAS
c***               output data file.
c***  Same Var	 - contains the number of variable names that appear
c***               in both InfileA and InfileB.
c***  New Nvar	 - contains the number of variables that are in the Geo-EAS
c***               output data file.
c***  Common Var - logical is true if the variable name appears in both
c***               InfileA and InfileB.
c***  Error	 - logical is true if a file could not be accessed or read,
c***               or if non-unique variable names appear in the same file.
c***  Tndata	 - contains the number of sample data records in the Geo-EAS
c***               output data file.
c***  Var Count	 - contains the number of variables that are to stored into
c***               Geo-EAS output data file.
c***  Ndata      - global variable that stores the number of sample records
c***               in InfileA ie., file 1.  This value is assigned in the
c***               subroutine READ NUMERICAL DATA. 
c***  Ndata2     - global variable that stores the number of sample records
c***               in InfileB ie., file 2.
c***  File2st    - stores the record number at which to start with when
c***               adding InfileB(file 2) records to InfileA(file 1). 
c***  File2rec   - stores the current record number being extracted
c***               from Scratch File 2 (InfileB's contents).

       include 'dataprep.dec'
       include '../common/screen.str'
	
      CHARACTER*14  InfileA, InfileB, OutfileA, Readmess*17
      INTEGER*4     Tndata, Var Count, I, J, K, T
      INTEGER*4     New Nvar, Same Var, File2st, File2rec
      REAL*4        Get Element, Value
      LOGICAL     Error, Common Var

      Readmess = ' Processing data '
c***  Read in file 2 ie., the file to append.
c***  set global variable Infil = Infile2.
      Infil = InfileB
      CALL Open Scratch Files
      CALL File Set Up (Error, Nvar2)
c***  Check for non-unique variable names within InfileB
      CALL Check for Same Name Variables(Error, Nvar2, 2)
      Recskip = Nvar2 + 3
      IF (.NOT. Error) 
c***    Read sample data from the Geo-EAS input data file and store
c***    in Scratch File 2. 
cccc     +  CALL Read Numerical Data(1,Infil,Nvar2,Nvar2,Error,Recskip)
     +  CALL Read Numerical Data(2,Infil,Nvar2,Nvar2,Error,Recskip)
      IF (.NOT. Error) THEN
        CALL Put Str (Readmess, 17, 20, 2, 2)
c***    Copy file 2 data over to file 2 data structures.
        DO 30 I = 1,Nvar2
          Varnam2(I) = Varnam(I)
          Units2(I)  = Units(I)
          Formats2(I) = Formats(I)
  30    CONTINUE
        Ndata2 = Ndata
cccc        DO 35 I = 1,Nvar2
cccc          DO 40 J = 1,Ndata2
cccc	     Value = Get Element(1,I,J,Nvar2)
cccc	     CALL Put Element(2,I,J,Nvar2,Value)
cccc  40      CONTINUE
cccc  35    CONTINUE
      ENDIF
c***  Read in file 1 ie., the file to be appended and retain info 
c***  in file 1 data structures. 
c***  Set global variable Infil = Infile1.
      Same Var = 0
      IF (.NOT. Error) THEN
	Infil = InfileA
c***    Read title, variable total, and variable names.
	CALL File Set Up (Error, Nvar)
c***    Check for non-unique variable names within InfileA.
        CALL Check for Same Name Variables(Error, Nvar, 1)
c***    Check for variable names that exist in both InfileA and InfileB.
	IF (.NOT. Error) THEN
          CALL Put Str (Readmess, 17, 20, 2, 2)
	  DO 10 I = 1, Nvar
	    DO 15 J = 1, Nvar2
	      IF (Varnam(I) .EQ. Varnam2(J)) Same Var = Same Var + 1
  15	    CONTINUE
  10	  CONTINUE
c***      Calculate the number of unique variables, ie., variables that
c***      exist in only one and not both infiles and are also unique in 
c***      that one file. 
	  New Nvar = Nvar + Nvar2 - Same Var
	  IF (New Nvar .GT. MAXVAR) THEN
	    Var Count = 999
	    GOTO 85
	  ELSE
            Recskip = Nvar + 3
c***        Read sample data from InfileA.
            CALL Read Numerical Data(1,Infil,Nvar,New Nvar,
     +                               Error,Recskip)
 	  ENDIF
	ENDIF
      ENDIF

      IF (.NOT. Error) THEN
        CALL Put Str (Readmess, 17, 20, 2, 2)
c***    Initialize Tarray elements to .false. 
	DO 45 I = 1, MAXVAR
	  Tarray(I) = .FALSE.
  45	CONTINUE
c***    Set Tndata to the greater of Ndata or Ndata2.
	Tndata = MAX0 (Ndata, Ndata2)
c***    Set Tndata to the total number of records contained
c***    in both Geo-EAS input data files but not greater 
c***    than MAXDATA.  The variable 'T' represents the number of
c***    sample records to be included from InfileB.  
        IF (Same Var .GT. 0) THEN
          IF ( Ndata .GE. Ndata2 )THEN
	    Tndata = Ndata 
	    T = 0
	    File2st = 1
          ELSE
	    Tndata  = Ndata2
	    T       = Ndata2
            File2st = Ndata + 1
          ENDIF
        ENDIF

c***    Check if same variable exists in both files, if so then add
c**     file 2 variable to file 1 and set Tarray element representing
c**     that variable to true (indicates this variable exists in file 2).
	Var Count = 0
cccc	Equal Var = 0
	DO 50 I = 1, Nvar
	  Common Var = .FALSE.
	  DO 55 J = 1, Nvar2
	    IF (Varnam(I) .EQ. Varnam2(J)) THEN
	      Common Var = .TRUE.
	      Tarray(J) = .TRUE.
cccc	      Equal Var = Equal Var + 1
	      Var Count = Var Count + 1
c***          Add file 2 records to file 1, for the common variable but
c***	      only if file 2 has more records than file 1.
c***          K - increments the Ndata value to the next record.
c***          File2st - indicates the record at which to start extracting
c***          from Scratch File 2(InfileB).
              K = 0
	      DO 60 File2rec = File2st, T
		K = K + 1
	        Value = Get Element(2,J,File2rec,Nvar2)
	        CALL Put Element(1,I,Ndata+K,New Nvar,Value)
  60	      CONTINUE
 	    ENDIF
  55	  CONTINUE
c***      If file 1 variable is not in file 2 then assign missing 
c***      values to all records succeeding file 1's original records, 
c***      but only if file 2's samples exceeds file 1's samples.
	  IF (.NOT. Common Var) THEN
	    Var Count = Var Count + 1
            DO 65 K = Ndata + 1, Tndata
	      CALL Put Element(1,I,K,New Nvar,Missing)
  65	    CONTINUE
	  ENDIF
  50    CONTINUE
c***    Add all variables in file 2 that were not included in file 1.
	DO 70 I = 1, Nvar2
	  IF (.NOT. Tarray(I))THEN
c***        Var Count maintains the total distinct variables of 
c***        both files.
	    Var Count = Var Count + 1
	    IF (Var Count .LE. MAXVAR) THEN
	      Varnam(Var Count) = Varnam2(I)
	      Units(Var Count)  = Units2(I)
	      Formats(Var Count)= Formats2(I)
c***          Assign uncommon file2 var values to file1 from 1 to 
c***          Ndata2 records.
	      DO 75 K = 1, Ndata2
	        Value = Get Element(2,I,K,Nvar2)
	        CALL Put Element(1,Var Count,K,New Nvar,Value)
  75	      CONTINUE
c***          Assign Missing values to the uncommon file2 variable 
c***          from Ndata to Tndata records, only if file 1's records 
c***          exceeds file 2's.
	      DO 80 K = Ndata2 + 1, Tndata
		CALL Put Element(1,Var Count,K,New Nvar,Missing)
  80	      CONTINUE
	    ELSE
c***          If total variables exceed MAXVAR, then set Var Count = 999.
	      Var Count = 999
	      GOTO 85
	    ENDIF
	  ENDIF
  70    CONTINUE
	Nvar = Var Count
	Ndata = Tndata
c***    Write results of merged to the designated output file.
	IF (.NOT. Error) THEN
	  Outfil = OutfileA
	  CALL Display Variables(Nvar)
	  CALL Write to Output File 
	ENDIF
      ENDIF

  85  IF (Var Count .EQ. 999) THEN
        CALL Flash (20, 2,
     +     'ERROR - Too many variables in files...(Press any key)')
      ENDIF
      CALL Close Scratch Files
      RETURN
      END

c**********************************************************************


