cc$DEBUG
cc$LARGE
C*********************************************************************
C*** File Name: SCATPLOT.FOR (Source code for the program SCATTER)
C*********************************************************************
C SUBROUTINE Scattergram (TITLE, XLABEL, YLABEL, XARR, YARR,
C                         NDATA, XMIN, XMAX, YMIN, YMAX,
C		          REGRESSION, TRUE SCALE, DEVCODE) 
C
C Formal arguments:
C  TITLE        - Title for the graph.
C  XLABEL       - X axis label.
C  YLABEL       - Y axis label.
C  XARR         - Array of X coordinates to plot.
C  YARR         - Array of Y coordinates to plot.
C  NDATA        - Number of data points to plot.
C  XMIN, YMIN   - Minimum X and Y coordinates for the graph.
C  XMAX, YMAX   - Maximum X and Y coordinates for the graph.
C  REGRESSION   - Indicates whether linear regression is calculated.
C  TRUE SCALE   - Indicates whether to plot the graph in true scale.
C  DEVCODE      - Type of graphics device in use.
C
C This routine sets up the graphics, draws and labels the axes, 
C then plots the data.  If the regression option was selected
C the regression line is drawn.
C 

      SUBROUTINE Scattergram(Title, Xlabel, Ylabel, Xarr, Yarr,
     +     Ndata, Xmin, Xmax, Ymin, Ymax,
     +     Regression, True Scale, DevCode,
     &     file prefix)

      IMPLICIT INTEGER*4 (A-Z)
      PARAMETER (OTHER = 0, HERC = 1, CGA = 2, EGA = 3)

C Passed variables
	CHARACTER	Title*(*), Xlabel*(*), Ylabel*(*), message*60
	CHARACTER	toast*60
        INTEGER*4	Ndata, DevCode
	REAL*4		Xmin, Xmax, Ymin, Ymax
	REAL*4		Xarr (1), Yarr (1)
	LOGICAL		Regression, True Scale

C Local Variables
cc        REAL*4		Xfudge, Yfudge, Xstart, Z, Y
	REAL*4		Dx, Dy, Tx, Ty, Mx, My, X1, X2, Y1, Y2
	REAL*4		TxMin, TxMax, TyMin, TyMax, Txtinc, TyTinc
	REAL*4		TMinx, Tmaxx, Tminy, TmaxY, tXfudge, tYfudge
	REAL*4		Slope, Intercept, RSquared, Aspect, Factor
	REAL*4		Missing

	INTEGER*4	Gmode, Ierr
	INTEGER*4	Red, Green, Blue, White, Yellow
cc	INTEGER*4       XmaxDev, YmaxDev, DevMaxX, DexMaxY
	INTEGER*4       XmaxDev, YmaxDev, DevMaxX

        character menu choice*10, file prefix*(*)
 
cc        call system ("xmessage -message scattergram &")
cclong 11/30/93
        CALL New Screen (3)
        call put menu ('POSTSCR')
        call get character value('PSFILEN',message)
        if (lenstr(message) .eq. 0) 
     &       call set character value('PSFILEN','scatter.ps')
cclong 11/30/93
 
	DATA Missing /1.E31/

C *** Determine colors to use
        include '../common/color.dec'

        Factor  = 1
        XmaxDev = 1
        YmaxDev = 1

C *** Code for Square (corrected by pixel aspect ratio) Device Plotting Space
	Aspect 	= FLOAT (Xmaxdev) / FLOAT (Ymaxdev)
	DevMaxY	= YMaxDev
	DevMaxX = DevMaxY / Factor

C *** Calculate tick mark and max axis values
 1      message = 'Computing Scatter Plot...type "q" to quit.'
        i = lenstr(message)
        CALL PutStr (message,i, 20, 2, 2)
        IF (True Scale) THEN
          TYmin = 0.
	  TyMin = AMIN1 (Xmin, Ymin)
	  TyMax = AMAX1 (Xmax, Ymax)
	  CALL Neatax (TYmin, TYmax, TYtinc, K)
	  Txtinc = TYtinc		
	  TxMax  = TyMax        
	  TxMin  = TyMin        
       ELSE
          TYmin = Ymin
          TYMAX = Ymax
          TXMIN = Xmin
          TXMAX = Xmax
	  CALL Neatax (TYmin, TYmax, TYtinc, K)
	  CALL Neatax (TXmin, TXmax, TXtinc, K)
       ENDIF

C *** Calculate other scaling information
	dx 	= Txmax - txmin
	dy 	= tymax - tymin
	txfudge	= dx * 0.1
	tyfudge	= dy * 0.1
	tminx 	= txmin - 3*txfudge
	tmaxx 	= txmax + 7*txfudge
	tminy 	= tymin - 2*tyfudge
	tmaxy 	= tymax + 3*tyfudge
	Tx 	= (txMax - txMin) * .005
	Ty 	= (tyMax - tyMin) * .007
	mx 	= dx * .01
	my 	= dy * .01

C *** Start graphics mode
	CALL Zsetup (Gmode, Ierr)

C *** Set text attributes (8x8, white on black border)
	CALL Zlbdir (0.0)
	
C *** Set scaling window
	CALL Zscale (tMinx, tMaxx, tMiny, tMaxy)

C *** Draw graph frame
        CALL Grframe (0,0, Blue)

C *** Draw and label the axes
	CALL Zcolor (White)
	CALL Axes ('Scatter Plot', Xlabel, Ylabel, Title,
     +		   Txmin, Txmax, Tymin, Tymax, Txtinc, Tytinc, DevCode)

C *** Draw 45 degree line
	CALL Zcolor (Green)
	CALL ZMove (TxMin, TYmin)
	
C *** Plot scattergram 
	DO 50 I = 1, Ndata
	   Tx = Xarr(I)
	   Ty = Yarr(I)
	   IF (Tx .GE. Missing .OR. Ty .GE. Missing) GOTO 50
           CALL Plus (Tx, Ty, Mx, My, Green, White)
 50	CONTINUE

C *** Do regression line
	IF (Regression) THEN
	   CALL Regress (Xarr, Yarr, Ndata, Slope, Intercept, 
     +                   RSquared)
	   CALL Zcolor (Red)
           X1 = TxMin
           Y1 = Slope*TxMin+Intercept
           X2 = TxMax
           Y2 = Slope*TxMax+Intercept
	   CALL Plot Regline (X1, Y1, X2, Y2, 
     +                      TxMin, TxMax, TyMin, TyMax)
	   CALL PlotStat (DevCode, TxMin, TxMax, TyMin, TyMax,
     +		          Ndata, Slope, Intercept, RSquared)
	ENDIF

cclong 11/30/93
      CALL Erase(20,2,76)
      call get character value('PSFILEN',message)
      CALL Erase(5,10,60)
      call putstr(message(1:60),60,5,10,2)
      CALL Get Menu Choice ('POSTSCR',Menu choice)
      if (menu choice .eq. 'File') then
         CALL Get Field
         call get character value('PSFILEN',message)
         IF (lenstr(message) .EQ. 0) THEN
            Message =
     &           'Blank file name given - no can do!'
            CALL Erase (20, 2, 76)
            CALL Flash (20, 2,message)
            goto 1
         ENDIF
      else if (menu choice .eq. 'Quit') then
         goto 666
      end if
      i = lenstr(message)
      call putstr(message(1:60),60,5,10,2)
      j = lenstr(fileprefix)
      toast=fileprefix
      call turn_ps_on(toast(1:j)//message(1:i))
      goto 1
 666  call reset()
cclong 11/30/93
cc      call system ("xmessage -message scattergramout &")
 
      RETURN
      END

C**********************************************************************
C SBROUTINE PLOT REGLINE (X1,Y1,X2,Y2,TXMIN,TXMAX,TYMIN,TYMAX)
C
C Formal Arguments:
C  X1, Y1    -  Minimum point on the regression line.
C  X2, Y2    -  Maximum point on the regression line.
C  TXMIN     -  Minimum X coordinate of the graph.
C  TXMAX     -  Maximum X coordinate of the graph.
C  TYMIN     -  Minimum Y coordinate of the graph.
C  TYMAX     -  Maximum X coordinate of the graph.
C
C This routine call CLIP to plot the regression line, given the graph 
C limits graph limits and equation Y = Slope * X + Intercept.
C	
	SUBROUTINE Plot Regline (X1,Y1,X2,Y2,TxMin,TxMax,TyMin,TyMax)

	IMPLICIT INTEGER*4 (A-Z)

	REAL*4	X1, X2, Y1, Y2, TxMin, TxMax, TyMin, TyMax 

        REAL*4  TX1,TX2, TY1, TY2

C Clip the line within the rectangle
        TX1 = X1
        TX2 = X2
        TY1 = Y1
        TY2 = Y2

        CALL Clip (TX1, TY1, TX2, TY2, TxMin, TxMax, TyMin, TyMax)
      
	RETURN
	END 

C**********************************************************************
C SUBROUTINE PLOTSTAT (DEVCODE, TXMIN, TXMAX, TYMIN, TYMAX,
C     		       NDATA, SLOPE, INTERCEPT, RSQUARED)
C                                                  
C Formal Arguments:
C  DEVCODE       - The type of graphics device.
C  TXMIN,TYMIN   - Minimum X and Y coordinates of the graph.
C  TXMAX,TYMAX   - Maximum X and Y coordinates of the graph.
C  NDATA         - Number of data values.
C  SLOPE         - Slope of the regression line.
C  INTERCEPT     - Y-intercept of the regression line.
C  RSQUARED      - The regression correlation coefficient.
C
C This routine plots the  regression statistics for the plot.
C
	SUBROUTINE PlotStat (DevCode, TxMin, TxMax, TyMin, TyMax,
     +			     Ndata, Slope, Intercept, RSquared)

	IMPLICIT INTEGER*4 (A-Z)
        
        PARAMETER (OTHER = 0, HERC = 1, CGA = 2, EGA = 3)

        REAL*4     TxMin, TxMax, TyMin, Tymax
        REAL*4	   Slope, Intercept, RSquared
       
		          
	INTEGER*4  NData, DevCode

	CHARACTER  Ifmt*8, Rfmt*8, NumStr*10, NULCH*1
        CHARACTER ich*22, scratch*25
	REAL*4     X, Y, My 
	
        DATA	Ifmt /'(I10)'  /
	DATA	Rfmt /'(F10.3)'/
        
        NULCH   = '0'
        X    = TXmax + ABS (TXmax - TXmin) * .02
        My   = ABS (TYmax - TYmin) * .075
        Y    = TYmax - My 
        CALL Zlborg (INT2 (2))
        CALL Zmove (X, Y)
        ich = 'Regression Results'
        CALL ZLABEL (ich)
        Y = Y - My*1.5
        
        WRITE (Numstr,Ifmt) Ndata
        CALL Zmove (X, Y)
        scratch = 'N Pairs       :'//Numstr
        call zlabel(scratch)
        Y = Y - My*1.5
        
        WRITE (Numstr,Rfmt) Slope 
        CALL Zmove (X, Y)
        scratch = 'Slope         :'//Numstr
        call zlabel(scratch)
        Y = Y - My
        
        WRITE (Numstr,Rfmt) Intercept
        CALL Zmove (X, Y)
        scratch = 'Intercept     :'//Numstr
        call zlabel(scratch)
        Y = Y - My
        
        WRITE (Numstr,Rfmt) RSquared
        CALL Zmove (X, Y)
        scratch = 'Correl. coeff.:'//Numstr
        call zlabel(scratch)
	  
	END

C*************************************************************************
C SUBROUTINE CLIP (X1, Y1, X2, Y2, XMIN, XMAX, YMIN, YMAX)
C
C Formal Arguments:
C  X1,Y1       -  First point on the line.
C  X2,Y2       -  Second point on the line.
C  XMIN,YMIN   -  Minimum X and Y coordinates for the boundary area.
C  XMAX,YMAX   -  Maximum X and Y coordinates for the boundary area.
C
C This routine is for clipping lines it was taken from pages 133-134 of
C "COMPUTER GRAPHICS", by  Hearn and Baker.

        SUBROUTINE Clip (X1, Y1, X2, Y2, TxMin, TxMax, TyMin, TyMax)

        REAL*4  X1, Y1, X2, Y2, TxMin, TxMax, TyMin, TyMax

        REAL*4  u1, u2, dx, dy

        LOGICAL Cliptest
       
        u1 = 0.0
        u2 = 1.0
        dx = X2 - X1
        IF (Cliptest(-dx, X1-TxMin, u1, u2)) THEN
           IF (Cliptest(dx, TxMax-X1, u1, u2)) THEN
              dy = Y2 - Y1
              IF (Cliptest(-dy, Y1-TyMin, u1, u2)) THEN
                 IF (Cliptest(dy, TyMax-Y1, u1, u2)) THEN

C *** If u1, and u2 are within range of 0 to 1, use to calculate
C *** new line endpoints.
                    IF (u2 .LT. 1.0) THEN
                       X2 = X1 + u2 * dx
                       Y2 = Y1 + u2 * dy
                    ENDIF
                    IF (u1 .GT. 0.0) THEN
                       X1 = X1 + u1 * dx
                       Y1 = Y1 + u1 * dy
                    ENDIF
                 ENDIF
              ENDIF
           ENDIF
        ENDIF

        CALL Zmove (X1, Y1)
        CALL Zplot (X2, Y2)

        RETURN
        END
             
C**************************************************************************
C LOGICAL FUNCTION CLIPTEST (P,Q,U1,U2)
C
C Formal Arguments:
C  P,Q   - Coordinates of the line.
C  U1,U2 - The part of the line that lies within the boundary.
C
C This routine determines whether the line can be rejected or whether
C the intersection parameters are to be adjusted.  Taken from  pages
C 133-134 of "COMPUTER GRAPHICS", by Hearn and Baker.
C
        LOGICAL FUNCTION Cliptest (p,q,u1,u2)
         
        REAL*4  r,p,q,u1,u2

        LOGICAL Result

        Result = .TRUE.

C *** Line from outside to inside of this boundary
        IF (p .LT. 0.0) THEN
           r = q/p

C *** Reject line or update u1 if appropriate
          IF (r .GT. u2) THEN
             Result = .FALSE.
          ELSE IF (r .GT. u1) THEN
             u1 = r
          ENDIF

C *** Line from outside to inside of this boundary
        ELSE IF (p .GT. 0.0) THEN
           r = q/p

C *** Reject line or update u2 if appropriate
           IF (r .LT. u1) THEN
              Result = .FALSE.
           ELSE IF (r .LT. u2) THEN
              u2 = r
           ENDIF

C *** p = 0; line parallel to boundary
        ELSE
           IF (q .LT. 0.0) Result = .FALSE.
        ENDIF

        Cliptest = Result

        RETURN
        END

