cc$LARGE
C*********************************************************************
C*** File Name: SCATTER.FOR (Source code for the program SCATTER)
C*********************************************************************
C                      PROGRAM SCATTER
C
C Scatter produces scatter plots of variables in a Geo-EAS data file.
C Options allow for log and semi-log plots and for a regression line
C to be calculated.  Scaling and numeric tickmark lableing for the axes
C are computed automatically.
C
C The main program performs the following:
C   Reads in the system defaults
C   Sets up the screen
C   Calls Get Screen Fld to display the screens and process the
C    user's requests
C   Writes to the system default file
C
C**************************************************************************

        PROGRAM SCATTER

        INCLUDE 'scatter.cmn'

      	LOGICAL Exists
        character*80 geoeas_home
          
        include '../common/color.dec'

cc Note that the get_geoeas_home line needs to come first, or there
cc will be problems on some terminals.
cc    The program will speed up if this is your private copy, and you
cc want to replace the call by the following example:
cc    My geoeas_dir is
cc             /aplm3/ael/dmyers/dgeoeas/unix_version
cc so I could replace the lines below by 
cc        CALL Read Structure ('/aplm3/ael/dmyers/dgeoeas/unix_version'
cc     &       //'/scatter/scatter.scr')
cc
        call graphics_in
        call get_geoeas_dir(10,geoeas_home)
        CALL Read Structure (geoeas_home(1:lenstr(geoeas_home))
     &       //'/scatter/scatter.scr')
cc
        CALL Set up Screen()   
        CALL Clear Screen
        CALL Read System Defaults
        IF (Intro) CALL Intro Screen
        CALL Get Screen Fld
        CALL Write System Defaults
        CALL Clear Screen

        END

C************************************************************************
C SUBROUTINE INTRO SCREEN 
C
C  This routine displays the screen frame/labels/fields for the 
C  introductory screen.           
C
        SUBROUTINE Intro Screen

        CALL Clear Screen
        CALL Set Screen(1)
        CALL Display Frame
        CALL Display Labels
        CALL Flash (20, 2, 'Press any key to begin the program ')

        RETURN
        END

C************************************************************************
C SUBROUTINE READ SYSTEM DEFAULTS 
C 
C This routine reads the system defaults file 'geoeas.def' and  
C initializes the system defaults.
C
        SUBROUTINE Read System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

        INCLUDE  'scatter.cmn'

        CHARACTER   String*80

        OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'UNKNOWN',
     +        ERR= 999)

        READ (10, '(A)', ERR=999,END=999) File Prefix
        READ (10, '(A)', ERR=999,END=999) Data File
        READ (10, '(A)', ERR=999,END=999) String

C *** Intro flag embedded as INTRO=ON, INTRO=anything else
C *** Intro is a global logical
        Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')

        CALL Set Character Value ('PREFIX    ',File Prefix)
        CALL Set Character Value ('DATA FILE ',Data File)

        CLOSE (10)
        RETURN
         
 999    CLOSE (10)
        File Prefix = ' '
        Intro = .TRUE.

        RETURN
        END

C************************************************************************
C SUBROUTINE WRITE SYSTEM DEFAULTS 
C
C This routine writes the system defaults to the system default file 
C 'geoeas.def'.
C
        SUBROUTINE Write System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

        INCLUDE  'scatter.cmn'


        OPEN (UNIT = 10,FILE='geoeas.def', STATUS ='UNKNOWN',
     +        ERR=999)

        WRITE (10, '(A)', ERR = 999) File Prefix
        WRITE (10, '(A)', ERR = 999) Data File

        IF (Intro) THEN
           WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
        ELSE
           WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
        ENDIF

 999    CLOSE (10)

        RETURN
        END

   
C**********************************************************************   
C SUBROUTINE GET NEW SCREEN (SCREEN NUMBER) 
C
C Formal Arguments:
C  SCREEN NUMBER  - Stores the screen number.  
C
C This routine sets the current screen to screen number and then displays
C the frame and labels for that screen. 
C

        SUBROUTINE Get New Screen (Screen Number)
         
        IMPLICIT INTEGER*4 (A-Z)
       
        INTEGER*4   Screen Number        

        CALL Clear Screen
        CALL Set Screen (Screen Number)
        CALL Display Frame
        CALL Display Labels

        RETURN
        END

C*********************************************************************
C INTEGER*4 FUNCTION TOGGLE POSITION (FIELD NAME) 
C 
C Formal Arguments:
C  FIELD NAME  - Stores the toggle field name. 
C
C This routine returns the toggle position for the specified toggle
C field.        
C
        INTEGER*4 FUNCTION Toggle Position (Field Name)
       
        IMPLICIT INTEGER*4   (A-Z)

        INCLUDE '../common/screen.str'
      
        CHARACTER*10   Field Name
        
        f = Fld Index (Field Name)
        Toggle Position = SC Position (f)

        RETURN
        END
 
C*********************************************************************
C SUBROUTINE ERASE MESSAGE 
C
C This routine erases messages displayed in the message area of the 
C screen.
C
        SUBROUTINE Erase Message

        IMPLICIT INTEGER*4 (A-Z)


	CALL Erase ( 9,42,35)
	CALL Erase (10,42,35)
	CALL Erase (11,42,35)
	CALL Erase (12,42,35)
	CALL Erase (13,42,35)
	CALL Erase (14,42,35)
	CALL Erase (15,42,35)
	CALL Erase (16,42,35)
	CALL Erase (17,42,35)
	CALL Erase (18,42,35)

	RETURN 
	END

C*********************************************************************
C SUBROUTINE DISPLAY FLDS 
C
C This routine displays the current screen fields.
C
        SUBROUTINE Display Flds

        IMPLICIT INTEGER*4 (A-Z)

        INCLUDE '../common/screen.str'

        INTEGER*4   Main Menu Field, Menu Field
  
        PARAMETER (Main Menu Field = 3)
        PARAMETER (Menu Field      = 4)


        DO 10 i = Begin Screen (Current Screen),
     +            End Screen (Current Screen)
            IF (SC Field Type (i) .NE. Main Menu Field .AND.
     +         SC Field Type (i) .NE. Menu Field)
     +        CALL Write Field (i, SC Attribute (i))
10      CONTINUE

        RETURN
        END           

C***************************************************************************
C SUBROUTINE GET FIELD GROUP 
C
C This routine gets the value for a group of screen fields.
C
  	SUBROUTINE Get Field Group 
	IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

        INCLUDE '../common/screen.str'
	
 10	CALL Get Field	
	IF (SC Field Type (Next Field) .NE. Main Menu Field .AND.
     +      SC Field Type (Next Field) .NE. Menu Field) GOTO 10

	RETURN
	END

C***********************************************************************
C SUBROUTINE OPEN FILE (NUNIT, NAMFIL, STAT, OPENERR)
C
C Formal Arguments:
C  NUNIT     -  File unit number.
C  NAMFIL    -  File name.
C  STAT      -  File status.
C  OPENERR   -  Logical variable set to true if an error occurs.
C
C This routine opens Nunit with Namfil and Stat parameterss and returns 
C Openerr = 0, if successful.
C
        SUBROUTINE Open File (Nunit, Namfil, Stat, Openerr)

        IMPLICIT INTEGER*4 (A-Z)

cclong	CHARACTER Namfil*(*), Stat*3
	CHARACTER Namfil*64, Stat*3

	print *,'nunit,namfil,stat,openerr',nunit,namfil,stat,openerr
	call grwait
	Openerr = 0
cc	IF (Stat .EQ. 'OLD') THEN
	   OPEN (UNIT = Nunit,
     +           FILE = Namfil,
     +           STATUS = Stat,
     +           ERR = 99)
cc	ELSE
cc           OPEN (UNIT = Nunit,
cc     +           FILE = Namfil,
cc     +           STATUS = Stat,
cc     +           ERR = 99)

cc	ENDIF

	RETURN

 99	Openerr = 1

	RETURN
	END


         

C----------------------------------------------------------------------
C New Screen (Screen number) - sets up a new screen
C
      SUBROUTINE New Screen (Screen Number)
      IMPLICIT INTEGER*4 (A-Z)
      
      INCLUDE '../common/screen.str'
      INCLUDE 'scatter.cmn'
      
      CALL Clear Screen
      CALL Set Screen (Screen Number)
      CALL Display Frame
      CALL Display Labels
      
      DO 10 I = Begin Screen (Current Screen),
     +     End Screen (Current Screen)
         CALL Write Field (i, SC Attribute (i))
 10   CONTINUE
      
C     Erase Message Line
      CALL Erase (20, 2, 75)
      RETURN
      END
      
C----------------------------------------------------------------------
