clarge
c**********************************************************************
c*** File name: TRANSA.FOR (Source code for the program TRANS)
c**********************************************************************
                      PROGRAM TRANSFORMATION

c*** This program performs operations on variables in Geo-EAS
c*** data files.  The operations may be unary, binary, or a 
c*** transform indicator.  An operand may be either a variable
c*** or a constant.  The operator maybe an operation such as addition or 
c*** finding the square root.  The results generated by the specified
c*** operation may replace the contents of an existing variable or a
c*** new variable may be created.
c*** The unary operations   include sqrt,log,ln,truncate,exp, and rank.
c*** The binary operations   include addition, division, multiplication,
c*** subtraction and exponentiation.  These operations are described in
c*** detail in the appropriate subroutines.
c*** Trans uses a temporary file called a Scratch File to store
c*** the data read from a Geo-EAS data file.  The temporary file 
c*** called ZZSCTCH1.FIL is deleted at the time the Trans program is
c*** exited.


c*** This program displays two screens, an intro screen and the main 
c*** screen (from which menus are displayed).  These screens are defined
c*** in the  'Trans.scr' file.  Part of the main screen, the operation box,
c*** displays different headings depending on the operation to be performed.
c*** The code reflects this with 'Put Str' calls and 'Erase' calls.  These
c*** calls are placed at specific points in the code so that the screen 
c*** changes accordingly.  Caution must therefore be used when moving 
c*** or changing these calls.    

c*** The main program performs the following:
c***       Displays the main menu and executes the user's requests by
c***       either calling the appropriate subroutine, or by processing the 
c***       request within the main program.

c***  VARIABLES:
c***  Menu	  - variable that stores the field name of the main menu.
c***  Menu Choice - variable that stores the user's menu selection.
c***  Title Field - variable that stores the field name of the title field.
c***  Choice	  - variable that stores a field name. 
c***  Noinfile	  - global logical variable.
c***  Calcomp	  - global logical variable. 
c***  Newvar	  - global logical variable.
c***  Oldvar	  - global logical variable.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*10   Dvariable, Svariable, Main Menu, Title Field
      CHARACTER      Heading1*9, File Name*14, Fname*8,Heading2*11
      CHARACTER      Screen File*9, Menu Choice*19, Ch*18 
      CHARACTER      Heading0*13, Writemess*24
      CHARACTER*10   Choice
      REAL*4         Value
      INTEGER*4      FLD Index, Ivar, I, J
      LOGICAL      Exist, Exit, Get Yes No, Yes, Error

      Dvariable   = 'DVARIABLE '
      Svariable   = 'SVARIABLE '
      Main Menu   = 'ME MAIN   '
      Title Field = 'TITLE     '
      Screen File = 'trans.scr'
      Heading0    = 'Variable Name'
      Heading1    = 'Variables'
      Heading2    = 'Description'
      Writemess	  = ' Opening scratch file...'
      Noinfile    = .TRUE.
      Newdata     = .FALSE.
      Error       = .FALSE.
      Missing     = 1.E+31

c***  initialize Prefix(global character string variable) to blanks.
      CALL Init Prefix
c***  initialize Tfprefix(global character string variable) to blanks.
      CALL Init Tfprefix
c***  read prefix and input data file default values.
      CALL Read System Defaults
ccc
        CALL set up screen()
ccc
c***  Display Intro screen if global logical 'Intro' is .TRUE.
      IF (Intro) CALL Intro Screen
      CALL Clear Screen
      CALL Set Screen(2)
      CALL Display Frame
      CALL Display Labels
      Calcomp = .FALSE.
      CALL Display One Field('PREFIX    ')
cccc      CALL Display One Field('INFILE    ')

c***  Display the main menu.
      CALL Put Menu (Main Menu)

c***  The following if then elseif statement exercises the main
c***  menu option selected by the user.  For all options, except
c***  quit, control is returned to label 10 after the option has
c***  been exercised.  
  10  CALL Get Menu Choice (Main Menu, Menu Choice)
      IF (Menu Choice .EQ. 'Quit       ') THEN
c***     If new data has been generated and not saved then sound a
c***     beep and display a message to that effect for the user.
	 IF (Newdata) THEN
	   Ch = ' No file was saved'
	   CALL Beep
	   CALL Put Str(Ch, 18, 20, 2, 2)
	   Exit = Get Yes No
     +	     (20, 20, ' do you want to quit? <Y/N>') 
	   CALL Erase (20, 2, 60)
	 ELSE
	   Exit = Get Yes No 
     +	       (20, 2, 'Do you REALLY want to exit the program? <Y/N>')
	 ENDIF

c***     If the user wishes to quit then call the following subroutines.
         IF (Exit) THEN
	   CALL Close Scratch Files
           CALL Clear Screen
           CALL reset Screen
	   CALL Write System Defaults
	   GOTO 50
         ENDIF

	 GOTO 10

      ELSE IF (Menu Choice .EQ. 'Prefix          ') THEN
c***    prompt user for prefix  ***
	Noinfile = .TRUE.
        CALL Put Str (Heading1,  9, 10, 35, 0)
  	Choice = 'PREFIX    '
	Next Field = FLD Index (Choice)
	CALL Get Field
	CALL Get Character Value (Choice, Prefix)
c***    prompt user for scratch file prefix  ***
  	Choice = 'TFPREFIX  '
	Next Field = FLD Index (Choice)
	CALL Get Field
	CALL Get Character Value (Choice, Tfprefix)
c***    Determine if the 'scratch files' can be generated with the 
c***    provided path.
        CALL Put Str (Writemess, 24, 20, 2, 2)
        File Name = 'ZZSCTCH1.FIL  '
        CALL Build Prefix Scratch File(File Name)
c***    Open scratch file to contain 'Data1' 'structure'. ***
        OPEN (UNIT=1,FILE=Prefile,ACCESS='DIRECT',RECL=4,
     +       FORM='unformatted',IOSTAT=I)
ccc     +        FORM='BINARY',IOSTAT=I)
        File Name = 'ZZSCTCH2.FIL  '
        CALL Build Prefix Scratch File(File Name)
c***    Close scratch file that stores 'Data1' 'structure'. ***
        CLOSE(1, STATUS='DELETE')
c*** 	If the directory does not exist then display a message and
c***    set the global variable Tfprefix as well as the scratch 
c***    file field to blanks.
	IF (I .GE. 1) THEN
	  CALL Flash(20, 2, 
     + 'ERROR - Scratch file cannot use this prefix...(press any key)')
          CALL Init Tfprefix
	  Choice = 'TFPREFIX  '
	  CALL Set Character Value (Choice, Tfprefix)
	  CALL Display One Field (Choice)
        ENDIF
c***    Erase message line.
	CALL Erase (20, 2, 30)

      ELSE IF (Menu Choice .EQ. 'Data                ') THEN
c***    Erase current display of variables and reset logicals.
	Call Clear Variable Box
	Newdata = .FALSE.
	Noinfile = .TRUE.
        CALL Put Str (Heading1,  9, 10, 35, 0)
c***    prompt user for input file ***
  	Fname = 'INFILE  '
	CALL Get Name of File(Fname,Infil,Error,1)
c***    if error return to menu.
	IF (Error) GOTO 10
c***    Read in variable names.
        CALL File Set Up (Error, Nvar)
c***    Set Recskip to record at which sample data first appears
c***    in Geo-EAS data file.
	Recskip = Nvar + 3
c***    If no error then read in sample data and display variable
c***    names.
	IF (Error) GOTO 10
        CALL Read Numerical Data(1,Infil,Nvar,MAXVAR,Error,Recskip)
	IF (Error) GOTO 10
	CALL Display Variables
	Noinfile = .FALSE.

      ELSE IF (Menu Choice .EQ. 'Save                ') THEN
	IF (Noinfile) THEN
     	  CALL Beep
	  CALL Flash (20, 2,
     +        'You must read in a data file...(press any key)')
	ELSE
c***      Prompt user for outfile ***
	  Fname = 'OUTFILE '
	  CALL Get Name of File(Fname, Outfil, Error,2)
C***      If error return to menu else write data to output file.
	  IF (Error) GOTO 10
          CALL Write to Output File 
	ENDIF

      ELSE IF (Menu Choice .EQ. 'Title      ') THEN
        CALL Put Str (Heading1,  9, 10, 35, 0)
	IF (Noinfile) THEN
     	 CALL Beep
	 CALL Flash (20, 2,
     +        'You must read in a data file...(press any key)')
	ELSE
c***     Get the title from the title field.
         Next Field = FLD Index (Title Field)
         CALL Get Field
	 CALL Get Character Value (Title Field, Title)
	ENDIF

      ELSE IF (Menu Choice .EQ. 'Create     ') THEN
c***  The Create option provides the user with the necessary commands 
c***  to produce variables that result from unary or binary operations.
c***  These commands are accessed through a series of menus. 
        CALL Put Str (Heading1,  9, 10, 35, 0)
	IF (Noinfile) THEN
     	 CALL Beep
	 CALL Flash (20, 2,
     +        'You must read in a data file...(press any key)')
        ELSE
c***     Reset all flags(logicals) and commence with the Create Option.
	 CALL Init Flags
	 Calcomp = .FALSE.
	 Newvar  = .FALSE.
	 Oldvar  = .FALSE.
	 CALL Create Option
c***     Erase the operation heading.
	 CALL Erase (8,  2, 13)
	 CALL Erase (8, 16, 53)
	 CALL Put Menu (Main Menu)
	ENDIF

      ELSE IF (Menu Choice .EQ. 'V_Delete   ') THEN
c***  Delete Option prompts the user for a specific variable to be
c***  deleted.  The corresponding toggle fields are then 're'loaded 
c***  and corresponding arrays are adjusted to reflect the deletion.
        CALL Put Str (Heading1,  9, 10, 35, 0)
	IF (Noinfile) THEN
     	 CALL Beep
	 CALL Flash (20, 2,
     +        'You must read in a data file...(press any key)')
	ELSE
	 CALL Put Str(Heading0, 13, 8, 2, 0)
c***     Prompt user for variable to be deleted.
	 Ivar = Toggle (Dvariable) + 1
	 Yes = Get Yes No
     +	  (20, 2, 'Do you really want to delete this variable?...(Y/N)')
	 IF (Yes) THEN
c***       If user is sure to delete this variable, then proceed with
c***       Delete Option.
	   CALL Delete Option(Ivar, Error)
	 ELSE
c***       Clear screen field displaying variable to be deleted.
           CALL Erase (8, 16, 53)
	 ENDIF
	ENDIF
      ENDIF
      GOTO 10

  50  CONTINUE
      END
C**********************************************************************
C        C r e a t e  O p t i o n     (subroutine)
C**********************************************************************
      SUBROUTINE Create Option

c*** This routine displays the 'Create Option' menu.  The user is prompted
c*** for either a current variable or a new variable to contain the results.
c*** After the user's choice is processed the screen displays a new
c*** menu that prompts the user for either a unary operation or a binary
c*** operation.	The appropriate subroutine is then called.
c***
c*** VARIABLES:
c*** Main menu	 - stores field name of main menu.
c*** Menu	 - stores field name of Create Option menu.
c*** Meubmain	 - stores field name of Operation menu.
c*** Menu Choice - stores the user's menu selection.
c*** Var	 - stores the toggle position of the item selected by
c***               the user.
c*** Newvar      - global logical variable.
c*** Oldvar      - global logical variable.
c*** Varnam      - global array containing variable names.
c*** Units       - global array containing measurement descriptions of
c***               of each variable.
c*** Tmpvnam     - global variable containing a variable name.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*10  Main Menu, Menu, Choice, Svariable Field, Meubmain
      CHARACTER     Menu Choice*19, Varchoice*10, Tvname*10 
      CHARACTER     Heading0*13,Heading1*5 
      INTEGER*4     Var, Toggle, FLD Index
      LOGICAL     Yes, Get Yes No

      Main Menu       = 'ME MENU   '
      Menu            = 'MECMAIN   '
      Svariable Field = 'SVARIABLE '
      Meubmain        = 'MEUBMAIN  '
      Heading0        = 'Variable Name'
      Heading1        = 'Units'
c***  Initialize unary and binary operand constants to zero.
      Choice = 'UOPERCONST'
      CALL Set 4 Floating Point Value (Choice, 0, 4)
      Choice = 'BOP1CONST '
      CALL Set 4 Floating Point Value (Choice, 0, 4)
      Choice = 'BOP2CONST '
      CALL Set 4 Floating Point Value (Choice, 0, 4)
c***  Display the Create Option menu.
      CALL Put Menu (Menu)
  10  CALL Get Menu Choice (Menu, Menu Choice)
      Choice = 'NEW VAR   '
      Varchoice = '          '
      CALL Set Character Value (Choice, Varchoice)
      IF (Menu Choice .EQ. 'Quit            ') THEN
	 CALL Erase (20, 2, 70)
	 CALL Clear Operation Box
c***     Erase operation name.
	 CALL Erase (8,  2, 13)
c***     Erase operation box.
	 CALL Erase (8, 16, 53)
	 RETURN
      ELSE IF (Menu Choice .EQ. 'New Variable    ') THEN
c***     Before this option is used a check to determine if the addition
c***     of a new variable exceeds the maximum number of variables
c***     permitted(MAXVAR) is performed.  If that maximum is exceeded an 
c***     error message is flashed and the user is returned to the menu.
c***     This option prompts the user for the name of a new variable
c***     along with the name of units, width field of output format and
c***     the number of decimal positions.
         IF((Nvar + 1) .GT. MAXVAR) THEN
	   CALL Flash(20,2,
     +	   'Error - Exceeding max number of variables (press any key)')
	   CALL Clear Operation Box
	   GOTO 10
	 ENDIF
c***     Prompt user for new variable name, retrieve this name from
c***     the screen field and assign to the variable 'tmpvnam'.
	 CALL Put Str (Heading0, 13, 8, 2, 0)
	 Choice = 'NEW VAR   '
	 Newvar = .TRUE.
	 Oldvar = .FALSE.
  	 Next Field = FLD Index (Choice)
         CALL Getfield
	 CALL Get Character Value (Choice, Tmpvnam)
	 CALL Erase (8,  2, 13)
c***     If variable name is all blanks then display message and
c***     return to menu.
	 IF (Tmpvnam .EQ. '          ') THEN
	   CALL Flash(20, 2,
     +        'Variable has not been specified...(press any key)')
           Newvar = .FALSE.
           Oldvar = .FALSE.
           GOTO 10
	 ENDIF
c***     Check if new var already exists if so then display message.
	 DO 40 I = 1, Nvar
	   IF (Tmpvnam .EQ. Varnam(I)) THEN
	     Newvar = .FALSE.
	     Oldvar = .TRUE.
	     CALL Flash(20, 2,
     +            'This variable already exists...(press any key)') 
	   ENDIF
  40     CONTINUE
         IF (Newvar) THEN
c***       Request the user for unit's description.
	   CALL Put Str(Heading1, 5, 8, 2, 0)
c***       Set UNITS screen field to blanks.
	   Choice = 'UNITS     '
	   Tunits = '          '
	   CALL Set Character Value (Choice, Tunits)
           Next Field = FLD Index (Choice)
	   CALL Getfield
	   CALL Get Character Value(Choice, Tunits)
	   CALL Erase (8,  2, 13)
	   CALL Erase (8, 16, 53)
	 ENDIF
	 Choice = 'NEW VAR   '
	 CALL Display One Field (Choice)
      ELSE IF (Menu Choice .EQ. 'Old Variable    ') THEN
c***     This option permits the user to pick an existing variable to
c***     contain the results.  Once the variable has been chosen, the
c***     user may change the variable name to a unique variable name.
	 Newvar = .FALSE.
	 Oldvar = .TRUE.
	 CALL Put Str (Heading0, 13, 8, 2, 0)
	 Var = Toggle (Svariable Field)	+ 1
	 CALL Get Toggle Value (Svariable Field, I, Tmpvnam)
	 Yes = Get Yes No
     +	  (20, 2, 
     +  ' Do you wish to change variable name or units?...(Y/N)')
	 IF (Yes) THEN
c***     If the user decides to change the variable name or units,
c***     then the new name is retrieved and checked to see if it 
c***     already exists, if so the user may repeat this process or
c***     retain the original variable name (which is stored in 'Tvname').
	   Tvname = Tmpvnam
	   Choice = 'NEW VAR   '
	   CALL Set Character Value (Choice, Tmpvnam)
  35	   Next Field = FLD Index (Choice)
           CALL Getfield
	   CALL Get Character Value (Choice, Tmpvnam)
	   CALL Erase (8,  2, 13)
	   IF (Tmpvnam .EQ. '          ') THEN
	     CALL Flash(20, 2,
     +        'Variable has not been specified...(press any key)')
             GOTO 35
	   ENDIF
c***       Check if new name already exists(not unique).  If the name
c***	   is unique then ask for the description of units.
	   IF (Tmpvnam .NE. Varnam(Var)) THEN
	     DO 45 I = 1, Nvar
	       IF (Tmpvnam .EQ. Varnam(I)) THEN
	         Yes = Get Yes No
     + (20, 2,'This name already exists, do you wish to rename...(Y/N)')
	         IF (Yes) THEN
                   GOTO 35
	         ELSE
c***		   If user wishes to retain old variable name then assign
c***               that name to 'Tmpvnam' and display it in the screen field.
		   Tmpvnam = Tvname
		   CALL Set Character Value(Choice,Tmpvnam)
		   CALL Display One Field(Choice)
	           CALL Erase (8,  2, 13)
		   GOTO 55
	         ENDIF
	       ENDIF
  45         CONTINUE
	   ENDIF
c***       Request the user for unit's description.
c***       Clear the variable name screen field and replace with
c***       description of units screen field.
	   CALL Erase (8,  2, 13)
	   CALL Put Str(Heading1, 5, 8, 2, 0)
	   Choice = 'UNITS     '
	   Tunits = Units(Var)
	   CALL Set Character Value (Choice, Tunits)
           Next Field = FLD Index (Choice)
	   CALL Getfield
	   CALL Get Character Value(Choice, Tunits)
c***       After the user has been prompted for the units description,
c***       clear that screen field and the heading and display the 
c***       variable name. 
	   CALL Erase (8,  2, 13)
	   CALL Erase (8, 16, 53)
   	   Choice = 'NEW VAR   '
   	   CALL Display One Field (Choice)
c***       Assign variable name and units to the data structures that will
c***       will be written to the Geo-EAS data file.
   	   Varnam(Var) = Tmpvnam
	   Units(Var)  = Tunits
           CALL Load New Toggle Values
           CALL Display Variables
	 ENDIF
         CALL Erase (8,  2, 13)
      ENDIF
c***  Get user's choice of binary or unary operation  ***
c***  and call appropriate subroutine.                ***
  55  IF (Newvar .OR. Oldvar) THEN
	 CALL Put Menu (Meubmain)
  15	 CALL Get Menu Choice (Meubmain, Menu Choice)
	 IF (Menu Choice .EQ. 'Unary Operation ') THEN
	   CALL Unary Process
c***       If a calculation has been performed (Calcomp = .true.) then 
c***       return to the calling program, else return to this menu.
	   IF (Calcomp) THEN
	     RETURN
	   ELSE
	     CALL Put Menu (Meubmain)
	   ENDIF
	 ELSE IF (Menu Choice .EQ. 'Binary Operation') THEN
	   CALL Select Binary Operand 1
c***       Refer to the immediately preceding comment.
	   IF (Calcomp) THEN
	     RETURN
	   ELSE
	     CALL Put Menu (Meubmain)
	   ENDIF
	 ELSE IF (Menu Choice .EQ. 'Indicator Transform') THEN
	   CALL Indicator Operation
c***       Refer to the immediately preceding comment.
	   IF (Calcomp) THEN
	     RETURN
	   ELSE
	     CALL Put Menu (Meubmain)
	   ENDIF
	 ELSE IF (Menu Choice .EQ. 'Quit        ') THEN
	   CALL Clear Operation Box
	   CALL Erase (8,  2, 13)
	   CALL Erase (8, 16, 53)
	   CALL Put Menu (Menu)
           GOTO 10
	 ENDIF
	 GOTO 15
      ENDIF
      GOTO 10
      RETURN
      END
c**********************************************************************
