
C Search.for
C---------------------------------------------------------------------
C Search        - Search Neighborhood  (Program XVALID)
C
C Passed Variables:   X0, Y0    - Center of search area
C                     Neighbors - list of pointers into Data array 
C                     N         - number of neighbors found
C                     Failed    - Flag for 'failed search criteria'
C Local Variables:    Ptr       - pointers for sort
C                     Octants   - octant counters
C                     Quadrants - quadrant counters
C                     Empty     - counter for conseq. empty sectors
C                     Aspect    - Ratio of search major to minor radius
C                     SrMajor2  - Squared max search radius (No SQRT call)
C                     X, Y      - Point to be tested  
C                     Dx, Dy    - X and Y distances
C                     Hx, Hy    - Dx, Dy corrected for aspect, ellipse angle
C                     Distance  - distance squared (used for sorting)
C                     Cosa, Sina - Ellipse angle Cosines, Sines (for rotation)
C                     WXmin, WYmin - Extents for preliminary search 
C                     WXmax, WYmax - "
 
        SUBROUTINE Search (X0, Y0, Neighbors, N, Failed)
	IMPLICIT INTEGER*4 (A-Z)

		include 'xvalid.dec'

        INTEGER*4       I, J, K, N, N1, Yptr, Dptr
ccc        INTEGER*4       Neighbors (1), Ptr (1) 
        INTEGER*4       Neighbors (1), Ptr (MAXDATA) 
        INTEGER*4       Quad, Oct, Quadrants (4), Octants (8)
        INTEGER*4       Empty 
        REAL*4          Dx, Dy, Hx, Hy, D, Ratio, SrMajor2
        REAL*4          Distance (64), Tx, Ty, MinDist2, Gamma
        REAL*4          Cosa, Sina, Dtr, X, Y, X0, Y0, D2
        REAL*4          WXmin, WXmax, WYmin, WYmax, Tiny
        LOGICAL         Failed
        character*60 stuff

        DATA Dtr 	/0.0174533/
	DATA Tiny	/1.0E-6/
        Failed   = .FALSE.

C Calculate SIN, COS first for efficiency
        Cosa     = COS (-SrAngle * Dtr)
        Sina     = SIN (-SrAngle * Dtr)

C Initialize search rectangle
        WXmin    = AMAX1 (X0 - SrMajor, Xmin)
        WXmax    = AMIN1 (X0 + SrMajor,	Xmax)
        WYmin    = AMAX1 (Y0 - SrMajor, Ymin)
        WYmax    = AMIN1 (Y0 + SrMajor, Ymax)

C Zero octants and quadrants
        DO 5 I = 1, 8
           Octants (I) = 0
           J = I/2
           IF (J .NE. 0) Quadrants (J) = 0
 5      CONTINUE
	MinDist2 	= MinDist * MinDist
        MaxP 		= MaxPs * Nsector	       

C Find start sample for preliminary search
	Yptr = 1
	IF (WYmin .GT. Yvalues (1)) THEN
            CALL SearchB (Yvalues, Ndata, WYmin, Yptr)
	    IF (Yptr .LE. 0) Yptr = 1
	ENDIF

C Look only at samples within the outer rectangle, then
C save pointers to those within the search ellipse  
C
C If the point is within the rectangle surrounding the search ellipse,
C then it must be >= the min distance parameter, to be kept
C If variogram distance parameter selected, then the distance used to
C determine 'closeness' is the gamma function value. If euclidean 
C distance was chosen, the distance components are scaled to a unit circle.
C Those distances within the circle will be <= 1.000
C
        N1 = 0
 10     IF (Yptr .LE. Ndata) THEN
           Dptr = YvalPtr (Yptr)
           X = XData (Dptr)
           Y = YData (Dptr) 
           IF (Y .LE. WYmax) THEN
              IF (X .GE. WXmin .AND. X .LE. WXmax) THEN
                 Dx = X - X0
                 Dy = Y - Y0
	         IF (ABS (Dx) .LT. Tiny .AND.
     +		     ABS (Dy) .LT. Tiny) GOTO 12 
                 D2 = Dx*Dx+Dy*Dy
	         IF (D2 .GE. MinDist2) THEN
                    Hx = Dx * Cosa - Dy * Sina
                    Hy = Dy * Cosa + Dx * Sina
                    Tx = Hx / SrMajor
                    Ty = Hy / SrMinor       
                    D  = Tx*Tx + Ty*Ty
                    IF (D .LE. 1.0)  THEN 
	               IF (.NOT. Euclidean) D2  = Gamma (Dx, Dy)
     		       CALL Insert (Distance, Ptr, D2, Dptr, N1, Maxp)   
	            ENDIF     
	         ENDIF
              ENDIF
 12           Yptr = Yptr + 1
              GOTO 10
           ENDIF
        ENDIF

C Check if enough samples were found to continue
        IF (N1 .LT. MinP .OR. N1 .LT. 1) THEN
            Failed = .TRUE.
	    IF (N1 .GT. 0) THEN
	      DO 15 I = 1, N1
                 Neighbors (I) = Ptr (I)
 15	      CONTINUE
	    ENDIF
	    N = N1
	    RETURN	    
        END IF

C determine which of the selected neighbors to keep
C if number of sectors = 1, then keep nearest MaxP points and return
	IF (Nsector .EQ. 1) THEN
	   N = 0
	   DO 18 I = 1, N1
              IF (N .LT. MaxP) THEN
                N = N + 1
                Neighbors (N) = Ptr (I)
	      ENDIF
 18        CONTINUE
	   RETURN
	ENDIF  	

C If quadrant or octant search selected, determine sectors
C only the 'closest' neighbors are kept - (using selected distance type)
	N = 0	
        DO 20 I = 1, N1
           X  = XData (Ptr (I))
           Y  = YData (Ptr (I))                
           Dx  = X - X0
           Dy  = Y - Y0  
           CALL Sector (Dx, Dy, Quad, Oct)
           Quadrants (Quad) = Quadrants (Quad) + 1
           Octants (Oct)    = Octants (Oct) + 1
           IF ((Nsector .EQ. 4 .AND.	Quadrants (Quad) .LE. MaxPs)
     +     .OR.(Nsector .EQ. 8 .AND. Octants (Oct) .LE. MaxPs)) THEN
              IF (N .LT. MaxP) THEN
                 N = N + 1
                 Neighbors (N) = Ptr (I)
	      ELSE
                 GOTO 25
              END IF
	   ENDIF
 20     CONTINUE

C Find number of consecutive empty sectors starting at each sector
C  If the number of empty sectors > Maxempty then return with no neighbors
 25     IF (Nsector .EQ. 8) THEN
           DO 30 I = 1, 8
              Empty = 0 
              IF (Octants (I) .EQ. 0) THEN
                 Empty = 1
                 DO 40 J = I+1, I+7
                   K = J
                   IF (K .GT. 8) K = MOD (J, 8)
                   IF (Octants (K) .EQ. 0) THEN
                      Empty = Empty + 1
                   ELSE
                      GOTO 30
                   ENDIF
 40              CONTINUE
                 IF (Empty .GT. MaxEmpty) THEN
                    Failed = .TRUE.
                    RETURN
                 END IF
              END IF
 30        CONTINUE
        ELSE IF (Nsector .EQ. 4) THEN
           DO 50 I = 1, 4
              Empty = 0 
              IF (Quadrants (I) .EQ. 0) THEN
                 Empty = 1
                 DO 60 J = I+1, I+3
                   K = J
                   IF (K .GT. 4) K = MOD (J, 4)
                   IF (Quadrants (K) .EQ. 0) THEN
                      Empty = Empty + 1
                   ELSE
                      GOTO 50
                   ENDIF
 60              CONTINUE
                 IF (Empty .GT. MaxEmpty) THEN
                    Failed = .TRUE.
                    RETURN
                 ENDIF
              ENDIF
 50        CONTINUE
        ENDIF
        RETURN
        END

C----------------------------------------------------------------------
C Sector - calculates quadrant and octant for dx,dy

        SUBROUTINE Sector (Dx, Dy, Quad, Oct)
	IMPLICIT INTEGER*4 (A-Z)
        REAL*4          Dx, DY, Adx, Ady
        INTEGER*4       Quad, Oct

        Adx = ABS (Dx)
        Ady = ABS (Dy)
        IF      (Dx .EQ. 0.0 .AND. Dy .EQ. 0.0) THEN
           Oct  = 1
           Quad = 1
	ELSE IF (Dy .GE. 0.0 .AND. Dx .GT. 0.0) THEN
           Quad = 1
	   IF  (Adx .GT. Ady) THEN
	      Oct = 1
	   ELSE 
	      Oct = 2
	   ENDIF
	ELSE IF (Dy .GT. 0.0 .AND. Dx .LE. 0.0) THEN
           Quad = 2 
   	   IF  (Adx .LT. Ady) THEN
   	      Oct = 3
   	   ELSE 
   	      Oct = 4
   	   ENDIF
   	ELSE IF (Dy .LE. 0.0 .AND. Dx .LT. 0.0) THEN
          Quad = 3
   	  IF  (Adx .GT. Ady) THEN
   	     Oct = 5
   	  ELSE 
   	     Oct = 6
   	  ENDIF
	ELSE IF (Dy .LT. 0.0 .AND. Dx .GE. 0.0) THEN
          Quad = 4
	  IF  (Adx .LT. Ady) THEN
	     Oct = 7
	  ELSE 
	     Oct = 8
	  ENDIF
	END IF
        RETURN
        END        

C----------------------------------------------------------------------
C	Insert - inserts Element, and ElementPtr into
C                      Vector, and Ptr (in Vector sorted order)
C                                
 	SUBROUTINE Insert (Vector, Ptr, Element, ElementPtr, N, MaxN)

	IMPLICIT INTEGER*4 (A-Z)

	INTEGER*4	I, N, MaxN, N2, Ptr (1), ElementPtr, Index
	REAL*4		Vector (1), Element
	
	IF (N .EQ. 0) THEN
	   Vector (1) = Element
	   Ptr    (1) = ElementPtr	 
	   N          = 1		 
	   RETURN				 
	ENDIF

        CALL SearchB (Vector, N, Element, Index)

	IF (Index .GT. N) THEN
	   IF (N .LT. MaxN) THEN
	     N = N + 1
             Vector (N) = Element
	     Ptr (N)    = ElementPtr
	   ENDIF
	ELSE
	   N2 = MIN (N, MaxN-1) 
  	   DO 10 I = N2, Index+1, -1
	      Vector (I+1) = Vector (I)        
	      Ptr    (I+1) = Ptr (I)
 10	   CONTINUE
	   Vector (Index+1) = Element
	   Ptr    (Index+1) = ElementPtr
	   IF (N .LT. MaxN) N = N + 1
	ENDIF

	RETURN
	END

                 
                 

