/************************************************************
 *                                                          *
 *  Permission is hereby granted  to  any  individual   or  *
 *  institution   for  use,  copying, or redistribution of  *
 *  the xgobi code and associated documentation,  provided  *
 *  that   such  code  and documentation are not sold  for  *
 *  profit and the  following copyright notice is retained  *
 *  in the code and documentation:                          *
 *     Copyright (c) 1990,1991,1992,1993,1994 Bellcore      *
 *                                                          *
 *  We welcome your questions and comments, and request     *
 *  that you share any modifications with us.               *
 *                                                          *
 *    Deborah F. Swayne            Dianne Cook              *
 *     dfs@bellcore.com         dicook@iastate.edu          *
 *      (201) 829-4263                                      *
 *                                                          *
 ************************************************************/

#include <stdio.h>
#include "xincludes.h"
#include "xgobitypes.h"
#include "xgobivars.h"

/* Functions used in this file */
void set_mono();
void help();
void show_message();
void set_wm_protocols();
/* */

#define NMENUS NCOLS+32
#define NSBARS 32
#define NCOMMANDS 512

typedef struct {
   Widget w;
   char *helpFile;
} HelpTable;

HelpTable pb_table[NCOMMANDS];
HelpTable sbar_table[NSBARS];
HelpTable menupb_table[NMENUS];

/* ARGSUSED */
XtActionProc
HelpSelect(w, evnt, params, nparams)
/*
 * Search the help tables looking for w, then call help()
*/
  Widget w;
  XEvent *evnt;
  String *params;
  Cardinal nparams;
{
  int j = 0;

  while (pb_table[j].w != NULL)
  {
    if (pb_table[j].w == w)
    {
      help(w, pb_table[j].helpFile);
      break;
    }
    else
      j++;
  }

  j = 0;
  while (sbar_table[j].w != NULL)
  {
    if (sbar_table[j].w == w)
    {
      help(w, sbar_table[j].helpFile);
      break;
    }
    else
      j++;
  }

  j = 0;
  while (menupb_table[j].w != NULL)
  {
    if (menupb_table[j].w == w)
    {
      help(w, menupb_table[j].helpFile);
      break;
    }
    else
      j++;
  }
}

/* ARGSUSED */
XtCallbackProc
help_cback(w, client_data, cback_data)
  Widget w;
  caddr_t client_data;
  caddr_t cback_data;
{
  help(w, "Help");
}

void
init_help(xg)
  xgobidata *xg;
{
  xg->nhelpids.pb = xg->nhelpids.menupb = xg->nhelpids.sbar = 0;
}

/* ARGSUSED */
XtCallbackProc
help_done_cback(w, client_data, cback_data)
  Widget w;
  XtPointer client_data;
  caddr_t cback_data;
{
  XtDestroyWidget(XtParent(XtParent(w)));
}

void
help(popup_pop, helpfile)
  Widget popup_pop;
  char *helpfile;
{
  char fname[100];
  char message[200];
  char *getenv();
  char *xgobidir;
  FILE *fopen(), *fp;
  extern xgobidata xgobi;

  xgobidir = getenv("XGOBIDIR");
  if (xgobidir == NULL || strlen(xgobidir) == 0)
  {
    xgobidir = (char *) XtMalloc((Cardinal) 150 * sizeof(char));
    (void) strcpy(xgobidir, DEFAULTDIR);
    if (xgobidir == NULL || strlen(xgobidir) == 0)
    {
      sprintf(message,
       "XGOBIDIR is not defined in your environment, and\n");
      strcat(message,
       "DEFAULTDIR is not defined in the XGobi Makefile;\n");
      strcat(message,
        "see the person who installed XGobi for help.\n");
      show_message(message, xgobi.shell);
      return;
    }
    else
    {
      (void) strcpy(fname, xgobidir);
      XtFree((XtPointer) xgobidir);
    }
  }
  else
  {
    (void) strcpy(fname, xgobidir);
  }

/*
 * Check that the file is good.
*/
  (void) strcat(fname, "/help/");
  (void) strcat(fname, helpfile);
  if ((fp = fopen(fname, "r")) == NULL)
  {
    sprintf(message,
      "Unable to open %s.\n", fname);
    strcat(message,
      "Is the shell variable XGOBIDIR the name of the directory\n");
    strcat(message,
      "which contains the help subdirectory?\n");
    show_message(message, xgobi.shell);
    return;
  }
  else
  {
    Widget hpopup, hframe, htext, hdone;
  /*
   * Create the popup itself.
  */
    hpopup = XtVaCreatePopupShell("Help",
      topLevelShellWidgetClass, popup_pop,
      XtNtitle, (String) "XGobi Help Window",
      XtNiconName, (String) "XGobi Help Window",
      NULL);
    if (mono) set_mono(hpopup);
  /*
   * Create the paned widget.
  */
    hframe = XtVaCreateManagedWidget("Form",
      panedWidgetClass, hpopup,
      XtNorientation, (XtOrientation) XtorientVertical,
      NULL);
    if (mono) set_mono(hframe);

  /*
   * Create the text widget.
  */
    htext = XtVaCreateManagedWidget("Text",
      asciiTextWidgetClass, hframe,
      XtNallowResize, (Boolean) True,
      XtNshowGrip, (Boolean) False,
      XtNtype, (XawAsciiType) XawAsciiFile,
      XtNstring, (String) fname,
      XtNscrollVertical, (XawTextScrollMode) XawtextScrollWhenNeeded,
      XtNdisplayCaret, (Boolean) False,
      XtNfont, (XFontStruct *) appdata.helpFont,
      NULL);
    if (mono) set_mono(htext);

  /*
   * Create the Done button.
  */
    hdone = XtVaCreateManagedWidget("Done",
      commandWidgetClass, hframe,
      XtNshowGrip, (Boolean) False,
      XtNskipAdjust, (Boolean) True,
      XtNlabel, (String) "Click here to dismiss",
      NULL);
    if (mono) set_mono(hdone);

    XtAddCallback(hdone, XtNcallback,
      (XtCallbackProc) help_done_cback, (XtPointer) NULL);

    XtPopup(hpopup, XtGrabNone);
    XRaiseWindow(display, XtWindow(hpopup));

    set_wm_protocols(hpopup);

    fclose(fp);
  }
  return;
}

void
add_pb_help(n, w, fname)
  int *n;
  Widget w;
  char *fname;
/*
 * Add a widget to the helptable for pushbutton widgets.
*/
{
  if (*n>0 && pb_table[*n-1].w == NULL)
    (*n)--;

  pb_table[*n].w = w;
  pb_table[*n].helpFile = fname;
  (*n)++;

  if (*n == NCOMMANDS)
    printf("Edit help.c and increase NCOMMANDS\n");
}

void
add_menupb_help(n, w, fname)
  int *n;
  Widget w;
  char *fname;
/*
 * Add a widget to the helptable for pushbutton widgets.
*/
{
  if (*n>0 && pb_table[*n-1].w == NULL)
    (*n)--;

  menupb_table[*n].w = w;
  menupb_table[*n].helpFile = fname;
  (*n)++;

  if (*n == NMENUS)
    printf("Edit help.c and increase NMENUS\n");
}

void
add_sbar_help(n, w, fname)
  int *n;
  Widget w;
  char *fname;
/*
 * Add a widget to the helptable for scrollbar widgets.
*/
{
  sbar_table[*n].w = w;
  sbar_table[*n].helpFile = fname;
  (*n)++;

  if (*n == NSBARS)
    printf("Edit help.c and increase NSBARS\n");
}
