/************************************************************
 *                                                          *
 *  Permission is hereby granted  to  any  individual   or  *
 *  institution   for  use,  copying, or redistribution of  *
 *  the xgobi code and associated documentation,  provided  *
 *  that   such  code  and documentation are not sold  for  *
 *  profit and the  following copyright notice is retained  *
 *  in the code and documentation:                          *
 *     Copyright (c) 1990,1991,1992,1993,1994 Bellcore      *
 *                                                          *
 *  We welcome your questions and comments, and request     *
 *  that you share any modifications with us.               *
 *                                                          *
 *    Deborah F. Swayne            Dianne Cook              *
 *     dfs@bellcore.com         dicook@iastate.edu          *
 *      (201) 829-4263                                      *
 *                                                          *
 ************************************************************/

#define XGOBIINTERN

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "xincludes.h"
#include "xgobitypes.h"
#include "xgobivars.h"
#include <X11/keysym.h>

xgobidata xgobi;
#include "xgobitop.h"

Boolean
RunWorkProcs()
{
  Boolean RunWorkProc();

  if (xgobi.is_iconified)
    return TRUE;
  else if (RunWorkProc(&xgobi))
    return FALSE;
  else
    return TRUE;
}

int
parse_command_line(ac, av, data_in, xg)
  int *ac;
  char **av;
  char *data_in;
  xgobidata *xg;
{
  int retval = 1;

/*
 * Now parse the command line.
*/
  for( ; (*ac)>1 && av[1][0]=='-'; (*ac)--,av++)
  {
    /*
     * -s:  xgobi initiated from inside S
    */
    if (strcmp(av[1], "-s") == 0)
      xg->data_mode = Sprocess;

/* Eventually could allow users to override the defaults here */
    /*
     * -binary:  read binary data
    else if (strcmp(av[1], "-binary") == 0)
      xg->data_mode = binary;
    */

    /*
     * -std:  look for one of mmx (default), msd, or mmd
    */
    else if (strcmp(av[1], "-std") == 0)
    {
      if (strcmp(av[2], "mmx") == 0)
      {
        xg->std_type = 0;
        av++; (*ac)--;
      }
      else if (strcmp(av[2], "msd") == 0)
      {
        xg->std_type = 1;
        av++; (*ac)--;
      }
      else if (strcmp(av[2], "mmd") == 0)
      {
        xg->std_type = 2;
        av++; (*ac)--;
      }
    }

    /*
     * -dev:  look for a float.  Only meaningful in msd or mmd cases.
    */
    else if (strcmp(av[1], "-dev") == 0)
    {
      xg->std_width = (float) atof(av[2]);
      av++; (*ac)--;
    }
  }
  (*ac)--;
  av++;

/*
 * Test the values
*/
  if (xg->std_type != 0 && xg->std_type != 1 && xg->std_type != 2)
  {
    (void) fprintf(stderr,
      "std: Standardization type not valid; aborting.\n");
    retval = 0;
    return(retval);
  }
  if (xg->std_width <= 0 || xg->std_width >= 100.)
  {
    (void) fprintf(stderr,
      "dev: Deviation value not valid; aborting.\n");
    retval = 0;
    return(retval);
  }

  if (xg->data_mode == Sprocess)
  {
    if (*ac < 3)
    {
      (void) fprintf(stderr,
        "Usage: xgobi [-s nrows ncols path] filename\n");
      retval = 0;
      return(retval);
    }
    else
    {
      xg->nrows = atoi(*av); av++;
      xg->ncols_used = atoi(*av); av++;
      xg->ncols = xg->ncols_used + 1;
      (void) strcpy(Spath0, *av); av++;
      (void) strcat(Spath0, "/");
      (void) strcpy(data_in, *av);
    }
  }
  else /* if (xg->data_mode == ascii || xg->data_mode == binary) */
  {
    if (*ac == 0)
      (void) strcpy(data_in, "stdin");
    else
      (void) strcpy(data_in, *av);
  }
  return(retval);
}

int
main(argc, argv)
  int argc;
  char *argv[];
{
  char data_in[100];
/*  int atoi();*/
  int make_xgobi();
  void find_mono();
  struct stat buf;
  /*char name[50];*/
  /*name[0] = '\0';*/

  xgobi.shell = XtAppInitialize(&app_con, "XGobi", 
    (XrmOptionDescRec *) NULL, (Cardinal) 0,
    &argc, argv, fallback_resources, (ArgList) NULL, (Cardinal) 0);
  display = XtDisplay(xgobi.shell);
  XtAppAddActions(app_con, added_actions, XtNumber(added_actions));
  find_mono(&mono);

/* This makes producing b&w pictures easier */
/*
  mono = True;
*/

/*
 * Initialize a few values before beginning to read the
 * command line.
*/
  xgobi.std_type = 0;
  xgobi.std_width = 2.0;
  xgobi.data_mode = ascii;

  if (!parse_command_line(&argc, argv, data_in, &xgobi))
    return(0);

#ifdef SENDMAIL
  /* In the local version, send mail when xgobi is initiated.  */
  if (stat("/usr/lib/sendmail", &buf) == 0)
  {
    system("/usr/lib/sendmail dfs@bellcore.com \
      <<!\nSubject: Starting XGobi\n.\n!");
  }
/*
  if (stat("/usr/public/bin/sendmail", &buf) == 0)
  {
    system("/usr/public/bin/sendmail - dfs@bellcore.com \
      +elec -check subject='Starting XGobi' file='/tmp/foo'\n");
  }
  else if (stat("/usr/local/bin/sendmail", &buf) == 0)
  * On 214 machines, don't use the "-check" argument *
  {
    system("/usr/local/bin/sendmail - dfs@bellcore.com \ 
      subject='Starting XGobi' file='/tmp/foo'");
  }
  * 214 solaric machines don't seem to have a user sendmail *
  else if (stat("/usr/lib/sendmail", &buf) == 0)
  {
    system("/usr/lib/sendmail dfs@bellcore.com \
      <<!\nSubject: Starting XGobi\n.\n!");
  }
*/
#endif

/*
 * make_xgobi( dataflag,
 *   data_name,
 *   datap, nr, nc,
 *     collab, rowlab,
 *     nlinks, connecting_lines,
 *   &xgobi,
 *   parent, name) : getting rid of the 'name' argument, dfs 5/1/94
*/

  if (make_xgobi(False,
    data_in,
    (float **) NULL, (int) NULL, (int) NULL,
    (char **) NULL, (char **) NULL,
    (int) NULL, (connect_lines *) NULL,
    &xgobi,
    (Widget) NULL) == 0)
    /*(Widget) NULL, name) == 0)*/
  {
    return(0);
  }
  else
    XGobiMainLoop(&xgobi);

  return(1);
}
